<?php
class ProjectSettings
{
	var $_table;

	var $_pageMode;

	var $_pageType;

	var $_page;

	var $_viewPage = PAGE_VIEW;

	var $_defaultViewPage = PAGE_VIEW;

	var $_editPage = PAGE_EDIT;

	var $_defaultEditPage = PAGE_EDIT;

	var $_tableData = array();

	var $_optionsFile = array();

	/**
	 * _auxTableData is used for all page-related settings such as getDefaultPage, getPageOption etc
	 * The only page so far where table != auxTable is Register
	 * table is <users> there and auxTable is GLOBAL_PAGES
	 */
	var $_auxTable = "";
	var $_auxTableData = array();

	var $_pageOptions = array();

	var $_mastersTableData = array();

	var $_detailsTableData = array();

	var $_dashboardElemPSet = array();

	function __construct($table = "", $pageType = "", $page = "", $pageTable = "")
	{
		if( !$table )
			$table = GLOBAL_PAGES;
		if( !$pageTable )
			$pageTable = $table;
		$this->setTable($table);
		if( $table == $pageTable) {
			$this->_auxTableData = & $this->_tableData;
			$this->_auxTable = $this->_table;
		} else {
			$this->loadAuxTable( $pageTable );
		}
		if( $page ) {
			//	determine the page type to avoid unnecessary permission reading
			//	when creating pSetSearch
			$this->_pageType = $this->getOriginalPageType( $page );
		}
		
		
		if($page && array_key_exists($page, $this->getPageIds())) {
			$this->setPage($page);
			$this->setPageType( $this->getPageType() );
		} else if( $pageType ) {
			$this->_pageType = $pageType;
			$page = $this->getDefaultPage( $pageType );
			if( $page )
			{
				$this->setPage( $page );
			}
		}
		if( $page && !$pageType ) {
			$pageType = $this->getPageType();
		}
		if ( $pageType ) {
			$this->setPageType($pageType);
		}
	}

	function table() {
		return $this->_table;
	}

	function loadPageOptions( $option = "" ) {
		if( $this->_pageOptions )
			return;
		importPageOptions( $this->_auxTable, $this->_page );
		global $page_options;
		$this->_pageOptions = &$page_options[$this->_auxTable][$this->_page];


	}

	function setPage($page) {
		$this->_page = $page;
		//	there is no such page!
		if( array_search( $page, $this->getPageIds() ) === FALSE ) {
			return;
		}
		$this->_optionsFile = getabspath("include/pages/". GetTableURL( $this->_auxTable ) . "_" . $page . ".json");
	}

	function setTable($table)
	{
		$this->_table = $table;
		global $tables_data, $field_labels, $fieldToolTips, $placeHolders, $page_titles, $detailsTablesData, $masterTablesData, $tableEvents, $bSubqueriesSupported;
		if(GetTableURL($table) != "") {
			importTableSettings( $table );
		}

		if(isset($tables_data[$this->_table]))
			$this->_tableData = &$tables_data[$this->_table];

		$this->_mastersTableData = &$masterTablesData[$this->_table];
		$this->_detailsTableData = &$detailsTablesData[$this->_table];

		$tableType = $this->getTableType();

		$this->_editPage = $this->getDefaultEditPageType($tableType);
		$this->_viewPage = $this->getDefaultViewPageType($tableType);
		$this->_defaultEditPage = $this->_editPage;
		$this->_defaultViewPage = $this->_viewPage;
	}

	function loadAuxTable($auxTable)
	{
		$this->_auxTable = $auxTable;
		global $tables_data;
		if(GetTableURL($auxTable) != "") {
			importTableSettings( $auxTable );
		}

		if(isset($tables_data[$this->_auxTable]))
			$this->_auxTableData = &$tables_data[$this->_auxTable];
	}


	function pageName() {
		return $this->_page;
	}

	/**
	 * Return table where the page belongs. <global> for Login, Register page desite table==usersTable
	 */
	function pageTable() {
		return $this->_auxTable;
	}

	function getDefaultViewPageType($tableType)
	{
		if($tableType == PAGE_CHART || $tableType == PAGE_REPORT)
			return $tableType;

		return PAGE_VIEW;
	}

	function getDefaultEditPageType($tableType)
	{
		if($tableType == PAGE_CHART || $tableType == PAGE_REPORT)
			return PAGE_SEARCH;

		return PAGE_EDIT;
	}

	function setPageType($page)
	{
		//	a deeper checking for table and page types compatibility might be added here
		if($this->isPageTypeForView($page))
		{
			$tableType = $this->getTableType();
			if($tableType != "report" && $tableType != "chart"
				&& ($page == PAGE_CHART || $page == PAGE_REPORT))
				$this->_viewPage = PAGE_LIST;
			else
				$this->_viewPage = $page;
			$this->_defaultViewPage = $this->getDefaultViewPageType($tableType);
		}
		if($this->isPageTypeForEdit($page))
		{
			$this->_editPage = $page;
			$this->_defaultEditPage = $this->getDefaultEditPageType($this->getTableType());
		}
	}

	function getDefaultPages() {
		$this->updatePages();
		return $this->getAuxTableData(".defaultPages");
	}

	function getDefaultPage( $type, $pageTable = "" ) {
		$this->updatePages();
		if( isAdminPage( $this->_auxTable ) )
			return $this->_pageType;
		$defPages =& $this->getAuxTableData(".defaultPages");
		$defPage = $defPages[$type];
		if( $defPage )
			return $defPage;
		return null;
	}

	function getPageIds() {
		$this->updatePages();
		$ret = $this->getAuxTableData(".pages");
		if( !is_array( $ret ) )
			return array();
		return $ret;
	}

	function getEditPageType()
	{
		return $this->_editPage;
	}

	function isPageTypeForView($ptype)
	{
		global $pageTypesForView;
		return in_array(strtolower($ptype), $pageTypesForView);
	}

	function isPageTypeForEdit($ptype)
	{
		global $pageTypesForEdit;
		return in_array(strtolower($ptype), $pageTypesForEdit);
	}

	function getTable($table, $page = "")
	{
		return new ProjectSettings($table, $page);
	}

	function getPageTypeByFieldEditFormat($field, $editFormat)
	{
		if(isset($this->_tableData[$field]) && isset($this->_tableData[$field][FORMAT_EDIT]))
		{
			foreach($this->_tableData[$field][FORMAT_EDIT] as $pageType => $editSettings)
			{
				if(isset($editSettings["EditFormat"]) && $editSettings["EditFormat"] == $editFormat)
					return $pageType;
			}
		}
		return "";
	}

	function getTableData($key)
	{
		if(!$this->isExistsTableKey($key))
			return $this->getDefaultValueByKey(substr($key,1));
		return $this->_tableData[$key];
	}

	function getAuxTableData($key)
	{
		if(!$this->isExistsAuxTableKey($key))
			return $this->getDefaultValueByKey(substr($key,1));
		return $this->_auxTableData[$key];
	}

	/**
	 * Specify path to arguments in $key1, $key2 etc
	 * Returns FALSE if not found
	 */
	function getPageOption($key1, $key2 = FALSE, $key3 = FALSE, $key4 = FALSE, $key5 = FALSE )
	{
		$this->loadPageOptions( $key1.$key2 );
		if( !isset( $this->_pageOptions[ $key1 ] ) )
			return FALSE;
		$keys = array( $key1 );
		if( $key2!== FALSE )
			$keys[]= $key2;
		if( $key3!== FALSE )
			$keys[]= $key3;
		if( $key4!== FALSE )
			$keys[]= $key4;
		if( $key5!== FALSE )
			$keys[]= $key5;

		$opt = &$this->_pageOptions;
		foreach( $keys as $k ) {
			if( !is_array( $opt ) )
				return FALSE;
			if( !isset( $opt[ $k ] ) )
				return FALSE;
			$opt = &$opt[$k];
		}
		return $opt;
	}

	/**
	 * Specify path to arguments in $key1, $key2 etc
	 * Returns FALSE if not found
	 */
	function getPageOptionArray( $keys )
	{
		$this->loadPageOptions();
		$opt = &$this->_pageOptions;
		foreach( $keys as $k ) {
			if( !is_array( $opt ) )
				return FALSE;
			if( !isset( $opt[ $k ] ) )
				return FALSE;
			$opt = &$opt[$k];
		}
		return $opt;
	}


	private function getEffectiveEditPage( $field )
	{
		if( $this->isSeparate($field) )
		{
			return $this->_editPage;
		}
		return $this->_defaultEditPage;
	}

	private function getEffectiveViewPage( $field )
	{
		if( $this->isSeparate($field) )
		{
			if( $this->_pageMode == EDIT_INLINE && $this->_viewPage != PAGE_VIEW )
				return PAGE_LIST;
			else if ( $this->_pageMode == LIST_MASTER && $this->_viewPage == PAGE_LIST )
				return PAGE_MASTER_INFO_LIST;
			else if ( $this->_pageMode == LIST_MASTER && $this->_viewPage == PAGE_REPORT )
				return PAGE_MASTER_INFO_REPORT;
			else if ( $this->_pageMode == PRINT_MASTER && $this->_viewPage == PAGE_RPRINT )
				return PAGE_MASTER_INFO_RPRINT;
			else if ( $this->_pageMode == PRINT_MASTER )
				return PAGE_MASTER_INFO_PRINT;

			return $this->_viewPage;
		}
		return $this->_defaultViewPage;
	}
	function getFieldData( $field, $key )
	{
		global $g_settingsType;
		if( $this->getEntityType() == titDASHBOARD )
		{
			return $this->getDashFieldData( $field, $key );
		}

		if(!isset($this->_tableData[$field]))
			return $this->getDefaultValueByKey($key);

		$settingType = $g_settingsType[$key];
		if($settingType == null)
			$settingType = "";

		switch($settingType)
		{
			case SETTING_TYPE_VIEW:
				$viewPage = $this->getEffectiveViewPage( $field );

				if(isset($this->_tableData[$field][FORMAT_VIEW][$viewPage][$key]))
					return $this->_tableData[$field][FORMAT_VIEW][$viewPage][$key];
				break;
			case SETTING_TYPE_EDIT:
				$editPage = $this->getEffectiveEditPage( $field );

				if(isset($this->_tableData[$field][FORMAT_EDIT][$editPage][$key]))
					return $this->_tableData[$field][FORMAT_EDIT][$editPage][$key];
				break;
			default:
				if (isset($this->_tableData[$field][$key]))
					return $this->_tableData[$field][$key];
				break;
		}
		return $this->getDefaultValueByKey($key);
	}

	function setFieldData( $field, $key, $value )
	{
		$oldValue = $this->getFieldData( $field, $key );
		global $g_settingsType;

		$settingType = $g_settingsType[$key];
		if($settingType == null)
			$settingType = "";

		switch($settingType)
		{
			case SETTING_TYPE_VIEW:
				$viewPage = $this->getEffectiveViewPage( $field );
				$this->_tableData[$field][FORMAT_VIEW][$viewPage][$key] = $value;
				break;
			case SETTING_TYPE_EDIT:
				$editPage = $this->getEffectiveEditPage( $field );
				$this->_tableData[$field][FORMAT_EDIT][$editPage][$key] = $value;
				break;
			default:
				$this->_tableData[$field][$key] = $value;
				break;
		}
		return $oldValue;
	}

	/**
	 * getTableName
	 * Returns table name the class is created for
	 */
	function getTableName()
	{
		return $this->_table;
	}

	/**
	 * findField
	 * Returns field name in correct case if the field is found. Empty string otherwise.
	 * @param {string} field name in arbitrary case, original or GoodFieldName result
	 */

	function findField( $f )
	{
		global $field_labels, $mlang_defaultlang;
		//	check exact match
		$fields = $this->getFieldsList();
		if( array_search( $f, $fields ) !== FALSE )
			return $f;

		//	check goodfieldname
		$gTable = GoodFieldName( $this->_table );
		if( isset( $field_labels[ $mlang_defaultlang ][ $f ] ) )
			return $this->getFieldByGoodFieldName( $f );

		//	case-insensitive check
		$f = strtoupper( $f );
		foreach( $fields as $ff )
		{
			if( strtoupper( $ff ) == $f )
				return $ff;

			if( strtoupper( GoodFieldName($ff) ) == $f )
				return $ff;
		}
		return "";

	}

	/**
	 * addCustomExpressionIndex
	 * Add new index to list, for determination of custom expressions position in SQL query
	 * @param {string} table wich contain a lookup field
	 * @param {string} name of lookup field
	 * @param {int} index
	 */
	function addCustomExpressionIndex($mainTable, $mainField, $index)
	{
		if(!$this->isExistsTableKey(".customExpressionIndexes"))
			$this->_tableData[".customExpressionIndexes"] = array();
		if(!isset($this->_tableData[".customExpressionIndexes"][$mainTable]))
			$this->_tableData[".customExpressionIndexes"][$mainTable] = array();
		$this->_tableData[".customExpressionIndexes"][$mainTable][$mainField] = $index;
	}

	/**
	 * getCustomExpressionIndex
	 * Get index of custom expression in SQL field
	 * @param {string} table wich contain a lookup field
	 * @param {string} name of lookup field
	 */
	function getCustomExpressionIndex($mainTable, $mainField)
	{
		if(!$this->isExistsTableKey(".customExpressionIndexes"))
			$this->_tableData[".customExpressionIndexes"] = array();
		if(isset($this->_tableData[".customExpressionIndexes"][$mainTable])
			&& isset($this->_tableData[".customExpressionIndexes"][$mainTable][$mainField]))
			return $this->_tableData[".customExpressionIndexes"][$mainTable][$mainField];

		return FALSE;
	}

	function isExistsTableKey($key)
	{
		if(!isset($this->_tableData[$key]))
			return false;
		return true;
	}

	function isExistsAuxTableKey($key)
	{
		if(!isset($this->_auxTableData[$key]))
			return false;
		return true;
	}

	function isExistsFieldKey($field, $key)
	{
		if(!$this->isExistsTableKey($field))
			return false;
		if(!isset($this->_tableData[$field][$key]))
			return false;
		return true;
	}

	function getDefaultValueByKey($key)
	{
		global $g_defaultOptionValues;
		if(isset($g_defaultOptionValues[$key]))
			return $g_defaultOptionValues[$key];
		return false;
	}

	/**
	 * Returns array of master tables , which are master for current detail table
	 * @return array if success otherwise false
	 */
	function getMasterTablesArr()
	{
		return $this->_mastersTableData;
	}

	/**
	 * Returns array of master keys for passed detailTable
	 *
	 * @param string $dTableName - it's detail data sourse table name,
	 * @return array if success otherwise false
	 */
	function getMasterKeysByDetailTable($dTableName, $default = array())
	{
		if(!$dTableName)
			return $default;
		foreach ($this->_detailsTableData as $dTableDataArr)
		{
			if ($dTableDataArr['dDataSourceTable'] == $dTableName)
				return $dTableDataArr['masterKeys'];
		}
		return $default;
	}

	/**
	 * Returns array of detail tables , which are detail for current master table
	 * @param string $tName - it's data source master table name
	 * @return array if success otherwise false
	 */
	function getDetailTablesArr()
	{
		return $this->_detailsTableData;
	}

	/**
	 * Returns array of detail keys for passed masterTable
	 *
	 * @param string $mTableName - it's master table name,
	 * @param string $tName - it's current (detail) table
	 * @return array if success otherwise default value
	 */
	function getDetailKeysByMasterTable($mTableName = "", $default = array())
	{
		if(!$mTableName)
			return $default;
		foreach($this->_mastersTableData as $mTableDataArr)
		{
			if ($mTableDataArr['mDataSourceTable'] == $mTableName)
				return $mTableDataArr['detailKeys'];
		}
		return $default;
	}

	/**
	 * Returns array of detail keys for passed detailTable
	 *
	 * @param string $dTableName - It's detail data sourse table name
	 * @param string $tName - current(master) table name
	 * @return array if success otherwise false
	 */
	function getDetailKeysByDetailTable($dTableName, $default = array())
	{
		foreach ($this->_detailsTableData as $dTableDataArr)
		{
			if ($dTableDataArr['dDataSourceTable'] == $dTableName)
				return $dTableDataArr['detailKeys'];
		}
		return $default;
	}

	/**
	 * Returns details preview Type
	 *
	 * @param string $dTableName - it's detail data sourse table name,
	 * @param string $tName - current(master) table name
	 * @return array if success otherwise false
	 */
	function getDPType($dTableName)
	{
		if(!$dTableName)
			return false;
		foreach ($this->_detailsTableData as $dTableDataArr)
		{
			if ($dTableDataArr['dDataSourceTable'] == $dTableName)
				return $dTableDataArr['previewOnList'];
		}
		return false;
	}

	function GetFieldByIndex($index)
	{
		foreach($this->_tableData as $key => $value)
		{
			if(!is_array($value))
				continue;
			elseif(!isset($value["Index"]))
				continue;
			if($value["Index"] == $index && $this->getFieldIndex($key))
				return $key;
		}
		return null;
	}

	//	Is field has separate type for editing and viewing
	function isSeparate($field)
	{
		return !!$this->_tableData[$field]["isSeparate"];
	}

	// return field label
	function label($field)
	{
		$result = GetFieldLabel( GoodFieldName($this->_table), GoodFieldName($field) );
		return $result != "" ? $result : $field;
	}

	//	return filename field if any
	//	viewFormat setting
	function getFilenameField($field)
	{
		return $this->getFieldData($field, "Filename");
	}

	//	return hyperlink prefix
	//	viewFormat setting
	function getLinkPrefix($field)
	{
		return $this->getFieldData($field, "LinkPrefix");
	}

	/**
	 * Get hyperlink type
	 */
	function getLinkType($field)
	{
		return $this->getFieldData($field, "hlType");
	}

	/**
	 * Get hyperlink display field for title
	 * @return string field name
	 */
	function getLinkDisplayField($field)
	{
		return $this->getFieldData($field, "hlTitleField");
	}

	function openLinkInNewWindow($field)
	{
		return $this->getFieldData($field, "hlNewWindow");
	}

	function getLinkWordNameType($field)
	{
		return $this->getFieldData($field, "hlLinkWordNameType");
	}

	function getLinkWordText($field)
	{
		return $this->getFieldData($field, "hlLinkWordText");
	}

	//	return database field type
	//	using ADO DataTypeEnum constants
	//	the full list available at:
	//	http://msdn.microsoft.com/library/default.asp?url=/library/en-us/ado270/htm/mdcstdatatypeenum.asp
	function getFieldType($field)
	{
		return $this->getFieldData($field, "FieldType");
	}

	function isAutoincField($field)
	{
		return $this->getFieldData($field, "AutoInc");
	}

	function getOraSequenceName($field)
	{
		return $this->getFieldData($field, "OraSequenceName");
	}

	function getDefaultValue($field, $table = "")
	{
		$tableName = $table ? $table : $this->_table;
		$editPage = $this->_editPage;
		if(!$this->isSeparate($field))
			return;
		return GetDefaultValue($field, $editPage, $tableName);
	}

	function isAutoUpdatable($field)
	{
		return $this->getFieldData($field, "autoUpdatable");
	}

	function getAutoUpdateValue($field)
	{
		$editPage = $this->_editPage;
		if(!$this->isSeparate($field))
			$editPage = $this->getDefaultEditPageType($this->getTableType());;
		return GetAutoUpdateValue($field, $editPage, $this->_table);
	}

	//	return Edit format
	//	editFormats
	function getEditFormat($field)
	{
		return $this->getFieldData($field, "EditFormat");
	}

	//	return View format
	//	viewFormat setting
	function getViewFormat($field)
	{
		return $this->getFieldData($field, "ViewFormat");
	}

	//	show time in datepicker or not
	function dateEditShowTime($field)
	{
		return $this->getFieldData($field, "ShowTime");
	}

	function lookupControlType($field)
	{
		return $this->getFieldData($field, "LCType");
	}

	function lookupListPageId($field)
	{
		return $this->getFieldData($field, "listPageId");
	}
	function lookupAddPageId($field)
	{
		return $this->getFieldData($field, "addPageId");
	}

	function isDeleteAssociatedFile($field)
	{
		return $this->getFieldData($field, "DeleteAssociatedFile");
	}

	//	is Lookup wizard dependent or not
	function useCategory($field)
	{
		return $this->getFieldData($field, "UseCategory");
	}

	//	is Lookup wizard with multiple selection
	function multiSelect($field)
	{
		return $this->getFieldData($field, "Multiselect");
	}

	/**
	 * When a field is edited as single-select lookup wizard
	 * Use case:
	 * if( $pSet->multiSelect() && $pSet->singleSelectLookupEdit() )
	 * multiselect on search, single-select on add/edit. 
	 * In that specific case multiple selected search values 
	 * should be interpreted as:
	 * field=value1 or field=value2 or etc
	 * 
	 */
	function singleSelectLookupEdit( $field )
	{
		$hasLookup = false;
		foreach( $this->_tableData[$field][FORMAT_EDIT] as $pageType => $editFormat ) {
			if( $pageType != "edit" && $pageType != "add" )
				continue;
			if( $editFormat["EditFormat"] != EDIT_FORMAT_LOOKUP_WIZARD )
				continue;
			$hasLookup = true;
			if( $editFormat["Multiselect"] )
				return false;
		}
		return $hasLookup;
	}

	/**
	 * Check whether the field is a true multiselect lookup.
	 * True multiselect has this option on Add or Edit page, not only on Search.
	*/
	function multiSelectLookupEdit( $field )
	{
		$hasLookup = false;
		foreach( $this->_tableData[$field][FORMAT_EDIT] as $pageType => $editFormat ) {
			if( $pageType != "edit" && $pageType != "add" )
				continue;
			if( $editFormat["EditFormat"] != EDIT_FORMAT_LOOKUP_WIZARD )
				continue;
			if( $editFormat["Multiselect"] )
				return true;
		}
		return false;
	}

	// Lookup wizard select size
	function selectSize($field)
	{
		return $this->getFieldData($field, "SelectSize");
	}

	function showThumbnail($field)
	{
		return $this->getFieldData($field, "ShowThumbnail");
	}

	function isImageURL($field)
	{
		return $this->getFieldData($field, "fieldIsImageUrl");
	}

	function showCustomExpr($field)
	{
		return $this->getFieldData($field, "ShowCustomExpr");
	}

	function showFileSize($field)
	{
		return $this->getFieldData($field, "ShowFileSize");
	}

	function showIcon($field)
	{
		return $this->getFieldData($field, "ShowIcon");
	}

	function getImageWidth($field)
	{
		return $this->getFieldData($field, "ImageWidth");
	}

	function getImageHeight($field)
	{
		return $this->getFieldData($field, "ImageHeight");
	}

	function getThumbnailWidth($field)
	{
		return $this->getFieldData($field, "ThumbWidth");
	}

	function getThumbnailHeight($field)
	{
		return $this->getFieldData($field, "ThumbHeight");
	}

	// Get nLookupType for current field
	function getLookupType($field)
	{
		return $this->getFieldData($field, "LookupType");
	}

	//Get lookup table name
	function getLookupTable($field)
	{
		return $this->getFieldData($field, "LookupTable");
	}

	/**
	 *	Returns true if Lookup Where clause is specified as PHP code expression
	 *	@return Boolean
	 */
	function isLookupWhereCode($field)
	{
		return $this->getFieldData($field, "LookupWhereCode");
	}

	function isLookupWhereSet($field)
	{
		if( $this->isLookupWhereCode( $field ) )
			return true;
		return 0 != strlen( $this->getFieldData($field, "LookupWhere") );
	}


	function getLookupWhere($field)
	{
		if( $this->isLookupWhereCode( $field )) {
			$tName = $this->_table;
			if( $this->getEntityType() == titDASHBOARD )
			{
				$dashSearchFields = $this->getDashboardSearchFields();
				if( isset($dashSearchFields[$field]) ) {
					$tName = $dashSearchFields[$field][0]["table"];
					$field = $dashSearchFields[$field][0]["field"];
				}
			}

			return GetLWWhere( $field, $this->getEffectiveEditPage( $field ), $tName );
		}

		return $this->getFieldData($field, "LookupWhere");
	}



	function getNotProjectLookupTableConnId($field)
	{
		return $this->getFieldData($field, "LookupConnId");
	}

	function getConnId()
	{
		$connId = $this->getTableData( ".connId" );
		if( $connId == "" ) {
			return "Tables";
		}
		return $connId;
	}

	function getLinkField($field)
	{
		return $this->getFieldData($field, "LinkField");
	}

	function getLWLinkFieldType($field)
	{
		return $this->getFieldData($field, "LinkFieldType");
	}

	function getDisplayField($field)
	{
		return $this->getFieldData($field, "DisplayField");
	}

	function getCustomDisplay($field)
	{
		return $this->getFieldData($field, 'CustomDisplay');
	}

	//	viewFormats
	function NeedEncode($field)
	{
		return $this->getFieldData($field, "NeedEncode");
	}

	/**
	 * Get array of validation for control
	 * return array - of validations
	 */
	function getValidation($field)
	{
		return $this->getFieldData($field, "validateAs");
	}


	/**
	 * Returns array of items for a given field
	 */
	function getFieldItems( $field )
	{
		return $this->getPageOption( "fields", "fieldItems", $field );
	}

	/**
	 * Returns array of group fields
	 */
	function getGroupFields()
	{
		return $this->getPageOption( "dataGrid", "groupFields");
	}

	/**
	 * Check is appear current field on list page
	 * return boolean - true or false
	 */
	function appearOnListPage($field)
	{
		if( array_search( $field, $this->getPageOption("fields", "gridFields") ) !== FALSE )
			return true;
		if( isReport( $this->getEntityType() ) ) {
			return array_search( $field, $this->getReportGroupFields() ) !== FALSE;
		}
		return false;
	}

	/**
	 * Check is appear current field on add page
	 * return boolean - true or false
	 */
	function appearOnAddPage($field)
	{
		return array_search( $field, $this->getPageOption("fields", "gridFields") ) !== FALSE;
	}

	/**
	 * Check is appear current field on inline add
	 * return boolean
	 */
	function appearOnInlineAdd($field)
	{
		return array_search( $field, $this->getInlineAddFields() ) !== FALSE;
		//return $this->getFieldData($field, "bInlineAdd");
	}

	/**
	 * Check is appear current field on edit page
	 * return boolean - true or false
	 */
	function appearOnEditPage($field)
	{
		return array_search( $field, $this->getPageOption("fields", "gridFields") ) !== FALSE;
	}

	/**
	 * Check is appear current field on edit page
	 * return boolean - true or false
	 */
	function appearOnInlineEdit($field)
	{
		return array_search( $field, $this->getInlineEditFields() ) !== FALSE;
		//return $this->getFieldData($field, "bInlineEdit");
	}

	/**
	 * Check is appear current field on edit page
	 * return boolean - true or false
	 */
	function appearOnUpdateSelected($field)
	{
		$updateOnEditFields = $this->getPageOption("fields", "updateOnEditFields");
		if( !$updateOnEditFields )
			return false;

		return array_search( $field, $this->getPageOption("fields", "updateOnEditFields") ) !== FALSE;
		//return $this->getFieldData($field, "bUpdateSelected");
	}

	function appearOnPage($field)
	{
		$gridFields = &$this->getPageOption("fields", "gridFields");
		if( !$gridFields )
			$ret = false;
		else
			$ret = ( array_search( $field, $gridFields ) !== FALSE );
		if( !$ret ) {
			if( $this->getPageType() === 'report' || $this->getPageType() === 'rprint' )
				return array_search( $field, $this->getReportGroupFields() ) !== false;
		}
		return $ret;
	}

	function appearOnSearchPanel($field)
	{
		$fields = &$this->getPageOption("fields", "searchPanelFields");
		if( !$fields )
			return false;

		return array_search( $field, $fields ) !== FALSE;
	}


	function appearAlwaysOnSearchPanel( $field ) {
		$fields  = &$this->getPageOption("listSearch", "alwaysOnPanelFields");
		if( !$fields )
			return false;

		return array_search( $field, $fields ) !== FALSE;

	}

	function getPageFields()
	{
		$fields = $this->getPageOption("fields", "gridFields" );
		if( isReport( $this->getEntityType() ) ) {
			return array_merge( $fields, $this->getReportGroupFields() );
		}
		return $fields;
	}

	/**
	 * Check is appear current field on view page
	 * return boolean - true or false
	 */
	function appearOnViewPage($field)
	{
		return array_search( $field, $this->getPageOption("fields", "gridFields") ) !== FALSE;
	}

	/**
	 * Check is appear current field on print page
	 * return boolean - true or false
	 */
	function appearOnPrinterPage($field)
	{
		return $this->appearOnListPage($field);
	}

	function isVideoUrlField($field)
	{
		return $this->getFieldData($field, "fieldIsVideoUrl");
	}

	function isAbsolute($field)
	{
		return $this->getFieldData($field, "Absolute");
	}

	function getAudioTitleField($field)
	{
		return $this->getFieldData($field, "audioTitleField");
	}

	function getVideoWidth($field)
	{
		return $this->getFieldData($field, "videoWidth");
	}

	function getVideoHeight($field)
	{
		return $this->getFieldData($field, "videoHeight");
	}

	function isRewindEnabled($field)
	{
		return $this->getFieldData($field, "RewindEnabled");
	}

	/**
	 * @param String field
	 * @return Array
	 */
	function getParentFieldsData( $field )
	{
		return $this->getFieldData($field, "categoryFields");
	}

	/**
	 * @param String field
	 * @return Array
	 */
	function getLookupParentFNames( $field )
	{
		$fNames = array();
		foreach( $this->getParentFieldsData( $field ) as $data )
		{
			$fNames[] = $data["main"];
		}
		return $fNames;
	}

	function isLookupUnique($field)
	{
		return $this->getFieldData($field, "LookupUnique");
	}

	function getLookupOrderBy($field)
	{
		return $this->getFieldData($field, "LookupOrderBy");
	}

	function isLookupDesc($field)
	{
		return $this->getFieldData($field, "LookupDesc");
	}

	function getOwnerTable($field)
	{
		return $this->getFieldData($field, "ownerTable");
	}

	function isFieldEncrypted($field)
	{
		return $this->getFieldData($field, "bIsEncrypted");
	}

	function isAllowToAdd($field)
	{
		return $this->getFieldData($field, "AllowToAdd");
	}

	function isSimpleAdd($field)
	{
		return $this->getFieldData($field, "SimpleAdd");
	}

	/**
	 * Get the array containing the autocomplete fields data
	 * basing on page's type and lookup wizard settings
	 * @param String field
	 * @return Array
	 */
	function getAutoCompleteFields($field)
	{
		$editPageType = $this->getEditPageType();

		if( $editPageType == PAGE_REGISTER || $editPageType == PAGE_ADD || $editPageType == PAGE_EDIT && ( $this->isSeparate($field) || $this->isAutoCompleteFieldsOnEdit($field) ) )
		{
			return $this->getFieldData($field, "autoCompleteFields");
		}

		return $this->getDefaultValueByKey("autoCompleteFields");
	}

	function isAutoCompleteFieldsOnEdit($field)
	{
		return $this->getFieldData($field, "autoCompleteFieldsOnEdit");
	}

	function isFreeInput($field)
	{
		return $this->getFieldData($field, "freeInput");
	}

	function getMapData($field)
	{
		return $this->getFieldData($field, "mapData");
	}

	function getFormatTimeAttrs($field)
	{
		return $this->getFieldData($field, "FormatTimeAttrs");
	}

	function getViewAsTimeFormatData( $field ) {
		return $this->getFieldData( $field, "timeFormatData" );
	}
	
	function showDaysInTimeTotals( $field ) {
		$formatData = $this->getViewAsTimeFormatData( $field );
		return $formatData ? $formatData["showDaysInTotals"] : false;
	}	
	
	/**
	 * Check is appear current field on export page
	 * return boolean - true or false
	 */
	function appearOnExportPage($field)
	{
		return $this->appearOnpage($field);
	}

	/**
	 * Return original table name for report or chart
	 */
	function getStrOriginalTableName()
	{
		return $this->getTableData(".strOriginalTableName");
	}

	function getSearchableFields()
	{
		if( $this->getEntityType() == titDASHBOARD ) {
			return $this->getTableData('.allSearchFields');
		}
		return $this->getTableData(".searchableFields");
	}

	function getAllSearchFields()
	{
		return $this->getEntityType() == titDASHBOARD ? $this->getTableData('.allSearchFields') : $this->getPageOption("fields", "searchPanelFields");
	}

	function getAdvSearchFields()
	{
		return $this->getEntityType() == titDASHBOARD ? $this->getAllSearchFields() : $this->getPageOption("fields", "gridFields");;
	}

	function isUseTimeForSearch()
	{
		return $this->getTableData(".isUseTimeForSearch");
	}

	function isUseToolTips()
	{
		return $this->getTableData(".isUseToolTips");
	}

	function isUseVideo()
	{
		return $this->getTableData(".isUseVideo");
	}

	function isUseAudio()
	{
		return $this->getTableData(".isUseAudio");
	}

	function isUseAudioOnDetails()
	{
		for($i = 0; $i < count($this->_detailsTableData); $i++)
		{
			if($this->_detailsTableData[$i]["isUseAudio"])
				return true;
		}
		return false;
	}

	function getTableType()
	{
		return $this->getTableData(".tableType");
	}

	function getShortTableName()
	{
		return GetTableURL( $this->_table );
	}

	function isShowAddInPopup()
	{
		return $this->getPageOption("list", "addInPopup");
	}

	function isShowEditInPopup()
	{
		return $this->getPageOption("list", "editInPopup");
	}

	function isShowViewInPopup()
	{
		return $this->getPageOption("list", "viewInPopup");
	}

	function isResizeColumns()
	{
		return $this->getTableData(".isResizeColumns");
	}

	function isUseAjaxSuggest()
	{
		return $this->getTableData(".isUseAjaxSuggest");
	}

/*
	function getDetailsLinksOnList()
	{
		return $this->getTableData(".detailsLinksOnList");
	}
*/

	function getAllPageFields()
	{
		return array_merge( $this->getPageFields(), $this->getAllSearchFields() );
	}

	function getPanelSearchFields()
	{
		return $this->getPageOption("listSearch", "alwaysOnPanelFields");
	}

	function getSearchPanelOptions()
	{
		return $this->GetTableData(".searchPanelOptions");
	}

	function getGoogleLikeFields()
	{
		return $this->getTableData(".googleLikeFields");
	}

	function getInlineEditFields()
	{
		return $this->getPageOption("fields", "inlineEditFields");
	}

	function getUpdateSelectedFields()
	{
		return $this->getPageOption("fields", "updateOnEditFields");
		//return $this->getTableData(".updateSelectedFields");
	}

	function getExportFields()
	{
		return $this->getPageOption("fields", "exportFields");
	}

	function getImportFields()
	{
		return $this->getPageOption("fields", "gridFields");
	}

	function getEditFields()
	{
		return $this->getPageOption("fields", "gridFields" );
	}

	function getInlineAddFields()
	{
		return $this->getPageOption("fields", "inlineAddFields");
	}

	function getAddFields()
	{
		return $this->getPageOption("fields", "gridFields" );
	}

	function getMasterListFields()
	{
		return $this->getPageOption("fields", "gridFields" );
	}

	function getViewFields()
	{
		return $this->getPageOption("fields", "gridFields" );
	}

	function getPrinterFields()
	{
		return $this->getPageFields();
	}

	function getListFields()
	{
		$fields = $this->getPageOption("fields", "gridFields" );
		if( isReport( $this->getEntityType() ) ) {
			return array_merge( $fields, $this->getReportGroupFields() );
		}
		return $fields;
	}

	function isAddPageEvents()
	{
		return $this->getAuxTableData(".addPageEvents");
	}

	function hasAjaxSnippet()
	{
		return $this->getTableData(".ajaxCodeSnippetAdded");
	}

	function hasButtonsAdded()
	{
		return $this->getPageOption("page", "hasCustomButtons");
	}

	function customButtons()
	{
		return $this->getPageOption("page", "customButtons");
	}

	function isUseFieldsMaps()
	{
		return $this->getTableData(".isUseFieldsMaps");
	}

	function getOrderIndexes()
	{
		return $this->getTableData(".orderindexes");
	}

	function getStrOrderBy()
	{
		return $this->getTableData(".strOrderBy");
	}

	function getSQLQuery()
	{
		$query = $this->getTableData(".sqlquery");
		if($query != null)
		{
			return $query;
		}
		return null;
	}
	/* Dashboard-safe version of getSQLQuery
	*/
	function getSQLQueryByField( $field )
	{
		if( $this->getTableType() == PAGE_DASHBOARD )
		{
			$query = $this->getDashTableData($field, ".sqlquery");
			if($query != null)
			{
				return $query;
			}
			return null;

		}
		else
		{
			$query = $this->getTableData(".sqlquery");
			if($query != null)
			{
				return $query;
			}
			return null;
		}

	}

	/**
	 * Create Thumbnail or not
	 */
	function getCreateThumbnail($field)
	{
		return $this->getFieldData($field, "CreateThumbnail");
	}

	/**
	 * Return Thumbnail prefix
	 */
	function getStrThumbnail($field)
	{
		return $this->getFieldData($field, "StrThumbnail");
	}

	/**
	 * Return Thumbnail prefix
	 */
	function getThumbnailSize($field)
	{
		return $this->getFieldData($field, "ThumbnailSize");
	}

	/**
	 * Resize on upload
	 */
	function getResizeOnUpload($field)
	{
		return $this->getFieldData($field, "ResizeImage");
	}

	/**
	 * True if FileField must work in old single-file mode
	 */
	function isBasicUploadUsed($field)
	{
		return $this->getFieldData($field, "CompatibilityMode");
	}

	/**
	 * True if file in FileField must be uploaded immediately after choosing or dropping
	 */
	function isAutoUpload($field)
	{
		return $this->getFieldData($field, "autoUpload");
	}

	/**
	 * Get size to reduce image after upload
	 */
	function getNewImageSize($field)
	{
		return $this->getFieldData($field, "NewSize");
	}

	function getAcceptFileTypes($field)
	{
		return $this->getFieldData($field, "acceptFileTypes");
	}

	function getAcceptFileTypesHtml($field)
	{
		return $this->getFieldData($field, "acceptFileTypesHtml");
	}

	/**
	 * Get maximum allowed size for uploaded files
	 */
	function getMaxFileSize($field)
	{
		return $this->getFieldData($field, "maxFileSize");
	}

	/**
	 * Get maximum allowed size for all uploaded files per field
	 */
	function getMaxTotalFilesSize($field)
	{
		return $this->getFieldData($field, "maxTotalFilesSize");
	}

	/**
	 * Get maximum allowed number of uploaded files
	 */
	function getMaxNumberOfFiles($field)
	{
		return $this->getFieldData($field, "maxNumberOfFiles");
	}

	/**
	 * Get maximum allowed width of uploaded image
	 */
	function getMaxImageWidth($field)
	{
		return $this->getFieldData($field, "maxImageWidth");
	}

	/**
	 * Get maximum allowed heiht of uploaded image
	 */
	function getMaxImageHeight($field)
	{
		return $this->getFieldData($field, "maxImageHeight");
	}

	/**
	 * Get size to reduce image after upload
	 */
	function getStrFilename($field)
	{
		return $this->getFieldData($field, "strFilename");
	}

	/**
	 * Return height of text area
	 * EditFormat setting
	 */
	function getNRows($field)
	{
		return $this->getFieldData($field, "nRows");
	}

	/**
	 * Return width of text area
	 * EditFormat setting
	 */
	function getNCols($field)
	{
		return $this->getFieldData($field, "nCols");
	}

	/**
	 * Return original table name
	 */
	function getOriginalTableName()
	{
		$result = $this->getTableData(".OriginalTable");
		return $result != "" ? $result : $this->_table;
	}

	/**
	 * Return list of key fields
	 */
	function getTableKeys()
	{
		return $this->getTableData(".Keys");
	}

	function isLargeTextTruncationSet($field)
	{
		return $this->getFieldData($field, "truncateText");
	}

	/**
	 * Return number of chars to show before "More..." link
	 */
	function getNumberOfChars($field)
	{
		return $this->getFieldData($field, "NumberOfChars");
	}

	/**
	 * Check if the field is an sql expression
	 */
	function isSQLExpression($field)
	{
		return $this->getFieldData($field, "isSQLExpression");
	}

	/**
	 * Get full field name
	 */
	function getFullFieldName($field)
	{
		return $this->getFieldData($field, "FullName");
	}

	/**
	 * set full field name
	 */
	function setFullFieldName($field, $value)
	{
		return $this->setFieldData($field, "FullName", $value);
	}


	/**
	 * Return table Owner ID field
	 */
	function getTableOwnerID()
	{
		return $this->getTableData(".OwnerID");
	}

	/**
	 * Is field marked as required
	 * EditFormat setting
	 */
	function isRequired($field)
	{
		return $this->getFieldData($field, "IsRequired");
	}

	function insertNull($field)
	{
		return $this->getFieldData($field, "insertNull");
	}

	/**
	 * IS use Rich Text Editor or not
	 */
	function isUseRTE($field)
	{
		return $this->getFieldData($field, "UseRTE");
	}

	/**
	 * Is use Rich Text Editor BASIC or not
	 */
	function isUseRTEBasic($field)
	{
		global $isUseRTEBasic;
		return $this->isUseRTE($field) && $isUseRTEBasic;
	}

	/**
	 * Is use Rich Text Editor FCK or not
	 */
	function isUseRTEFCK($field)
	{
		global $isUseRTECK;
		return $this->isUseRTE($field) && $isUseRTECK;
	}

	/**
	 * check if RTE CKEditor version 5 is used in project
	 */
	function isUseRTECKNew($field)
	{
		global $isUseRTECKNew;
		return $this->isUseRTE($field) && $isUseRTECKNew;
	}	
	
	/**
	 * Is use Rich Text Editor INNOVA or not
	 */
	function isUseRTEInnova($field)
	{
		global $isUseRTEInnova;
		return $this->isUseRTE($field) && $isUseRTEInnova;
	}

	/**
	 * Add timestamp to filename when files uploading or not
	 * EditFormat setting
	 */
	function isUseTimestamp($field)
	{
		return $this->getFieldData($field, "UseTimestamp");
	}

	function getFieldIndex($field)
	{
		return $this->getFieldData($field, "Index");
	}

	function getEntityType()
	{
		return $this->getTableData(".entityType");
	}


	/**
	 * Return Date field edit type
	 */
	function getDateEditType($field)
	{
		return $this->getFieldData($field, "DateEditType");
	}

	function getHTML5InputType($field)
	{
		return $this->getFieldData($field, "HTML5InuptType");
	}

	/**
	 * Return text edit parameters
	 */
	function getEditParams($field)
	{
		return $this->getFieldData($field, "EditParams");
	}


	function getControlWidth($field)
	{
		return $this->getFieldData($field, "controlWidth");
	}

	/**
	 * Check whether field is viewable
	 */
	function checkFieldPermissions($field)
	{
		return $this->appearOnPage( $field );
	}

	function getTableOwnerIdField()
	{
		return $this->getTableData(".mainTableOwnerID");
	}

	function isHorizontalLookup($field)
	{
		return $this->getFieldData($field, "HorizontalLookup");
	}

	function isDecimalDigits($field)
	{
		return $this->getFieldData($field, "DecimalDigits");
	}

	function getLookupValues($field)
	{
		return $this->getFieldData($field, "LookupValues");
	}

	function hasEditPage()
	{
		return !!$this->getDefaultPage( "edit" );
	}

	function hasAddPage()
	{
		return !!$this->getDefaultPage( "add" );
	}

	function hasListPage()
	{
		return !!$this->getDefaultPage( "list" );
	}

	function hasImportPage()
	{
		return !!$this->getDefaultPage( "import" );
	}

	function hasInlineEdit()
	{
		return $this->getPageOption("list", "inlineEdit");
	}

	function hasUpdateSelected()
	{
		return $this->getPageOption("list", "updateSelected");
	}

	function updateSelectedButtons()
	{
		//return $this->getPageOption("edit", "updateSelectedButtons");
		$data = $this->labeledButtons();
		return $data['update_records'];
	}

	function activatonMessages()
	{
		$data = $this->labeledButtons();
		if( !is_array( $data['register_activate_message'] ) )
			return array();
		return $data['register_activate_message'];
	}

	function labeledButtons()
	{
		return $this->getPageOption("page", "labeledButtons");
	}

	function printPagesLabelsData()
	{
		$data = $this->labeledButtons();
		return $data['print_pages'];
	}

	function detailsFoundLabelsData()
	{
		$data = $this->labeledButtons();
		return $data['details_found'];
	}


	function hasSortByDropdown()
	{
		return $this->getPageOption("list", "sortDropdown");
	}

	function getSortControlSettingsJSONString()
	{
		return $this->getTableData(".strSortControlSettingsJSON");
	}
	function getClickActionJSONString()
	{
		return $this->getTableData(".strClickActionJSON");
	}

	function hasCopyPage()
	{
		return true;
		//return $this->getPageOption("pageLinks", "copy");
	}
	function hasViewPage()
	{
		return !!$this->getDefaultPage("view");
	}
	function hasExportPage()
	{
		return !!$this->getDefaultPage("export");
	}
	function hasPrintPage()
	{
		return !!$this->getDefaultPage("print") || !!$this->getDefaultPage("rprint");
	}
	function hasDelete()
	{
		return $this->getPageOption("list", "delete");
	}
	function getTotalsFields()
	{
		$ret = array();
		foreach( $this->getPageOption('totals') as $field => $totals ) {
			if( $totals /* ?? */ && $totals["totalsType"] ) {
				$ret[] = array(
					"fName" => $field,
					"numRows" => 0,
					"totalsType" => $totals["totalsType"],
					"viewFormat" => $this->getViewFormat( $field )

				);
			}
		}
		return $ret;
	}

	function getExportTxtFormattingType()
	{
		return $this->getPageOption("export","format");
	}

	function getExportDelimiter()
	{
		return $this->getPageOption("export","delimiter");
	}

	function chekcExportDelimiterSelection()
	{
		return $this->getPageOption("export","selectDelimiter");
	}

	function checkExportFieldsSelection()
	{
		return $this->getPageOption("export","selectFields");
	}

	function exportFileTypes()
	{
		return $this->getPageOption("export", "exportFileTypes");
	}

	function getLoginFormType()
	{
		return $this->getPageOption("loginForm", "loginForm");
	}

	function getAdvancedSecurityType()
	{
		if( !GetGlobalData("createLoginPage",null) || (GetGlobalData("nLoginMethod", null) != SECURITY_TABLE && GetGlobalData("nLoginMethod", null) != SECURITY_AD ) )
			return ADVSECURITY_ALL;
		return $this->getTableData(".nSecOptions");
	}
	function displayLoading()
	{
		return $this->getTableData(".isDisplayLoading");
	}
	function getRecordsPerPageArray()
	{
		return $this->getTableData(".arrRecsPerPage");
	}
	function getGroupsPerPageArray()
	{
		return $this->getTableData(".arrGroupsPerPage");
	}

	function isReportWithGroups()
	{
		return !!$this->getPageOption("newreport", "reportInfo", "groupFields" );
		//return $this->getTableData(".reportGroupFields");
	}

	function isCrossTabReport()
	{
		return $this->getPageOption("newreport", "reportInfo", "crosstab" );
		//return $this->getTableData(".crossTabReport");
	}

	function getReportGroupFields()
	{
		$ret = array();
		foreach( $this->getPageOption("newreport", "reportInfo", "groupFields" ) as $g ) {
			$ret[] = $g["field"];
		}
		return $ret;
		//return $this->getTableData(".reportGroupFieldsList");
	}

	function getReportGroupFieldsData()
	{
		$ret = array();
		foreach( $this->getPageOption("newreport", "reportInfo", "groupFields" ) as $idx => $g ) {
			$gdata = array();
			$gdata["strGroupField"] = $g["field"];
			$gdata["groupInterval"] = $g["interval"];
			$gdata["groupOrder"] = $idx + 1;
			$gdata["showGroupSummary"] = $g["summary"];
			$gdata["crossTabAxis"] = $g["axis"];
			$ret[] = $gdata;
		}
		return $ret;
//		return $this->getTableData(".reportGroupFieldsData");
	}

	function reportHasHorizontalSummary()
	{
		return $this->getPageOption("newreport", "reportInfo", "horizSummary" );
	}

	function reportHasVerticalSummary()
	{
		return $this->getPageOption("newreport", "reportInfo", "vertSummary" );
	}

	function reportHasPageSummary()
	{
		return $this->getPageOption("newreport", "reportInfo", "pageSummary" );
	}

	function reportHasGlobalSummary()
	{
		return $this->getPageOption("newreport", "reportInfo", "globalSummary" );
	}

	function getReportLayout()
	{
		//return  $this->getTableData(".reportLayout");
		$rLayout =  $this->getPageOption("newreport", "reportInfo", "layout" );
		if( $rLayout === 'stepped' ) {
			return REPORT_STEPPED;
		} 
		else if( $rLayout === 'align' ) {
			return REPORT_ALIGN;
		}
		else if( $rLayout === 'outline' ) {
			return REPORT_OUTLINE;
		}
		else if( $rLayout === 'block' ) {
			return REPORT_BLOCK;
		} else {
			return REPORT_TABULAR;
		}
	}

	function isGroupSummaryCountShown()
	{
		foreach( $this->getPageOption("newreport", "reportInfo", "groupFields" ) as $g ) {
			if( $g["summary"] ) {
				return true;
			}
		}
		return false;
	}

	function reportDetailsShown()
	{
		return $this->getPageOption("newreport", "reportInfo", "showData" );
	}

	function reportTotalFieldsExist()
	{
		foreach( $this->getPageOption("newreport", "reportInfo", "fields" ) as $f ) {
			if( $f["sum"] || $f["min"] || $f["max"] || $f["avg"] ) {
				return true;
			}
		}
		return false;
	}

	/**
	 * boolean
	 * True if the field has 'max' total in report
	 */
	function reportFieldInfo( $field ) {
		foreach( $this->getPageOption("newreport", "reportInfo", "fields" ) as $f ) {
			if( $f["field"] === $field ) {
				return $f;
			}
		}
		return null;
	}

	function noRecordsOnFirstPage()
	{
		return $this->getTableData(".noRecordsFirstPage");
	}

	function isViewPagePDF()
	{
		return $this->getTableData(".isViewPagePDF");
	}

	function isLandscapeViewPDFOrientation()
	{
		return $this->getTableData(".isLandscapeViewPDFOrientation");
	}

	function isViewPagePDFFitToPage()
	{
		return $this->getTableData(".isViewPagePDFFitToPage");
	}

	function getViewPagePDFScale()
	{
		return $this->getTableData(".nViewPagePDFScale");
	}

	function isLandscapePrinterPagePDFOrientation()
	{
		return $this->getTableData(".isLandscapePrinterPagePDFOrientation");
	}

	function isPrinterPagePDFFitToPage()
	{
		return $this->getTableData(".isPrinterPagePDFFitToPage");
	}

	function getPrinterPagePDFScale()
	{
		return $this->getTableData(".nPrinterPagePDFScale");
	}

	function isPrinterPageFitToPage()
	{
		return $this->getTableData(".isPrinterPageFitToPage");
	}

	function getPrinterPageScale()
	{
		return $this->getTableData(".nPrinterPageScale");
	}

	function getPrinterSplitRecords()
	{
		return $this->getTableData(".nPrinterSplitRecords");
	}

	function getPrinterPDFSplitRecords()
	{
		return $this->getTableData(".nPrinterPDFSplitRecords");
	}

	function isPrinterPagePDF()
	{
		return $this->getPageOption("pdf", "pdfView");
	}

	function hasCaptcha() {
		return $this->getPageOption( "captcha", "captcha" );
	}

	function hasBreadcrumb() {
		return $this->getPageOption( "misc", "breadcrumb" );
	}

	function isSearchRequiredForFiltering()
	{
		return $this->getTableData(".searchIsRequiredForFilters");
	}
	function warnLeavingPages()
	{
		return $this->getTableData(".warnLeavingPages");
	}

	function hideEmptyViewFields()
	{
		return $this->getTableData(".hideEmptyFieldsOnView");
	}

	function getInitialPageSize()
	{
		if( isReport( $this->getEntityType() ) ) {
			if( $this->isReportWithGroups() ) {
				return $this->getTableData(".pageSizeGroups");
			} else {
				return $this->getTableData(".pageSizeRecords");
			}

		}
		return $this->getTableData(".pageSize");
	}

	function getRecordsPerRowList()
	{
		return $this->getPageOption( "page", "recsPerRow" );
	}

	function getRecordsPerRowPrint()
	{
		return $this->getPageOption( "page", "recsPerRow" );
	}

	function getRecordsLimit()
	{
		return $this->getTableData(".recsLimit");
	}

	function useMoveNext()
	{
		return $this->getPageOption( "misc", "nextPrev" );
	}

	function highlightRows()
	{
		return $this->getTableData(".rowHighlite");
	}

	function hasInlineAdd()
	{
		return $this->getpageOption("list", "inlineAdd");
	}

	function getListGridLayout()
	{
		return $this->getPageOption("page", "gridType");
	}

	function getPrintGridLayout()
	{
		return $this->getPageOption("page", "gridType");
	}

	/*
	function getReportPrintLayout()
	{
		return $this->getTableData(".printReportLayout");
	}
	*/

	function getPrinterPageOrientation()
	{
		return $this->getTableData(".printerPageOrientation");
	}


	function getReportPrintGroupsPerPage()
	{
		if( $this->isReportWithGroups() )
			return $this->getTableData(".reportPrintGroupsPerPage");
		else 
			return $this->getTableData(".reportPrintRecordsPerPage");

	}

	function getReportPrintPDFGroupsPerPage()
	{
		return $this->getTableData(".reportPrintPDFGroupsPerPage");
	}

	function ajaxBasedListPage()
	{
		return $this->getTableData(".listAjax");
	}

	function isMultistepped()
	{
		return $this->getPageOption("page", "multiStep");
	}

	function hasVerticalBar()
	{
		return $this->getPageOption("page", "verticalBar");
	}


	/**
	 * Returns array of tabs, which use on list/report/chart page
	 * @return array
	 */
	function getGridTabs()
	{
		return $this->getTableData(".arrGridTabs");
	}


	function highlightSearchResults()
	{
		return $this->getTableData(".highlightSearchResults");
	}


	function getFieldsList()
	{
		if(is_null($this->_tableData))
			return array();
		$t = array_keys($this->_tableData);
		$arr = array();
		foreach($t as $f)
			if(substr($f,0,1)!=".")
				$arr[] = $f;
		return $arr;
	}

	function getFieldCount() {
		return count( $this->getFieldsList() );
	}


	function getBinaryFieldsIndices()
	{
		$fields = $this->getFieldsList();
		$out = array();
		foreach($fields as $idx => $f)
		{
			if(IsBinaryType($this->getFieldType($f)))
				$out[] = $idx + 1;
		}
		return $out;
	}

	function getNBFieldsList()
	{
		$t = $this->getFieldsList();
		$arr = array();
		foreach($t as $f)
			if(!IsBinaryType($this->getFieldType($f)))
				$arr[] = $f;
		return $arr;
	}

	/**
	 * Get field by good field name
	 * @param string	$field The good field name
	 * @return string
	 */
	function getFieldByGoodFieldName($field)
	{
		foreach($this->_tableData as $key => $value)
		{
			if( is_array( $value ) && count($value) > 1)
			{
				if($value["GoodName"] == $field)
					return $key;
			}
		}
		return "";
	}

	/**
	 * getUploadFolder
	 * Return inputed value or calculated path for upload folder
	 * @param {string} field name
	 * @param {array} file info (name, type, size)
	 */
	function getUploadFolder($field, $fileData = array())
	{
		if($this->isUploadCodeExpression($field))
			$path = GetUploadFolderExpression($field, $fileData, $this->_table );
		else
			$path = $this->getFieldData($field, "UploadFolder");
				if(strlen($path) && substr($path,strlen($path)-1) != "/")
			$path.="/";
		return $path;
	}

	function isMakeDirectoryNeeded($field)
	{
		return $this->isUploadCodeExpression($field) || !$this->isAbsolute($field);
	}

	function getFinalUploadFolder($field, $fileData = array())
	{
		if($this->isAbsolute($field))
			$path = $this->getUploadFolder($field, $fileData);
		else
			$path = getabspath($this->getUploadFolder($field, $fileData));
				if(strlen($path) && substr($path,strlen($path)-1) != "/")
			$path.="/";
		return $path;
	}

	function isUploadCodeExpression($field)
	{
		return $this->getFieldData($field, "UploadCodeExpression");
	}

	function &getQueryObject()
	{
		$queryObj = $this->getSQLQuery();
		return $queryObj;
	}

	function getListOfFieldsByExprType($needaggregate)
	{
		$query = &$this->getSQLQuery();
		if( !$query ) {
			return array();
		}
		$fields = $this->getFieldsList();
		$out = array();
		foreach($fields as $idx=>$f)
		{
			$aggr = $query->IsAggrFuncField($idx);
			if($needaggregate && $aggr || !$needaggregate && !$aggr)
				$out[] = $f;
		}
		return $out;
	}

	function isAggregateField($field)
	{
		$query = &$this->getSQLQuery();
		$idx = $this->getFieldIndex($field) - 1;
		return $query->IsAggrFuncField($idx);
	}

	/**
	 * Check if searching is case insensitive for the table
	 * @return Boolean
	 */
	function getNCSearch()
	{
		return $this->getTableData(".NCSearch");
	}

	function getChartType()
	{
		return $this->getTableData(".chartType");
	}

	function getChartRefreshTime()
	{
		return $this->getTableData(".ChartRefreshTime");
	}

	function getChartXml()
	{
		return $this->getTableData(".chartXml");
	}

	function auditEnabled()
	{
		return $this->getTableData(".audit");
	}

	function isSearchSavingEnabled()
	{
		return $this->getPageOption( "listSearch", "searchSaving" );
	}

	function isAllowShowHideFields()
	{
		if ( $this->getScrollGridBody() )
			return false;

		return $this->getPageOption( "list", "showHideFields" );
		//return $this->getTableData(".allowShowHideFields");
	}

	function isAllowFieldsReordering()
	{
		if ( $this->getScrollGridBody() || $this->getRecordsPerRowList() > 1)
			return false;

		return $this->getPageOption( "list", "reorderFields" );
	}

	function lockingEnabled()
	{
		return $this->getTableData(".locking");
	}

	function hasEncryptedFields()
	{
		return $this->getTableData(".hasEncryptedFields");
	}

	function showSearchPanel()
	{
		//return $this->getTableData(".showSearchPanel");
		return $this->getPageOption( "listSearch", "searchPanel" );
	}

	function isFlexibleSearch()
	{
		// return $this->getTableData(".flexibleSearch");
		return !$this->getPageOption( "listSearch", "fixedSearchPanel" );
	}

	function getSearchRequiredFields()
	{
		return $this->getPageOption("fields", "searchRequiredFields");
	}

	function showSimpleSearchOptions()
	{
//		return $this->getTableData(".showSimpleSearchOptions");
		return $this->getPageOption("listSearch", "simpleSearchOptions");
	}

	function getFieldsToHideIfEmpty()
	{
		return $this->getPageOption("fields", "hideEmptyFields");
	}

	function getFilterFields()
	{
		$ret = $this->getPageOption("fields", "filterFields");
		if( !$ret )
			return array();
		return $ret;
	}

	function getFilterFieldFormat($field)
	{
		return $this->getFieldData($field, "filterFormat");
	}

	function getFilterFieldTotal($field)
	{
		return $this->getFieldData($field, "filterTotals");
	}

	function showWithNoRecords($field)
	{
		return $this->getFieldData($field, "showWithNoRecords");
	}

	function getFilterSortValueType($field)
	{
		return $this->getFieldData($field, "sortValueType");
	}

	function isFilterSortOrderDescending($field)
	{
		return $this->getFieldData($field, "descendingOrder");
	}

	function getNumberOfVisibleFilterItems($field)
	{
		return $this->getFieldData($field, "numberOfVisibleItems");
	}

	function getFilterByInterval($field)
	{
		return $this->getFieldData($field, "filterBy");
	}

	function getParentFilterName($field)
	{
		return $this->getFieldData($field, "parentFilterField");
	}

	function getFilterIntervals($field)
	{
		return $this->getFieldData($field, "filterIntervals");
	}

	function showCollapsed($field)
	{
		return $this->getFieldData($field, "showCollapsed");
	}

	function getFilterIntervalDatabyIndex($field, $idx)
	{
		$intervalData = array();

		$filterIntervalsData = $this->getFilterIntervals($field);
		foreach($filterIntervalsData as $interval)
		{
			if($interval["index"] == $idx)
			{
				$intervalData = $interval;
				break;
			}
		}

		return $intervalData;
	}

	function getFilterTotalsField($field)
	{
		return $this->getFieldData($field, "filterTotalFields");
	}

	function getFilterFiledMultiSelect($field)
	{
		return $this->getFieldData($field, "filterMultiSelect");
	}
	
	function getBooleanFilterMessageData( $field, $checked ) {
		if( $checked ) {
			return array (
					"text" => $this->getFieldData($field, "filterCheckedMessageText"),
					"type" => $this->getFieldData($field, "filterCheckedMessageType")
				);
		}
		return array (
				"text" => $this->getFieldData($field, "filterUncheckedMessageText"),
				"type" => $this->getFieldData($field, "filterUncheckedMessageType")
			);
	}
	

	function getFilterStepType($field)
	{
		return $this->getFieldData($field, "filterSliderStepType");
	}

	function getFilterStepValue($field)
	{
		return $this->getFieldData($field, "filterSliderStepValue");
	}

	function getFilterKnobsType($field)
	{
		return $this->getFieldData($field, "filterKnobsType");
	}

	function isFilterApplyBtnSet($field)
	{
		return $this->getFieldData($field, "filterApplyBtn");
	}

	function isCaseInsensitiveUsername()
	{
		global $caseInsensitiveUsername;
		return $caseInsensitiveUsername;
	}

	/**
	 * Returns true if Users table is included into the project ( and has ist own SQL string )
	 * @return boolean
	 */
	function usersTableInProject()
	{
		return GetGlobalData("usersTableInProject",false);
	}


	function getStrField($field)
	{
		return $this->getFieldData($field, "strField");
	}

	function getSourceSingle( $field )
	{
		return $this->getFieldData($field, "sourceSingle");
	}

	function getFieldSource( $field, $listRequest )
	{
		return $listRequest 
			? $this->getFieldData( $field, "strField" )
			: $this->getFieldData( $field, "sourceSingle" );
	}

	function getScrollGridBody()
	{
		return $this->getTableData(".scrollGridBody");
	}

	/**
	 * Is 'UpdateLatLng' ticked for the table
	 */
	function isUpdateLatLng()
	{
		return $this->getTableData(".geocodingEnabled");
	}

	/**
	 * get geocoding data for the table
	 */
	function getGeocodingData()
	{
		return $this->getTableData(".geocodingData");
	}

	function allowDuplicateValues($field)
	{
		return !$this->getFieldData($field, "denyDuplicates");
	}

	function getDashFieldData($field, $key)
	{
		$dashSearchFields = $this->getDashboardSearchFields();
		$dfield = $dashSearchFields[ $field ];
		if( $dfield )
			$table = $dfield[0]["table"];
		if( !$dfield || !$table)
			return $this->getDefaultValueByKey( $key );

		if( !$this->_dashboardElemPSet[ $table ] )
			$this->_dashboardElemPSet[ $table ] = new ProjectSettings( $table, $this->_editPage );

		return $this->_dashboardElemPSet[ $table ]->getFieldData( $dfield[0]["field"], $key );
	}

	function getDashTableData($field, $key)
	{
		$dashSearchFields = $this->getDashboardSearchFields();
		$tableSettings = new ProjectSettings($dashSearchFields[$field][0]["table"], $this->_editPage);
		return $tableSettings->getTableData( $key );
	}


	/**
	 * It returns an empty array if the 'Select all' (search options) is ticked
	 */
	function getSearchOptionsList($field)
	{
		return $this->getFieldData($field, "searchOptionsList");
	}


	function getDefaultSearchOption($field)
	{
		$defaultOpt = $this->getFieldData($field, "defaultSearchOption");

		if( !$defaultOpt )
		{
			$searchOptionsList = $this->getSearchOptionsList($field);
			if( count($searchOptionsList) )
				$defaultOpt = $searchOptionsList[0];
		}
		return $defaultOpt;
	}

	/**
	 * Get status show list of thumbnails
	 * @param string $field field name
	 * @return boolean
	 */
	function showListOfThumbnails($field)
	{
		return $this->getFieldData($field, "ShowListOfThumbnails");
	}

	static function isMenuTreelike( $menuName )
	{
		global $menuTreelikeFlags;
		return $menuTreelikeFlags[$menuName];
	}

	function setPageMode($pageMode)
	{
		$this->_pageMode = $pageMode;
	}

	function editPageHasDenyDuplicatesFields()
	{
		foreach($this->getEditFields() as $fieldName)
		{
			if( !$this->allowDuplicateValues($fieldName) )
			{
				return true;
			}
		}
		return false;
	}

	/**
	 * Get a field's RTE type for RTE fields
	 * and the empty string for others
	 * @param {String} $field 	A field's name
	 * @return {String}
	 */
	function getRTEType($field)
	{
		if($this->isUseRTEBasic($field))
		{
			return "RTE";
		}
		
		if( $this->isUseRTECKNew($field) )
		{
			return "RTECK_NEW";
		}
		
		if($this->isUseRTEFCK($field))
		{
			return "RTECK";
		}
		if($this->isUseRTEInnova($field))
		{
			return "RTEINNOVA";
		}
		return "";
	}

	/**
	 * Get the list of the fields that must be hidden on the List page on a particular device
	 * @param {Number} Device code : 1-4
	 * @return {Array} array in the form of: $arr[$field]=true, where the $field must be hidden.
	 */
	function getHiddenFields($device)
	{
		$list = $this->getTableData( ".hideMobileList" );
		if( isset( $list[$device] ) )
			return $list[$device];
		return array();
	}

	/**
	 * Get the list of the fields that must be hidden on the List page on a particular device
	 * @param {Number} Device code : 1-4
	 * @return {Array} array in the form of: $arr[$goodfieldName]=true, where the $field must be hidden.
	 */
	function getHiddenGoodNameFields($device)
	{
		$hGoodFields = array();
		$hFields = $this->getHiddenFields($device);
		foreach ( $hFields as $field => $isShow ) {
			$hGoodFields[GoodFieldName($field)] = $isShow;
		}

		return $hGoodFields;
	}

	/**
	 * Checks if the 'columns By Device' is enabled
	 * @return {Boolean}
	 */
	function columnsByDeviceEnabled()
	{
		$list = $this->getTableData( ".hideMobileList" );
		foreach( $list as $d => $v )
		{
			if( $v )
				return true;
		}
		return false;
	}


	/**
	 * Build CSS media clause for given device code.
	 * Thanks to mobile device vendors, there are plenty of weird heuristics here, take it easy.
	 */
	static function getDeviceMediaClause($device)
	{
		if( $device == DESKTOP )
		{
			// width >= 1281

			//	Desktop. We don't care about prehistoric VGA displays
			return "@media (min-device-width: 1281px)";
		}
		else if( $device == TABLET_10_IN )	//	10" tablets
		{
			// width >= 768 and height == 1024 ( iPad )
			// or width between 1025 and 1280 and height is less than 1023
			// or the same with width and height interchanged

			// All iPads including mini go here, because there is no way to tell full-sized iPad from mini.
			return "@media (device-width: 768px) and (device-height: 1024px)".

			// The rest of contemporary 10" devices supposedly go here
				" , (min-device-width: 1025px) and (max-device-width: 1280px) and (max-device-height: 1023px) , (min-device-height: 1025px) and (max-device-height: 1280px) and (max-device-width: 1023px)";
		}
		else if( $device == TABLET_7_IN )
		{
			// width between 401 and 1024 and height between 401 and 800
			// or the same with width and height interchanged

			return "@media (min-device-height: 401px) and (max-device-height: 800px) and (min-device-width: 401px) and (max-device-width: 1024px) , (min-device-height: 401px) and (min-device-width: 401px) and (max-device-height: 1024px) and (max-device-width: 800px)";
		}

		else if( $device == SMARTPHONE_LANDSCAPE )
		{
			// landscape mode and width or height no more than 400
			return "@media (orientation: landscape) and (max-device-height: 400px), (orientation: landscape) and (max-device-width: 400px)";
		}

		else if( $device == SMARTPHONE_PORTRAIT )
		{
			// potrait mode and width or height no more than 400
			return "@media (orientation: portrait) and (max-device-height: 400px), (orientation: portrait) and (max-device-width: 400px)";
		}
	}

	/**
	 * @return Mixed
	 */
	public static function getForLogin()
	{
		return new ProjectSettings("system_user", PAGE_LIST);
	}

	/**
	 * Returns the list of the dashboard search fields.
	 * Each field is an array of "table", "field" pairs.
	 */
	function getDashboardSearchFields()
	{
		return $this->getTableData(".searchFields");
	}
	/**
	 * Returns the list of the dashboard elements
	 * Each element is represented by the array of "name", "table" and "type"
	 */
	function getDashboardElements()
	{
		return $this->getTableData(".dashElements");
	}

	/**
	 * @param String dashElementName
	 * @return Array
	 */
	function getDashboardElementData( $dashElementName )
	{
		$dElements = $this->getTableData(".dashElements");
		foreach( $dElements as $dElemData )
		{
			if( $dElemData["elementName"] == $dashElementName )
				return $dElemData;
		}
		return array();
	}

	/**
	 * @return Number
	 */
	function getAfterAddAction()
	{
		return $this->getTableData(".afterAddAction");
	}

	/**
	 * @return String
	 */
	function getAADetailTable()
	{
		return $this->getTableData(".afterAddActionDetTable");
	}

	/**
	 * @return Boolean
	 */
	function checkClosePopupAfterAdd()
	{
		return $this->getTableData(".closePopupAfterAdd");
	}

	/**
	 * @return Number
	 */
	function getAfterEditAction()
	{
		return $this->getTableData(".afterEditAction");
	}

	/**
	 * @return String
	 */
	function getAEDetailTable()
	{
		return $this->getTableData(".afterEditActionDetTable");
	}

	/**
	 * @return Boolean
	 */
	function checkClosePopupAfterEdit()
	{
		return $this->getTableData(".closePopupAfterEdit");
	}

	function getMapIcon($field, $data)
	{
		if( !$this->isMapIconCustom($field) ) {
			$mapData = $this->getMapData($field);
			if( $mapData["mapIcon"] != "" )
				return "images/menuicons/" . $mapData["mapIcon"];
			return "";
		}
		else
		{
			return getCustomMapIcon($field, "", $data);
		}
	}

	/**
	 * @param String dashElementName
	 * @param Array data
	 * @return String
	 */
	function getDashMapIcon( $dashElementName, $data )
	{
		$dashElementData = $this->getDashboardElementData( $dashElementName );

		if( $dashElementData["isMarkerIconCustom"] )
			return getDashMapCustomIcon( $this->_table, $dashElementName, $data );

		if( $dashElementData["iconF"] )
			return $dashElementData["iconF"];

		return "";
	}

	function isMapIconCustom($field)
	{
		$mapData = $this->getMapData($field);
		return $mapData["isMapIconCustom"];
	}

	function getDetailsBadgeColor()
	{
		return $this->getTableData(".badgeColor");
	}

	function & helperFormItems()
	{
		return $this->getPageOption("layoutHelper", "formItems");
	}
	function & helperItemsByType()
	{
		return $this->getPageOption("layoutHelper", "itemsByType");
	}

	function & allFieldItems()
	{
		return $this->getPageOption("fields", "fieldItems");
	}

	function & helperItemVisibility()
	{
		return $this->getPageOption("layoutHelper", "itemVisibility");
	}

	function & helperCellMaps()
	{
		return $this->getPageOption("layoutHelper", "cellMaps");
	}

	function detailsShowCount($dTable)
	{
		return $this->getPageOption("details", $dTable, "showCount" );
	}

	function detailsHideEmpty($dTable)
	{
		return $this->getPageOption("details", $dTable, "hideEmptyChild" );
	}
	
	function detailsHideEmptyPreview($dTable)
	{
		return $this->getPageOption("details", $dTable, "hideEmptyPreview" );
	}	

	function detailsPreview($dTable)
	{
		return $this->getPageOption("details", $dTable, "displayPreview" );
	}

	function detailsProceedLink($dTable)
	{
		return $this->getPageOption("details", $dTable, "showProceedLink" );
	}

	function detailsPrint($dTable)
	{
		return $this->getPageOption("details", $dTable, "printDetails" );
	}

	function detailsLinks()
	{
		return $this->getPageOption("allDetails", "linkType" );
	}
	function detailsPageId( $dTable )
	{
		return $this->getPageOption("details", $dTable, "previewPageId" );
	}

	function masterPreview( $mTable )
	{
		return $this->getPageOption("master", $mTable, "preview" );
	}

	/**
	 * @return boolean if success otherwise false
	 */
	function hasMap()
	{
		return !!$this->getPageOption("events", "maps" );
	}
	function maps()
	{
		return $this->getPageOption("events", "maps" );
	}

	function mapsData()
	{
		return $this->getPageOption("events", "mapsData" );
	}

	function buttons()
	{
		return $this->getPageOption("events", "buttons" );
	}

	function getPageType( $page = "" )
	{
		if( !$page )
			$page = $this->_page;
		return $this->_auxTableData[ ".pages" ][ $page ];
	}

	/**
	 * returns page type without applying page permissions
	 */
	function getOriginalPageType( $page = "" )
	{
		if( !$page )
			$page = $this->_page;
		return $this->_auxTableData[ ".originalPages" ][ $page ];
	}

	function welcomeItems()
	{
		return $this->getPageOption("welcome", "welcomeItems");
	}

	function getMultipleImgMode( $field )
	{
		return $this->getFieldData( $field, "multipleImgMode" );
	}

	function getMaxImages( $field )
	{
		return $this->getFieldData( $field, "maxImages" );
	}
	function isGalleryEnabled( $field )
	{
		return $this->getFieldData( $field, "showGallery" );
	}
	function getGalleryMode( $field )
	{
		return $this->getFieldData( $field, "galleryMode" );
	}
	function getCaptionMode( $field )
	{
		return $this->getFieldData( $field, "captionMode" );
	}
	function getCaptionField( $field )
	{
		return $this->getFieldData( $field, "captionField" );
	}

	/**
	 * @return boolean
	 */
	function getImageBorder( $field )
	{
		return $this->getFieldData( $field, "imageBorder" );
	}

	function getImageFullWidth( $field )
	{
		return $this->getFieldData( $field, "imageFullWidth" );
	}

	/**
	 * Page exists
	 * @return Boolean
	 */
	function pageTypeAvailable( $pageType ) {
		$pagesByType =& $this->_auxTableData[".pagesByType"];
		return !!$pagesByType[ $pageType ];
	}

	function updatePages() {
		
		if( $this->_auxTableData[".pagesUpdated"] ) {
			return;
		}
 
		//	don't filter pages available to all users
		if( $this->_pageType == PAGE_LOGIN 
			|| $this->_pageType == PAGE_REGISTER
			|| $this->_pageType == PAGE_REMIND
			|| $this->_pageType == PAGE_REMIND_SUCCESS
		) {
			return;
		}


		$this->_auxTableData[".pagesUpdated"] = true;

		$restrictedPages = Security::getRestrictedPages( $this->_auxTable );
		if( !$restrictedPages ) {
			return;
		}
		$pages =& $this->_auxTableData[".pages"];
		$pagesByType =& $this->_auxTableData[".pagesByType"];
		$newPages = array();
		$defaultPages =& $this->_auxTableData[".defaultPages"];
		foreach( $pages as $p => $type ) {
			if( !$restrictedPages[$p] ) {
				$newPages[ $p ] = $type;
			}
			else {
				$idx = array_search( $p, $pagesByType[$type] );
				unset( $pagesByType[$type][$idx] );
				if( $defaultPages[ $type ] == $p ) {
					$defaultPages[ $type ] = "";
					//	pick the first available page
					foreach( $pagesByType[$type] as $d ) {
						$defaultPages[ $type ] = $d;
						break;
					}
				}
			}
		}
		$this->_auxTableData[".pages"] = &$newPages;
	}
	
	/**
	 * Revert pages list to original.
	 * This is called from Security::setRestrictedPages/setAllowedPages
	 */
	function resetPages() {
		unset( $this->_auxTableData[".pagesUpdated"] );
		$this->_auxTableData[".defaultPages"] = $this->_auxTableData[".originalDefaultPages"];
		$this->_auxTableData[".pages"] = $this->_auxTableData[".originalPages"];
	}

	/**
	 * returns array of page ids ignoring user permissions
	 */
	function getOriginalPagesByType( $pageType) {
		return $this->_auxTableData[".originalPagesByType"][ $pageType ];
	}

	function &getDataSourceOps()
	{
		return $this->_tableData[".operations"];
	}

	/**
	 * @return Boolean
	 */
	function groupChart() {
		return $this->getTableData( ".groupChart" );
	}

	/**
	 * @return Integer
	 */
	function chartLabelInterval() {
		return $this->getTableData( ".chartLabelInterval" );
	}

	/**
	 * @return Array
	 */
	function chartSeries() {
		return $this->getTableData( ".chartSeries" );
	}

	/**
	 * @return String
	 */
	function chartLabelField() {
		return $this->getTableData( ".chartLabelField" );
	}

	function getViewPageType() {
		return $this->_viewPage;
	}

	function spreadsheetGrid() {
		return $this->getPageOption( "list", "spreadsheetMode");
	}

	/**
	 * Tell whether the edit format involves file uploading
	 * @return boolean
	 */
	public static function uploadEditType( $editFormat ) {
		return $editFormat == EDIT_FORMAT_DATABASE_FILE || $editFormat == EDIT_FORMAT_DATABASE_IMAGE || $editFormat == EDIT_FORMAT_FILE;
	}
	
	public function addNewRecordAutomatically() {
		return $this->getPageOption( "list", "addNewRecordAutomatically" );
	}

	public function reorderRows() {
		return $this->getPageOption( "list", "reorderRows" ) && $this->reorderRowsField() != '';
	}

	public function reorderRowsField() {
		return $this->getPageOption( "list", "reorderRowsField");
	}

	public function inlineAddBottom() {
		return !!( $this->getPageOption( "list", "addToBottom") || $this->spreadsheetGrid() && $this->addNewRecordAutomatically() );
	}

}


function fillProjectEntites()
{
	global $projectEntities, $projectEntitiesReverse, $tablesByGoodName, $tablesByUpperCase, $tablesByUpperGoodname;
	if( count( $projectEntities ) )
		return;
	$projectEntities[ "business_partner" ] = array( "url" => "business_partner", "type" => 0 );
	$projectEntitiesReverse[ "business_partner" ] = "business_partner";
	$projectEntities[ "item" ] = array( "url" => "item", "type" => 0 );
	$projectEntitiesReverse[ "item" ] = "item";
	$projectEntities[ "credit_terms" ] = array( "url" => "credit_terms", "type" => 0 );
	$projectEntitiesReverse[ "credit_terms" ] = "credit_terms";
	$projectEntities[ "shipping_terms" ] = array( "url" => "shipping_terms", "type" => 0 );
	$projectEntitiesReverse[ "shipping_terms" ] = "shipping_terms";
	$projectEntities[ "cash_receipt" ] = array( "url" => "cash_receipt", "type" => 0 );
	$projectEntitiesReverse[ "cash_receipt" ] = "cash_receipt";
	$projectEntities[ "purchase" ] = array( "url" => "purchase", "type" => 0 );
	$projectEntitiesReverse[ "purchase" ] = "purchase";
	$projectEntities[ "cash_disbursment" ] = array( "url" => "cash_disbursment", "type" => 0 );
	$projectEntitiesReverse[ "cash_disbursment" ] = "cash_disbursment";
	$projectEntities[ "general_journal" ] = array( "url" => "general_journal", "type" => 0 );
	$projectEntitiesReverse[ "general_journal" ] = "general_journal";
	$projectEntities[ "report_center" ] = array( "url" => "report_center", "type" => 0 );
	$projectEntitiesReverse[ "report_center" ] = "report_center";
	$projectEntities[ "Sales Report" ] = array( "url" => "Sales_Report", "type" => 1 );
	$projectEntitiesReverse[ "Sales_Report" ] = "Sales Report";
	$projectEntities[ "Disbursement Reports" ] = array( "url" => "Disbursement_Reports", "type" => 1 );
	$projectEntitiesReverse[ "Disbursement_Reports" ] = "Disbursement Reports";
	$projectEntities[ "Financial Reports" ] = array( "url" => "Financial_Reports", "type" => 1 );
	$projectEntitiesReverse[ "Financial_Reports" ] = "Financial Reports";
	$projectEntities[ "Purchase Reports" ] = array( "url" => "Purchase_Reports", "type" => 1 );
	$projectEntitiesReverse[ "Purchase_Reports" ] = "Purchase Reports";
	$projectEntities[ "Receivable Reports" ] = array( "url" => "Receivable_Reports", "type" => 1 );
	$projectEntitiesReverse[ "Receivable_Reports" ] = "Receivable Reports";
	$projectEntities[ "Taxation Reports" ] = array( "url" => "Taxation_Reports", "type" => 1 );
	$projectEntitiesReverse[ "Taxation_Reports" ] = "Taxation Reports";
	$projectEntities[ "system_user" ] = array( "url" => "system_user", "type" => 0 );
	$projectEntitiesReverse[ "system_user" ] = "system_user";
	$projectEntities[ "fiscal_year" ] = array( "url" => "fiscal_year", "type" => 0 );
	$projectEntitiesReverse[ "fiscal_year" ] = "fiscal_year";
	$projectEntities[ "beg_bal" ] = array( "url" => "beg_bal", "type" => 0 );
	$projectEntitiesReverse[ "beg_bal" ] = "beg_bal";
	$projectEntities[ "ewt" ] = array( "url" => "ewt", "type" => 0 );
	$projectEntitiesReverse[ "ewt" ] = "ewt";
	$projectEntities[ "bank" ] = array( "url" => "bank", "type" => 0 );
	$projectEntitiesReverse[ "bank" ] = "bank";
	$projectEntities[ "cost_center" ] = array( "url" => "cost_center", "type" => 0 );
	$projectEntitiesReverse[ "cost_center" ] = "cost_center";
	$projectEntities[ "General Journal Reports" ] = array( "url" => "General_Journal_Reports", "type" => 1 );
	$projectEntitiesReverse[ "General_Journal_Reports" ] = "General Journal Reports";
	$projectEntities[ "Inventory Reports" ] = array( "url" => "Inventory_Reports", "type" => 1 );
	$projectEntitiesReverse[ "Inventory_Reports" ] = "Inventory Reports";
	$projectEntities[ "accounting_period" ] = array( "url" => "accounting_period", "type" => 0 );
	$projectEntitiesReverse[ "accounting_period" ] = "accounting_period";
	$projectEntities[ "bank_recon" ] = array( "url" => "bank_recon", "type" => 1 );
	$projectEntitiesReverse[ "bank_recon" ] = "bank_recon";
	$projectEntities[ "deposit" ] = array( "url" => "deposit", "type" => 0 );
	$projectEntitiesReverse[ "deposit" ] = "deposit";
	$projectEntities[ "account_classification" ] = array( "url" => "account_classification", "type" => 0 );
	$projectEntitiesReverse[ "account_classification" ] = "account_classification";
	$projectEntities[ "chart_of_accounts" ] = array( "url" => "chart_of_accounts", "type" => 0 );
	$projectEntitiesReverse[ "chart_of_accounts" ] = "chart_of_accounts";
	$projectEntities[ "recon_CR" ] = array( "url" => "recon_CR", "type" => 1 );
	$projectEntitiesReverse[ "recon_CR" ] = "recon_CR";
	$projectEntities[ "Recon_deposit" ] = array( "url" => "Recon_deposit", "type" => 1 );
	$projectEntitiesReverse[ "Recon_deposit" ] = "Recon_deposit";
	$projectEntities[ "Recon_CV" ] = array( "url" => "Recon_CV", "type" => 1 );
	$projectEntitiesReverse[ "Recon_CV" ] = "Recon_CV";
	$projectEntities[ "Recon_GV_dep" ] = array( "url" => "Recon_GV_dep", "type" => 1 );
	$projectEntitiesReverse[ "Recon_GV_dep" ] = "Recon_GV_dep";
	$projectEntities[ "Recon_GV_CV" ] = array( "url" => "Recon_GV_CV", "type" => 1 );
	$projectEntitiesReverse[ "Recon_GV_CV" ] = "Recon_GV_CV";
	$projectEntities[ "vat_atc" ] = array( "url" => "vat_atc", "type" => 0 );
	$projectEntitiesReverse[ "vat_atc" ] = "vat_atc";
	$projectEntities[ "rdo" ] = array( "url" => "rdo", "type" => 0 );
	$projectEntitiesReverse[ "rdo" ] = "rdo";
	$projectEntities[ "dr" ] = array( "url" => "dr", "type" => 0 );
	$projectEntitiesReverse[ "dr" ] = "dr";
	$projectEntities[ "purchase_order" ] = array( "url" => "purchase_order", "type" => 0 );
	$projectEntitiesReverse[ "purchase_order" ] = "purchase_order";
	$projectEntities[ "purchase_order_receiving" ] = array( "url" => "purchase_order_receiving", "type" => 0 );
	$projectEntitiesReverse[ "purchase_order_receiving" ] = "purchase_order_receiving";
	$projectEntities[ "System_preference" ] = array( "url" => "System_preference", "type" => 1 );
	$projectEntitiesReverse[ "System_preference" ] = "System_preference";
	$projectEntities[ "ppe_list" ] = array( "url" => "ppe_list", "type" => 0 );
	$projectEntitiesReverse[ "ppe_list" ] = "ppe_list";
	$projectEntities[ "discounts" ] = array( "url" => "discounts", "type" => 0 );
	$projectEntitiesReverse[ "discounts" ] = "discounts";
	$projectEntities[ "item_group" ] = array( "url" => "item_group", "type" => 0 );
	$projectEntitiesReverse[ "item_group" ] = "item_group";
	$projectEntities[ "location" ] = array( "url" => "location", "type" => 0 );
	$projectEntitiesReverse[ "location" ] = "location";
	$projectEntities[ "meetings" ] = array( "url" => "meetings", "type" => 0 );
	$projectEntitiesReverse[ "meetings" ] = "meetings";
	$projectEntities[ "calls" ] = array( "url" => "calls", "type" => 0 );
	$projectEntitiesReverse[ "calls" ] = "calls";
	$projectEntities[ "campaign" ] = array( "url" => "campaign", "type" => 0 );
	$projectEntitiesReverse[ "campaign" ] = "campaign";
	$projectEntities[ "reimbursement" ] = array( "url" => "reimbursement", "type" => 0 );
	$projectEntitiesReverse[ "reimbursement" ] = "reimbursement";
	$projectEntities[ "reimbursement_detail" ] = array( "url" => "reimbursement_detail", "type" => 0 );
	$projectEntitiesReverse[ "reimbursement_detail" ] = "reimbursement_detail";
	$projectEntities[ "Target_customers" ] = array( "url" => "Target_customers", "type" => 1 );
	$projectEntitiesReverse[ "Target_customers" ] = "Target_customers";
	$projectEntities[ "quotation" ] = array( "url" => "quotation", "type" => 0 );
	$projectEntitiesReverse[ "quotation" ] = "quotation";
	$projectEntities[ "case" ] = array( "url" => "case", "type" => 0 );
	$projectEntitiesReverse[ "case" ] = "case";
	$projectEntities[ "production" ] = array( "url" => "production", "type" => 0 );
	$projectEntitiesReverse[ "production" ] = "production";
	$projectEntities[ "issue" ] = array( "url" => "issue", "type" => 0 );
	$projectEntitiesReverse[ "issue" ] = "issue";
	$projectEntities[ "issue_detail" ] = array( "url" => "issue_detail", "type" => 0 );
	$projectEntitiesReverse[ "issue_detail" ] = "issue_detail";
	$projectEntities[ "disposed" ] = array( "url" => "disposed", "type" => 0 );
	$projectEntitiesReverse[ "disposed" ] = "disposed";
	$projectEntities[ "disposed_detail" ] = array( "url" => "disposed_detail", "type" => 0 );
	$projectEntitiesReverse[ "disposed_detail" ] = "disposed_detail";
	$projectEntities[ "backload" ] = array( "url" => "backload", "type" => 0 );
	$projectEntitiesReverse[ "backload" ] = "backload";
	$projectEntities[ "backload_detail" ] = array( "url" => "backload_detail", "type" => 0 );
	$projectEntitiesReverse[ "backload_detail" ] = "backload_detail";
	$projectEntities[ "vehicle" ] = array( "url" => "vehicle", "type" => 0 );
	$projectEntitiesReverse[ "vehicle" ] = "vehicle";
	$projectEntities[ "uom" ] = array( "url" => "uom", "type" => 0 );
	$projectEntitiesReverse[ "uom" ] = "uom";
	$projectEntities[ "trip_ticket" ] = array( "url" => "trip_ticket", "type" => 0 );
	$projectEntitiesReverse[ "trip_ticket" ] = "trip_ticket";
	$projectEntities[ "approver_list" ] = array( "url" => "approver_list", "type" => 0 );
	$projectEntitiesReverse[ "approver_list" ] = "approver_list";
	$projectEntities[ "area" ] = array( "url" => "area", "type" => 0 );
	$projectEntitiesReverse[ "area" ] = "area";
	$projectEntities[ "currency" ] = array( "url" => "currency", "type" => 0 );
	$projectEntitiesReverse[ "currency" ] = "currency";
	$projectEntities[ "position" ] = array( "url" => "position", "type" => 0 );
	$projectEntitiesReverse[ "position" ] = "position";
	$projectEntities[ "service_center" ] = array( "url" => "service_center", "type" => 0 );
	$projectEntitiesReverse[ "service_center" ] = "service_center";
	$projectEntities[ "tax_status" ] = array( "url" => "tax_status", "type" => 0 );
	$projectEntitiesReverse[ "tax_status" ] = "tax_status";
	$projectEntities[ "competitor" ] = array( "url" => "competitor", "type" => 0 );
	$projectEntitiesReverse[ "competitor" ] = "competitor";
	$projectEntities[ "campaign_type" ] = array( "url" => "campaign_type", "type" => 0 );
	$projectEntitiesReverse[ "campaign_type" ] = "campaign_type";
	$projectEntities[ "fixed_asset_return_detail" ] = array( "url" => "fixed_asset_return_detail", "type" => 0 );
	$projectEntitiesReverse[ "fixed_asset_return_detail" ] = "fixed_asset_return_detail";
	$projectEntities[ "fixed_assets_return" ] = array( "url" => "fixed_assets_return", "type" => 0 );
	$projectEntitiesReverse[ "fixed_assets_return" ] = "fixed_assets_return";
	$projectEntities[ "Prospect_customers" ] = array( "url" => "Prospect_customers", "type" => 1 );
	$projectEntitiesReverse[ "Prospect_customers" ] = "Prospect_customers";
	$projectEntities[ "trip_detail" ] = array( "url" => "trip_detail", "type" => 0 );
	$projectEntitiesReverse[ "trip_detail" ] = "trip_detail";
	$projectEntities[ "beg_baldetail" ] = array( "url" => "beg_baldetail", "type" => 0 );
	$projectEntitiesReverse[ "beg_baldetail" ] = "beg_baldetail";
	$projectEntities[ "cash_disbursement_detail2" ] = array( "url" => "cash_disbursement_detail2", "type" => 0 );
	$projectEntitiesReverse[ "cash_disbursement_detail2" ] = "cash_disbursement_detail2";
	$projectEntities[ "case_solutions" ] = array( "url" => "case_solutions", "type" => 0 );
	$projectEntitiesReverse[ "case_solutions" ] = "case_solutions";
	$projectEntities[ "cash_disbursment_detail" ] = array( "url" => "cash_disbursment_detail", "type" => 0 );
	$projectEntitiesReverse[ "cash_disbursment_detail" ] = "cash_disbursment_detail";
	$projectEntities[ "cash_receipt_detail" ] = array( "url" => "cash_receipt_detail", "type" => 0 );
	$projectEntitiesReverse[ "cash_receipt_detail" ] = "cash_receipt_detail";
	$projectEntities[ "cash_receipt_detail2" ] = array( "url" => "cash_receipt_detail2", "type" => 0 );
	$projectEntitiesReverse[ "cash_receipt_detail2" ] = "cash_receipt_detail2";
	$projectEntities[ "general_journal_detail" ] = array( "url" => "general_journal_detail", "type" => 0 );
	$projectEntitiesReverse[ "general_journal_detail" ] = "general_journal_detail";
	$projectEntities[ "employee_list" ] = array( "url" => "employee_list", "type" => 1 );
	$projectEntitiesReverse[ "employee_list" ] = "employee_list";
	$projectEntities[ "petty_cash" ] = array( "url" => "petty_cash", "type" => 0 );
	$projectEntitiesReverse[ "petty_cash" ] = "petty_cash";
	$projectEntities[ "petty_cash_detail" ] = array( "url" => "petty_cash_detail", "type" => 0 );
	$projectEntitiesReverse[ "petty_cash_detail" ] = "petty_cash_detail";
	$projectEntities[ "center_replenishment" ] = array( "url" => "center_replenishment", "type" => 0 );
	$projectEntitiesReverse[ "center_replenishment" ] = "center_replenishment";
	$projectEntities[ "center_replenishment_detail" ] = array( "url" => "center_replenishment_detail", "type" => 0 );
	$projectEntitiesReverse[ "center_replenishment_detail" ] = "center_replenishment_detail";
	$projectEntities[ "credit_memo" ] = array( "url" => "credit_memo", "type" => 0 );
	$projectEntitiesReverse[ "credit_memo" ] = "credit_memo";
	$projectEntities[ "credit_memo_detail" ] = array( "url" => "credit_memo_detail", "type" => 0 );
	$projectEntitiesReverse[ "credit_memo_detail" ] = "credit_memo_detail";
	$projectEntities[ "debit_memo" ] = array( "url" => "debit_memo", "type" => 0 );
	$projectEntitiesReverse[ "debit_memo" ] = "debit_memo";
	$projectEntities[ "debit_memo_detail" ] = array( "url" => "debit_memo_detail", "type" => 0 );
	$projectEntitiesReverse[ "debit_memo_detail" ] = "debit_memo_detail";
	$projectEntities[ "project_replenishment" ] = array( "url" => "project_replenishment", "type" => 0 );
	$projectEntitiesReverse[ "project_replenishment" ] = "project_replenishment";
	$projectEntities[ "project_replenishment_detail" ] = array( "url" => "project_replenishment_detail", "type" => 0 );
	$projectEntitiesReverse[ "project_replenishment_detail" ] = "project_replenishment_detail";
	$projectEntities[ "business_partner_group" ] = array( "url" => "business_partner_group", "type" => 0 );
	$projectEntitiesReverse[ "business_partner_group" ] = "business_partner_group";
	$projectEntities[ "Sales_Representatives" ] = array( "url" => "Sales_Representatives", "type" => 1 );
	$projectEntitiesReverse[ "Sales_Representatives" ] = "Sales_Representatives";
	$projectEntities[ "sales" ] = array( "url" => "sales", "type" => 0 );
	$projectEntitiesReverse[ "sales" ] = "sales";
	$projectEntities[ "sales_detail" ] = array( "url" => "sales_detail", "type" => 0 );
	$projectEntitiesReverse[ "sales_detail" ] = "sales_detail";
	$projectEntities[ "item_bill_of_materials" ] = array( "url" => "item_bill_of_materials", "type" => 0 );
	$projectEntitiesReverse[ "item_bill_of_materials" ] = "item_bill_of_materials";
	$projectEntities[ "item_uom_conversion" ] = array( "url" => "item_uom_conversion", "type" => 0 );
	$projectEntitiesReverse[ "item_uom_conversion" ] = "item_uom_conversion";
	$projectEntities[ "tranasactions_for_approval" ] = array( "url" => "tranasactions_for_approval", "type" => 0 );
	$projectEntitiesReverse[ "tranasactions_for_approval" ] = "tranasactions_for_approval";
	$projectEntities[ "deposit_detail" ] = array( "url" => "deposit_detail", "type" => 0 );
	$projectEntitiesReverse[ "deposit_detail" ] = "deposit_detail";
	$projectEntities[ "purchase_detail" ] = array( "url" => "purchase_detail", "type" => 0 );
	$projectEntitiesReverse[ "purchase_detail" ] = "purchase_detail";
	$projectEntities[ "dr_detail" ] = array( "url" => "dr_detail", "type" => 0 );
	$projectEntitiesReverse[ "dr_detail" ] = "dr_detail";
	$projectEntities[ "purchase_order_detail" ] = array( "url" => "purchase_order_detail", "type" => 0 );
	$projectEntitiesReverse[ "purchase_order_detail" ] = "purchase_order_detail";
	$projectEntities[ "Sales_Order_for_checking" ] = array( "url" => "Sales_Order_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Sales_Order_for_checking" ] = "Sales_Order_for_checking";
	$projectEntities[ "sales_order_detail_for_checking" ] = array( "url" => "sales_order_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "sales_order_detail_for_checking" ] = "sales_order_detail_for_checking";
	$projectEntities[ "Sales_Order_for_approval" ] = array( "url" => "Sales_Order_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Sales_Order_for_approval" ] = "Sales_Order_for_approval";
	$projectEntities[ "sales_order_detail_for_approval" ] = array( "url" => "sales_order_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "sales_order_detail_for_approval" ] = "sales_order_detail_for_approval";
	$projectEntities[ "Quotation_for_approval" ] = array( "url" => "Quotation_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Quotation_for_approval" ] = "Quotation_for_approval";
	$projectEntities[ "Quotation_for_checking" ] = array( "url" => "Quotation_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Quotation_for_checking" ] = "Quotation_for_checking";
	$projectEntities[ "quotation_detail_for_checking" ] = array( "url" => "quotation_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "quotation_detail_for_checking" ] = "quotation_detail_for_checking";
	$projectEntities[ "quotation_detail_for_approval" ] = array( "url" => "quotation_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "quotation_detail_for_approval" ] = "quotation_detail_for_approval";
	$projectEntities[ "Reimbursement_for_checking" ] = array( "url" => "Reimbursement_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Reimbursement_for_checking" ] = "Reimbursement_for_checking";
	$projectEntities[ "reimbursement_detail_for_checking" ] = array( "url" => "reimbursement_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "reimbursement_detail_for_checking" ] = "reimbursement_detail_for_checking";
	$projectEntities[ "Reimbursement_for_approval" ] = array( "url" => "Reimbursement_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Reimbursement_for_approval" ] = "Reimbursement_for_approval";
	$projectEntities[ "reimbursement_detail_for_approval" ] = array( "url" => "reimbursement_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "reimbursement_detail_for_approval" ] = "reimbursement_detail_for_approval";
	$projectEntities[ "Project_Replenishment_for_checking" ] = array( "url" => "Project_Replenishment_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Project_Replenishment_for_checking" ] = "Project_Replenishment_for_checking";
	$projectEntities[ "project_replenishment_detail_for_checking" ] = array( "url" => "project_replenishment_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "project_replenishment_detail_for_checking" ] = "project_replenishment_detail_for_checking";
	$projectEntities[ "Project_Replenishment_for_approval" ] = array( "url" => "Project_Replenishment_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Project_Replenishment_for_approval" ] = "Project_Replenishment_for_approval";
	$projectEntities[ "project_replenishment_detail_for_approval" ] = array( "url" => "project_replenishment_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "project_replenishment_detail_for_approval" ] = "project_replenishment_detail_for_approval";
	$projectEntities[ "center_replenishment_detail_for_checking" ] = array( "url" => "center_replenishment_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "center_replenishment_detail_for_checking" ] = "center_replenishment_detail_for_checking";
	$projectEntities[ "center_replenishment_detail_for_approval" ] = array( "url" => "center_replenishment_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "center_replenishment_detail_for_approval" ] = "center_replenishment_detail_for_approval";
	$projectEntities[ "Purchases_Order_for_checking2" ] = array( "url" => "Purchases_Order_for_checking2", "type" => 1 );
	$projectEntitiesReverse[ "Purchases_Order_for_checking2" ] = "Purchases_Order_for_checking2";
	$projectEntities[ "purchase_order_detail_for_checking" ] = array( "url" => "purchase_order_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "purchase_order_detail_for_checking" ] = "purchase_order_detail_for_checking";
	$projectEntities[ "Purchases_Order_for_approval2" ] = array( "url" => "Purchases_Order_for_approval2", "type" => 1 );
	$projectEntitiesReverse[ "Purchases_Order_for_approval2" ] = "Purchases_Order_for_approval2";
	$projectEntities[ "purchase_order_detail_for_approval" ] = array( "url" => "purchase_order_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "purchase_order_detail_for_approval" ] = "purchase_order_detail_for_approval";
	$projectEntities[ "Receiving_for_checking2" ] = array( "url" => "Receiving_for_checking2", "type" => 1 );
	$projectEntitiesReverse[ "Receiving_for_checking2" ] = "Receiving_for_checking2";
	$projectEntities[ "purchase_order_receiving_detail_for_checking" ] = array( "url" => "purchase_order_receiving_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "purchase_order_receiving_detail_for_checking" ] = "purchase_order_receiving_detail_for_checking";
	$projectEntities[ "Receiving_for_approval2" ] = array( "url" => "Receiving_for_approval2", "type" => 1 );
	$projectEntitiesReverse[ "Receiving_for_approval2" ] = "Receiving_for_approval2";
	$projectEntities[ "purchase_order_receiving_detail_for_approval" ] = array( "url" => "purchase_order_receiving_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "purchase_order_receiving_detail_for_approval" ] = "purchase_order_receiving_detail_for_approval";
	$projectEntities[ "Delivery_Receipt_for_checking" ] = array( "url" => "Delivery_Receipt_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Delivery_Receipt_for_checking" ] = "Delivery_Receipt_for_checking";
	$projectEntities[ "dr_detail_for_checking" ] = array( "url" => "dr_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "dr_detail_for_checking" ] = "dr_detail_for_checking";
	$projectEntities[ "Delivery_Receipt_for_approval" ] = array( "url" => "Delivery_Receipt_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Delivery_Receipt_for_approval" ] = "Delivery_Receipt_for_approval";
	$projectEntities[ "dr_detail_for_approval" ] = array( "url" => "dr_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "dr_detail_for_approval" ] = "dr_detail_for_approval";
	$projectEntities[ "Production_for_checking" ] = array( "url" => "Production_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Production_for_checking" ] = "Production_for_checking";
	$projectEntities[ "production_detail_for_checking" ] = array( "url" => "production_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "production_detail_for_checking" ] = "production_detail_for_checking";
	$projectEntities[ "Production_for_approval" ] = array( "url" => "Production_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Production_for_approval" ] = "Production_for_approval";
	$projectEntities[ "production_detail_for_approval" ] = array( "url" => "production_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "production_detail_for_approval" ] = "production_detail_for_approval";
	$projectEntities[ "Issue_for_checking" ] = array( "url" => "Issue_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Issue_for_checking" ] = "Issue_for_checking";
	$projectEntities[ "supply_issue_detail_for_checking" ] = array( "url" => "supply_issue_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "supply_issue_detail_for_checking" ] = "supply_issue_detail_for_checking";
	$projectEntities[ "Issue_for_approval" ] = array( "url" => "Issue_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Issue_for_approval" ] = "Issue_for_approval";
	$projectEntities[ "supply_issue_detail_for_approval" ] = array( "url" => "supply_issue_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "supply_issue_detail_for_approval" ] = "supply_issue_detail_for_approval";
	$projectEntities[ "Return_for_checking" ] = array( "url" => "Return_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Return_for_checking" ] = "Return_for_checking";
	$projectEntities[ "supply_return_detail_for_checking" ] = array( "url" => "supply_return_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "supply_return_detail_for_checking" ] = "supply_return_detail_for_checking";
	$projectEntities[ "Return_for_approval" ] = array( "url" => "Return_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Return_for_approval" ] = "Return_for_approval";
	$projectEntities[ "supply_return_detail_approval" ] = array( "url" => "supply_return_detail_approval", "type" => 1 );
	$projectEntitiesReverse[ "supply_return_detail_approval" ] = "supply_return_detail_approval";
	$projectEntities[ "supply_disposed_detail_for_checking" ] = array( "url" => "supply_disposed_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "supply_disposed_detail_for_checking" ] = "supply_disposed_detail_for_checking";
	$projectEntities[ "Disposal_for_approval" ] = array( "url" => "Disposal_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Disposal_for_approval" ] = "Disposal_for_approval";
	$projectEntities[ "supply_disposed_detail_for_approval" ] = array( "url" => "supply_disposed_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "supply_disposed_detail_for_approval" ] = "supply_disposed_detail_for_approval";
	$projectEntities[ "Fixed_Asset_Issue_for_checking" ] = array( "url" => "Fixed_Asset_Issue_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Fixed_Asset_Issue_for_checking" ] = "Fixed_Asset_Issue_for_checking";
	$projectEntities[ "Fixed_Asset_Issue_for_approval" ] = array( "url" => "Fixed_Asset_Issue_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Fixed_Asset_Issue_for_approval" ] = "Fixed_Asset_Issue_for_approval";
	$projectEntities[ "fixed_asset_issue_detail_for_approval" ] = array( "url" => "fixed_asset_issue_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "fixed_asset_issue_detail_for_approval" ] = "fixed_asset_issue_detail_for_approval";
	$projectEntities[ "fixed_asset_issue_detail_for_checking" ] = array( "url" => "fixed_asset_issue_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "fixed_asset_issue_detail_for_checking" ] = "fixed_asset_issue_detail_for_checking";
	$projectEntities[ "Fixed_Asset_Return_for_checking" ] = array( "url" => "Fixed_Asset_Return_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Fixed_Asset_Return_for_checking" ] = "Fixed_Asset_Return_for_checking";
	$projectEntities[ "fixed_asset_return_detail_for_checking" ] = array( "url" => "fixed_asset_return_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "fixed_asset_return_detail_for_checking" ] = "fixed_asset_return_detail_for_checking";
	$projectEntities[ "Fixed_Asset_Return_for_approval" ] = array( "url" => "Fixed_Asset_Return_for_approval1", "type" => 1 );
	$projectEntitiesReverse[ "Fixed_Asset_Return_for_approval1" ] = "Fixed_Asset_Return_for_approval";
	$projectEntities[ "fixed_asset_return_detail_for_approval" ] = array( "url" => "fixed_asset_return_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "fixed_asset_return_detail_for_approval" ] = "fixed_asset_return_detail_for_approval";
	$projectEntities[ "Petty_Cash_for_checking" ] = array( "url" => "Petty_Cash_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Petty_Cash_for_checking" ] = "Petty_Cash_for_checking";
	$projectEntities[ "petty_cash_detail_for_checking" ] = array( "url" => "petty_cash_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "petty_cash_detail_for_checking" ] = "petty_cash_detail_for_checking";
	$projectEntities[ "Petty_Cash_for_approval" ] = array( "url" => "Petty_Cash_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Petty_Cash_for_approval" ] = "Petty_Cash_for_approval";
	$projectEntities[ "petty_cash_detail_for_approval" ] = array( "url" => "petty_cash_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "petty_cash_detail_for_approval" ] = "petty_cash_detail_for_approval";
	$projectEntities[ "Deposit_for_checking" ] = array( "url" => "Deposit_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Deposit_for_checking" ] = "Deposit_for_checking";
	$projectEntities[ "deposit_detail_for_checking" ] = array( "url" => "deposit_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "deposit_detail_for_checking" ] = "deposit_detail_for_checking";
	$projectEntities[ "Deposit_for_approval" ] = array( "url" => "Deposit_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Deposit_for_approval" ] = "Deposit_for_approval";
	$projectEntities[ "deposit_detail_for_approval" ] = array( "url" => "deposit_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "deposit_detail_for_approval" ] = "deposit_detail_for_approval";
	$projectEntities[ "Credit_Memo_for_checking" ] = array( "url" => "Credit_Memo_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Credit_Memo_for_checking" ] = "Credit_Memo_for_checking";
	$projectEntities[ "credit_memo_detail_for_checking" ] = array( "url" => "credit_memo_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "credit_memo_detail_for_checking" ] = "credit_memo_detail_for_checking";
	$projectEntities[ "Credit_Memo_for_approval" ] = array( "url" => "Credit_Memo_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Credit_Memo_for_approval" ] = "Credit_Memo_for_approval";
	$projectEntities[ "credit_memo_detail_for_approval" ] = array( "url" => "credit_memo_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "credit_memo_detail_for_approval" ] = "credit_memo_detail_for_approval";
	$projectEntities[ "Debit_Memo_for_checking" ] = array( "url" => "Debit_Memo_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Debit_Memo_for_checking" ] = "Debit_Memo_for_checking";
	$projectEntities[ "debit_memo_detail_for_checking" ] = array( "url" => "debit_memo_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "debit_memo_detail_for_checking" ] = "debit_memo_detail_for_checking";
	$projectEntities[ "debit_memo_for_approval" ] = array( "url" => "debit_memo_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "debit_memo_for_approval" ] = "debit_memo_for_approval";
	$projectEntities[ "debit_memo_detail_for_approval" ] = array( "url" => "debit_memo_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "debit_memo_detail_for_approval" ] = "debit_memo_detail_for_approval";
	$projectEntities[ "Purchases_for_checking" ] = array( "url" => "Purchases_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Purchases_for_checking" ] = "Purchases_for_checking";
	$projectEntities[ "credit_purchase_detail_for_checking" ] = array( "url" => "credit_purchase_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "credit_purchase_detail_for_checking" ] = "credit_purchase_detail_for_checking";
	$projectEntities[ "Purchases_for_approval" ] = array( "url" => "Purchases_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Purchases_for_approval" ] = "Purchases_for_approval";
	$projectEntities[ "credit_purchase_detail_for_approval" ] = array( "url" => "credit_purchase_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "credit_purchase_detail_for_approval" ] = "credit_purchase_detail_for_approval";
	$projectEntities[ "Sales_for_checking" ] = array( "url" => "Sales_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Sales_for_checking" ] = "Sales_for_checking";
	$projectEntities[ "Sales_for_approval" ] = array( "url" => "Sales_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Sales_for_approval" ] = "Sales_for_approval";
	$projectEntities[ "credit_sales_detail_for_checking" ] = array( "url" => "credit_sales_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "credit_sales_detail_for_checking" ] = "credit_sales_detail_for_checking";
	$projectEntities[ "credit_sales_detail_for_approval" ] = array( "url" => "credit_sales_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "credit_sales_detail_for_approval" ] = "credit_sales_detail_for_approval";
	$projectEntities[ "Cash_Disbursement_for_checking" ] = array( "url" => "Cash_Disbursement_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Cash_Disbursement_for_checking" ] = "Cash_Disbursement_for_checking";
	$projectEntities[ "check_voucher_detail_for_checking" ] = array( "url" => "check_voucher_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "check_voucher_detail_for_checking" ] = "check_voucher_detail_for_checking";
	$projectEntities[ "check_voucher_detail2_for_checking" ] = array( "url" => "check_voucher_detail2_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "check_voucher_detail2_for_checking" ] = "check_voucher_detail2_for_checking";
	$projectEntities[ "Cash_Disbursement_for_approval" ] = array( "url" => "Cash_Disbursement_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Cash_Disbursement_for_approval" ] = "Cash_Disbursement_for_approval";
	$projectEntities[ "check_voucher_detail_for_approval" ] = array( "url" => "check_voucher_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "check_voucher_detail_for_approval" ] = "check_voucher_detail_for_approval";
	$projectEntities[ "check_voucher_detail2_for_approval" ] = array( "url" => "check_voucher_detail2_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "check_voucher_detail2_for_approval" ] = "check_voucher_detail2_for_approval";
	$projectEntities[ "document" ] = array( "url" => "document", "type" => 0 );
	$projectEntitiesReverse[ "document" ] = "document";
	$projectEntities[ "General_Journal_for_checking" ] = array( "url" => "General_Journal_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "General_Journal_for_checking" ] = "General_Journal_for_checking";
	$projectEntities[ "general_journal_detail_for_checking" ] = array( "url" => "general_journal_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "general_journal_detail_for_checking" ] = "general_journal_detail_for_checking";
	$projectEntities[ "General_Journal_for_approval" ] = array( "url" => "General_Journal_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "General_Journal_for_approval" ] = "General_Journal_for_approval";
	$projectEntities[ "general_journal_detail_for_approval" ] = array( "url" => "general_journal_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "general_journal_detail_for_approval" ] = "general_journal_detail_for_approval";
	$projectEntities[ "for_submissions" ] = array( "url" => "for_submissions", "type" => 0 );
	$projectEntitiesReverse[ "for_submissions" ] = "for_submissions";
	$projectEntities[ "submission_details" ] = array( "url" => "submission_details", "type" => 0 );
	$projectEntitiesReverse[ "submission_details" ] = "submission_details";
	$projectEntities[ "sales_order_detail_billing_dates" ] = array( "url" => "sales_order_detail_billing_dates", "type" => 0 );
	$projectEntitiesReverse[ "sales_order_detail_billing_dates" ] = "sales_order_detail_billing_dates";
	$projectEntities[ "Submission_Search" ] = array( "url" => "Submission_Search", "type" => 1 );
	$projectEntitiesReverse[ "Submission_Search" ] = "Submission_Search";
	$projectEntities[ "Submission_Reminders" ] = array( "url" => "Submission_Reminders", "type" => 1 );
	$projectEntitiesReverse[ "Submission_Reminders" ] = "Submission_Reminders";
	$projectEntities[ "years" ] = array( "url" => "years", "type" => 0 );
	$projectEntitiesReverse[ "years" ] = "years";
	$projectEntities[ "document_type" ] = array( "url" => "document_type", "type" => 0 );
	$projectEntitiesReverse[ "document_type" ] = "document_type";
	$projectEntities[ "estimate" ] = array( "url" => "estimate", "type" => 0 );
	$projectEntitiesReverse[ "estimate" ] = "estimate";
	$projectEntities[ "estimate_detail" ] = array( "url" => "estimate_detail", "type" => 0 );
	$projectEntitiesReverse[ "estimate_detail" ] = "estimate_detail";
	$projectEntities[ "purchase_request" ] = array( "url" => "purchase_request", "type" => 0 );
	$projectEntitiesReverse[ "purchase_request" ] = "purchase_request";
	$projectEntities[ "purchase_request_detail" ] = array( "url" => "purchase_request_detail", "type" => 0 );
	$projectEntitiesReverse[ "purchase_request_detail" ] = "purchase_request_detail";
	$projectEntities[ "inv_adj_detail" ] = array( "url" => "inv_adj_detail", "type" => 0 );
	$projectEntitiesReverse[ "inv_adj_detail" ] = "inv_adj_detail";
	$projectEntities[ "inventory_adj" ] = array( "url" => "inventory_adj", "type" => 0 );
	$projectEntitiesReverse[ "inventory_adj" ] = "inventory_adj";
	$projectEntities[ "inventory_tranfer" ] = array( "url" => "inventory_tranfer", "type" => 0 );
	$projectEntitiesReverse[ "inventory_tranfer" ] = "inventory_tranfer";
	$projectEntities[ "inventory_transfer_detail" ] = array( "url" => "inventory_transfer_detail", "type" => 0 );
	$projectEntitiesReverse[ "inventory_transfer_detail" ] = "inventory_transfer_detail";
	$projectEntities[ "Customers" ] = array( "url" => "Customers", "type" => 1 );
	$projectEntitiesReverse[ "Customers" ] = "Customers";
	$projectEntities[ "Suppliers" ] = array( "url" => "Suppliers", "type" => 1 );
	$projectEntitiesReverse[ "Suppliers" ] = "Suppliers";
	$projectEntities[ "Target_contacts" ] = array( "url" => "Target_contacts", "type" => 1 );
	$projectEntitiesReverse[ "Target_contacts" ] = "Target_contacts";
	$projectEntities[ "Prospect_contacts" ] = array( "url" => "Prospect_contacts", "type" => 1 );
	$projectEntitiesReverse[ "Prospect_contacts" ] = "Prospect_contacts";
	$projectEntities[ "Customer_Contacts" ] = array( "url" => "Customer_Contacts", "type" => 1 );
	$projectEntitiesReverse[ "Customer_Contacts" ] = "Customer_Contacts";
	$projectEntities[ "Supplier_Contacts" ] = array( "url" => "Supplier_Contacts", "type" => 1 );
	$projectEntitiesReverse[ "Supplier_Contacts" ] = "Supplier_Contacts";
	$projectEntities[ "Statement_of_Account" ] = array( "url" => "Statement_of_Account", "type" => 1 );
	$projectEntitiesReverse[ "Statement_of_Account" ] = "Statement_of_Account";
	$projectEntities[ "Inventory_Adjustment_for_checking" ] = array( "url" => "Inventory_Adjustment_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Inventory_Adjustment_for_checking" ] = "Inventory_Adjustment_for_checking";
	$projectEntities[ "inv_adj_detail_for_checking" ] = array( "url" => "inv_adj_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "inv_adj_detail_for_checking" ] = "inv_adj_detail_for_checking";
	$projectEntities[ "Inventory_Adjustment_for_approval" ] = array( "url" => "Inventory_Adjustment_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Inventory_Adjustment_for_approval" ] = "Inventory_Adjustment_for_approval";
	$projectEntities[ "inv_adj_detail_for_approval" ] = array( "url" => "inv_adj_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "inv_adj_detail_for_approval" ] = "inv_adj_detail_for_approval";
	$projectEntities[ "for_checking" ] = array( "url" => "for_checking", "type" => 1 );
	$projectEntitiesReverse[ "for_checking" ] = "for_checking";
	$projectEntities[ "for_approval" ] = array( "url" => "for_approval", "type" => 1 );
	$projectEntitiesReverse[ "for_approval" ] = "for_approval";
	$projectEntities[ "Estimate_for_approval" ] = array( "url" => "Estimate_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Estimate_for_approval" ] = "Estimate_for_approval";
	$projectEntities[ "Estimate_for_checking" ] = array( "url" => "Estimate_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Estimate_for_checking" ] = "Estimate_for_checking";
	$projectEntities[ "estimate_detail_for_checking" ] = array( "url" => "estimate_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "estimate_detail_for_checking" ] = "estimate_detail_for_checking";
	$projectEntities[ "estimate_detail_for_approval" ] = array( "url" => "estimate_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "estimate_detail_for_approval" ] = "estimate_detail_for_approval";
	$projectEntities[ "Follow_up" ] = array( "url" => "Follow_up", "type" => 1 );
	$projectEntitiesReverse[ "Follow_up" ] = "Follow_up";
	$projectEntities[ "coming_meetings" ] = array( "url" => "coming_meetings", "type" => 1 );
	$projectEntitiesReverse[ "coming_meetings" ] = "coming_meetings";
	$projectEntities[ "tasks" ] = array( "url" => "tasks", "type" => 0 );
	$projectEntitiesReverse[ "tasks" ] = "tasks";
	$projectEntities[ "Project_phases" ] = array( "url" => "Project_phases", "type" => 1 );
	$projectEntitiesReverse[ "Project_phases" ] = "Project_phases";
	$projectEntities[ "origin_fa" ] = array( "url" => "origin_fa", "type" => 0 );
	$projectEntitiesReverse[ "origin_fa" ] = "origin_fa";
	$projectEntities[ "origin_serialized_supply" ] = array( "url" => "origin_serialized_supply", "type" => 0 );
	$projectEntitiesReverse[ "origin_serialized_supply" ] = "origin_serialized_supply";
	$projectEntities[ "Prospect_customers_to_Customer" ] = array( "url" => "Prospect_customers_to_Customer", "type" => 1 );
	$projectEntitiesReverse[ "Prospect_customers_to_Customer" ] = "Prospect_customers_to_Customer";
	$projectEntities[ "production_request" ] = array( "url" => "production_request", "type" => 0 );
	$projectEntitiesReverse[ "production_request" ] = "production_request";
	$projectEntities[ "production_request_detail" ] = array( "url" => "production_request_detail", "type" => 0 );
	$projectEntitiesReverse[ "production_request_detail" ] = "production_request_detail";
	$projectEntities[ "department" ] = array( "url" => "department", "type" => 0 );
	$projectEntitiesReverse[ "department" ] = "department";
	$projectEntities[ "journal_entry_cash_disbursement" ] = array( "url" => "journal_entry_cash_disbursement", "type" => 0 );
	$projectEntitiesReverse[ "journal_entry_cash_disbursement" ] = "journal_entry_cash_disbursement";
	$projectEntities[ "journal_entry_cash_receipt" ] = array( "url" => "journal_entry_cash_receipt", "type" => 0 );
	$projectEntitiesReverse[ "journal_entry_cash_receipt" ] = "journal_entry_cash_receipt";
	$projectEntities[ "journal_entry_center_replenishment" ] = array( "url" => "journal_entry_center_replenishment", "type" => 0 );
	$projectEntitiesReverse[ "journal_entry_center_replenishment" ] = "journal_entry_center_replenishment";
	$projectEntities[ "journal_entry_credit_memo" ] = array( "url" => "journal_entry_credit_memo", "type" => 0 );
	$projectEntitiesReverse[ "journal_entry_credit_memo" ] = "journal_entry_credit_memo";
	$projectEntities[ "journal_entry_debit_memo" ] = array( "url" => "journal_entry_debit_memo", "type" => 0 );
	$projectEntitiesReverse[ "journal_entry_debit_memo" ] = "journal_entry_debit_memo";
	$projectEntities[ "journal_entry_depreciation" ] = array( "url" => "journal_entry_depreciation", "type" => 0 );
	$projectEntitiesReverse[ "journal_entry_depreciation" ] = "journal_entry_depreciation";
	$projectEntities[ "journal_entry_general_journal" ] = array( "url" => "journal_entry_general_journal", "type" => 0 );
	$projectEntitiesReverse[ "journal_entry_general_journal" ] = "journal_entry_general_journal";
	$projectEntities[ "journal_entry_inventory_adjustment" ] = array( "url" => "journal_entry_inventory_adjustment", "type" => 0 );
	$projectEntitiesReverse[ "journal_entry_inventory_adjustment" ] = "journal_entry_inventory_adjustment";
	$projectEntities[ "journal_entry_project_replenishment" ] = array( "url" => "journal_entry_project_replenishment", "type" => 0 );
	$projectEntitiesReverse[ "journal_entry_project_replenishment" ] = "journal_entry_project_replenishment";
	$projectEntities[ "journal_entry_purchases" ] = array( "url" => "journal_entry_purchases", "type" => 0 );
	$projectEntitiesReverse[ "journal_entry_purchases" ] = "journal_entry_purchases";
	$projectEntities[ "journal_entry_replenishment" ] = array( "url" => "journal_entry_replenishment", "type" => 0 );
	$projectEntitiesReverse[ "journal_entry_replenishment" ] = "journal_entry_replenishment";
	$projectEntities[ "journal_entry_sales" ] = array( "url" => "journal_entry_sales", "type" => 0 );
	$projectEntitiesReverse[ "journal_entry_sales" ] = "journal_entry_sales";
	$projectEntities[ "journal_entry_service_revenue" ] = array( "url" => "journal_entry_service_revenue", "type" => 0 );
	$projectEntitiesReverse[ "journal_entry_service_revenue" ] = "journal_entry_service_revenue";
	$projectEntities[ "Credit_Card_monitoring" ] = array( "url" => "Credit_Card_monitoring", "type" => 1 );
	$projectEntitiesReverse[ "Credit_Card_monitoring" ] = "Credit_Card_monitoring";
	$projectEntities[ "sales_report_per_item Chart" ] = array( "url" => "sales_report_per_item_Chart", "type" => 3 );
	$projectEntitiesReverse[ "sales_report_per_item_Chart" ] = "sales_report_per_item Chart";
	$projectEntities[ "Purchase_Request_for_approval" ] = array( "url" => "Purchase_Request_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Purchase_Request_for_approval" ] = "Purchase_Request_for_approval";
	$projectEntities[ "purchase_request_detail_approval" ] = array( "url" => "purchase_request_detail_approval", "type" => 1 );
	$projectEntitiesReverse[ "purchase_request_detail_approval" ] = "purchase_request_detail_approval";
	$projectEntities[ "Production_Request_for_approval" ] = array( "url" => "Production_Request_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Production_Request_for_approval" ] = "Production_Request_for_approval";
	$projectEntities[ "production_request_detail_for_approval" ] = array( "url" => "production_request_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "production_request_detail_for_approval" ] = "production_request_detail_for_approval";
	$projectEntities[ "Inventory_Transfer_for_approval" ] = array( "url" => "Inventory_Transfer_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Inventory_Transfer_for_approval" ] = "Inventory_Transfer_for_approval";
	$projectEntities[ "inventory_transfer_detail_for_approval" ] = array( "url" => "inventory_transfer_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "inventory_transfer_detail_for_approval" ] = "inventory_transfer_detail_for_approval";
	$projectEntities[ "Purchase_Request_for_checking" ] = array( "url" => "Purchase_Request_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Purchase_Request_for_checking" ] = "Purchase_Request_for_checking";
	$projectEntities[ "Production_Request_for_checking" ] = array( "url" => "Production_Request_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Production_Request_for_checking" ] = "Production_Request_for_checking";
	$projectEntities[ "Inventory_Transfer_for_checking" ] = array( "url" => "Inventory_Transfer_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Inventory_Transfer_for_checking" ] = "Inventory_Transfer_for_checking";
	$projectEntities[ "inventory_transfer_detail_for_checking" ] = array( "url" => "inventory_transfer_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "inventory_transfer_detail_for_checking" ] = "inventory_transfer_detail_for_checking";
	$projectEntities[ "Backload_for_approval" ] = array( "url" => "Backload_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Backload_for_approval" ] = "Backload_for_approval";
	$projectEntities[ "backload_detail_for_approval" ] = array( "url" => "backload_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "backload_detail_for_approval" ] = "backload_detail_for_approval";
	$projectEntities[ "Backload_for_checking" ] = array( "url" => "Backload_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Backload_for_checking" ] = "Backload_for_checking";
	$projectEntities[ "backload_detail_for_checking" ] = array( "url" => "backload_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "backload_detail_for_checking" ] = "backload_detail_for_checking";
	$projectEntities[ "Cash_Receipt_for_approval" ] = array( "url" => "Cash_Receipt_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Cash_Receipt_for_approval" ] = "Cash_Receipt_for_approval";
	$projectEntities[ "Cash_Receipt_for_checking" ] = array( "url" => "Cash_Receipt_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Cash_Receipt_for_checking" ] = "Cash_Receipt_for_checking";
	$projectEntities[ "cash_receipt_detail_for_checking" ] = array( "url" => "cash_receipt_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "cash_receipt_detail_for_checking" ] = "cash_receipt_detail_for_checking";
	$projectEntities[ "cash_receipt_detail_for_approval" ] = array( "url" => "cash_receipt_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "cash_receipt_detail_for_approval" ] = "cash_receipt_detail_for_approval";
	$projectEntities[ "Service_Report_for_approval" ] = array( "url" => "Service_Report_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Service_Report_for_approval" ] = "Service_Report_for_approval";
	$projectEntities[ "Service_Report_for_checking" ] = array( "url" => "Service_Report_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Service_Report_for_checking" ] = "Service_Report_for_checking";
	$projectEntities[ "service_report_detail_for_approval" ] = array( "url" => "service_report_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "service_report_detail_for_approval" ] = "service_report_detail_for_approval";
	$projectEntities[ "service_report_detail_for_checking" ] = array( "url" => "service_report_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "service_report_detail_for_checking" ] = "service_report_detail_for_checking";
	$projectEntities[ "journal_entry_production" ] = array( "url" => "journal_entry_production", "type" => 0 );
	$projectEntitiesReverse[ "journal_entry_production" ] = "journal_entry_production";
	$projectEntities[ "journal_entry_deposit" ] = array( "url" => "journal_entry_deposit", "type" => 0 );
	$projectEntitiesReverse[ "journal_entry_deposit" ] = "journal_entry_deposit";
	$projectEntities[ "production_request_bom" ] = array( "url" => "production_request_bom", "type" => 0 );
	$projectEntitiesReverse[ "production_request_bom" ] = "production_request_bom";
	$projectEntities[ "production_bom" ] = array( "url" => "production_bom", "type" => 0 );
	$projectEntitiesReverse[ "production_bom" ] = "production_bom";
	$projectEntities[ "company_info" ] = array( "url" => "company_info", "type" => 0 );
	$projectEntitiesReverse[ "company_info" ] = "company_info";
	$projectEntities[ "return" ] = array( "url" => "return", "type" => 0 );
	$projectEntitiesReverse[ "return" ] = "return";
	$projectEntities[ "return_detail" ] = array( "url" => "return_detail", "type" => 0 );
	$projectEntitiesReverse[ "return_detail" ] = "return_detail";
	$projectEntities[ "fixed_asset_issue" ] = array( "url" => "fixed_asset_issue", "type" => 0 );
	$projectEntitiesReverse[ "fixed_asset_issue" ] = "fixed_asset_issue";
	$projectEntities[ "fixed_asset_issue_detail" ] = array( "url" => "fixed_asset_issue_detail", "type" => 0 );
	$projectEntitiesReverse[ "fixed_asset_issue_detail" ] = "fixed_asset_issue_detail";
	$projectEntities[ "production_request_detail_for_checking" ] = array( "url" => "production_request_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "production_request_detail_for_checking" ] = "production_request_detail_for_checking";
	$projectEntities[ "cash_disbursment_detail_for_approval" ] = array( "url" => "cash_disbursment_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "cash_disbursment_detail_for_approval" ] = "cash_disbursment_detail_for_approval";
	$projectEntities[ "cash_disbursment_detail_for_checking" ] = array( "url" => "cash_disbursment_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "cash_disbursment_detail_for_checking" ] = "cash_disbursment_detail_for_checking";
	$projectEntities[ "purchase_request_detail_for_checking" ] = array( "url" => "purchase_request_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "purchase_request_detail_for_checking" ] = "purchase_request_detail_for_checking";
	$projectEntities[ "trip_ticket_for_approval" ] = array( "url" => "trip_ticket_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "trip_ticket_for_approval" ] = "trip_ticket_for_approval";
	$projectEntities[ "trip_ticket_for_checking" ] = array( "url" => "trip_ticket_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "trip_ticket_for_checking" ] = "trip_ticket_for_checking";
	$projectEntities[ "trip_detail_for_approval" ] = array( "url" => "trip_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "trip_detail_for_approval" ] = "trip_detail_for_approval";
	$projectEntities[ "trip_detail_for_checking" ] = array( "url" => "trip_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "trip_detail_for_checking" ] = "trip_detail_for_checking";
	$projectEntities[ "sales_report" ] = array( "url" => "sales_report1", "type" => 2 );
	$projectEntitiesReverse[ "sales_report1" ] = "sales_report";
	$projectEntities[ "Customer_Balance_Report" ] = array( "url" => "Customer_Balance_Report", "type" => 2 );
	$projectEntitiesReverse[ "Customer_Balance_Report" ] = "Customer_Balance_Report";
	$projectEntities[ "Customer_Balance_Report_detail" ] = array( "url" => "Customer_Balance_Report_detail", "type" => 2 );
	$projectEntitiesReverse[ "Customer_Balance_Report_detail" ] = "Customer_Balance_Report_detail";
	$projectEntities[ "Collection_Report_per_Customer" ] = array( "url" => "Collection_Report_per_Customer", "type" => 2 );
	$projectEntitiesReverse[ "Collection_Report_per_Customer" ] = "Collection_Report_per_Customer";
	$projectEntities[ "Aging_of_AR_Report" ] = array( "url" => "Aging_of_AR_Report", "type" => 2 );
	$projectEntitiesReverse[ "Aging_of_AR_Report" ] = "Aging_of_AR_Report";
	$projectEntities[ "Aging_of_AR_Report_detail" ] = array( "url" => "Aging_of_AR_Report_detail", "type" => 2 );
	$projectEntitiesReverse[ "Aging_of_AR_Report_detail" ] = "Aging_of_AR_Report_detail";
	$projectEntities[ "Payment_per_Supplier_Report" ] = array( "url" => "Payment_per_Supplier_Report", "type" => 2 );
	$projectEntitiesReverse[ "Payment_per_Supplier_Report" ] = "Payment_per_Supplier_Report";
	$projectEntities[ "Aging of_AP_Report" ] = array( "url" => "Aging_of_AP_Report", "type" => 2 );
	$projectEntitiesReverse[ "Aging_of_AP_Report" ] = "Aging of_AP_Report";
	$projectEntities[ "Aging of_AP_Report_detail" ] = array( "url" => "Aging_of_AP_Report_detail", "type" => 2 );
	$projectEntitiesReverse[ "Aging_of_AP_Report_detail" ] = "Aging of_AP_Report_detail";
	$projectEntities[ "Inventory_Balance_Report" ] = array( "url" => "Inventory_Balance_Report", "type" => 2 );
	$projectEntitiesReverse[ "Inventory_Balance_Report" ] = "Inventory_Balance_Report";
	$projectEntities[ "Inventory_Valuation_Report" ] = array( "url" => "Inventory_Valuation_Report", "type" => 2 );
	$projectEntitiesReverse[ "Inventory_Valuation_Report" ] = "Inventory_Valuation_Report";
	$projectEntities[ "Inventory_Below_Reorder_Point_Report" ] = array( "url" => "Inventory_Below_Reorder_Point_Report", "type" => 2 );
	$projectEntitiesReverse[ "Inventory_Below_Reorder_Point_Report" ] = "Inventory_Below_Reorder_Point_Report";
	$projectEntities[ "Sales_Report_per_BP" ] = array( "url" => "Sales_Report_per_BP", "type" => 2 );
	$projectEntitiesReverse[ "Sales_Report_per_BP" ] = "Sales_Report_per_BP";
	$projectEntities[ "Sales_Report_per_Item" ] = array( "url" => "Sales_Report_per_Item", "type" => 2 );
	$projectEntitiesReverse[ "Sales_Report_per_Item" ] = "Sales_Report_per_Item";
	$projectEntities[ "Sales_Report_per_Cost_Center" ] = array( "url" => "Sales_Report_per_Cost_Center", "type" => 2 );
	$projectEntitiesReverse[ "Sales_Report_per_Cost_Center" ] = "Sales_Report_per_Cost_Center";
	$projectEntities[ "Sales_Report_per_BP_detail" ] = array( "url" => "Sales_Report_per_BP_detail", "type" => 2 );
	$projectEntitiesReverse[ "Sales_Report_per_BP_detail" ] = "Sales_Report_per_BP_detail";
	$projectEntities[ "Sales_Report_per_Item_detail" ] = array( "url" => "Sales_Report_per_Item_detail", "type" => 2 );
	$projectEntitiesReverse[ "Sales_Report_per_Item_detail" ] = "Sales_Report_per_Item_detail";
	$projectEntities[ "Sales_Report_per_Cost_Center_detail" ] = array( "url" => "Sales_Report_per_Cost_Center_detail", "type" => 2 );
	$projectEntitiesReverse[ "Sales_Report_per_Cost_Center_detail" ] = "Sales_Report_per_Cost_Center_detail";
	$projectEntities[ "Purchase_Report" ] = array( "url" => "Purchase_Report", "type" => 2 );
	$projectEntitiesReverse[ "Purchase_Report" ] = "Purchase_Report";
	$projectEntities[ "Purchase_Report_per_BP" ] = array( "url" => "Purchase_Report_per_BP", "type" => 2 );
	$projectEntitiesReverse[ "Purchase_Report_per_BP" ] = "Purchase_Report_per_BP";
	$projectEntities[ "Purchase_Report_per_Item" ] = array( "url" => "Purchase_Report_per_Item", "type" => 2 );
	$projectEntitiesReverse[ "Purchase_Report_per_Item" ] = "Purchase_Report_per_Item";
	$projectEntities[ "Purchase_Report_per_Cost_Center" ] = array( "url" => "Purchase_Report_per_Cost_Center", "type" => 2 );
	$projectEntitiesReverse[ "Purchase_Report_per_Cost_Center" ] = "Purchase_Report_per_Cost_Center";
	$projectEntities[ "Purchase_Report_per_BP_detail" ] = array( "url" => "Purchase_Report_per_BP_detail", "type" => 2 );
	$projectEntitiesReverse[ "Purchase_Report_per_BP_detail" ] = "Purchase_Report_per_BP_detail";
	$projectEntities[ "Purchase_Report_per_Item_detail" ] = array( "url" => "Purchase_Report_per_Item_detail", "type" => 2 );
	$projectEntitiesReverse[ "Purchase_Report_per_Item_detail" ] = "Purchase_Report_per_Item_detail";
	$projectEntities[ "Purchase_Report_per_Cost_Center_detail" ] = array( "url" => "Purchase_Report_per_Cost_Center_detail", "type" => 2 );
	$projectEntitiesReverse[ "Purchase_Report_per_Cost_Center_detail" ] = "Purchase_Report_per_Cost_Center_detail";
	$projectEntities[ "Sales_Report_per_sales_rep" ] = array( "url" => "Sales_Report_per_sales_rep", "type" => 2 );
	$projectEntitiesReverse[ "Sales_Report_per_sales_rep" ] = "Sales_Report_per_sales_rep";
	$projectEntities[ "PDC_Registry_Report" ] = array( "url" => "PDC_Registry_Report", "type" => 2 );
	$projectEntitiesReverse[ "PDC_Registry_Report" ] = "PDC_Registry_Report";
	$projectEntities[ "Available_Vehicles" ] = array( "url" => "Available_Vehicles1", "type" => 2 );
	$projectEntitiesReverse[ "Available_Vehicles1" ] = "Available_Vehicles";
	$projectEntities[ "Trip_Ticket_Report" ] = array( "url" => "Trip_Ticket_Report", "type" => 2 );
	$projectEntitiesReverse[ "Trip_Ticket_Report" ] = "Trip_Ticket_Report";
	$projectEntities[ "Purchase_Request_Report" ] = array( "url" => "Purchase_Request_Report", "type" => 2 );
	$projectEntitiesReverse[ "Purchase_Request_Report" ] = "Purchase_Request_Report";
	$projectEntities[ "Purchase_Import_Costing" ] = array( "url" => "Purchase_Import_Costing", "type" => 2 );
	$projectEntitiesReverse[ "Purchase_Import_Costing" ] = "Purchase_Import_Costing";
	$projectEntities[ "Production_Costing_Report" ] = array( "url" => "Production_Costing_Report", "type" => 2 );
	$projectEntitiesReverse[ "Production_Costing_Report" ] = "Production_Costing_Report";
	$projectEntities[ "Production_Report" ] = array( "url" => "Production_Report", "type" => 2 );
	$projectEntitiesReverse[ "Production_Report" ] = "Production_Report";
	$projectEntities[ "Closed_Opportunities" ] = array( "url" => "Closed_Opportunities", "type" => 2 );
	$projectEntitiesReverse[ "Closed_Opportunities" ] = "Closed_Opportunities";
	$projectEntities[ "Cases_with_Sotutions" ] = array( "url" => "Cases_with_Sotutions", "type" => 2 );
	$projectEntitiesReverse[ "Cases_with_Sotutions" ] = "Cases_with_Sotutions";
	$projectEntities[ "Totals_Cases_Created" ] = array( "url" => "Totals_Cases_Created", "type" => 2 );
	$projectEntitiesReverse[ "Totals_Cases_Created" ] = "Totals_Cases_Created";
	$projectEntities[ "Totals_Open_Cases_by_Agent" ] = array( "url" => "Totals_Open_Cases_by_Agent", "type" => 2 );
	$projectEntitiesReverse[ "Totals_Open_Cases_by_Agent" ] = "Totals_Open_Cases_by_Agent";
	$projectEntities[ "Campaign_Contacts" ] = array( "url" => "Campaign_Contacts", "type" => 2 );
	$projectEntitiesReverse[ "Campaign_Contacts" ] = "Campaign_Contacts";
	$projectEntities[ "Campaign_leads" ] = array( "url" => "Campaign_leads", "type" => 2 );
	$projectEntitiesReverse[ "Campaign_leads" ] = "Campaign_leads";
	$projectEntities[ "Campaign_ROI_Analysis" ] = array( "url" => "Campaign_ROI_Analysis", "type" => 2 );
	$projectEntitiesReverse[ "Campaign_ROI_Analysis" ] = "Campaign_ROI_Analysis";
	$projectEntities[ "Campaign_Revenue" ] = array( "url" => "Campaign_Revenue", "type" => 2 );
	$projectEntitiesReverse[ "Campaign_Revenue" ] = "Campaign_Revenue";
	$projectEntities[ "Opportunity_Schedule" ] = array( "url" => "Opportunity_Schedule", "type" => 2 );
	$projectEntitiesReverse[ "Opportunity_Schedule" ] = "Opportunity_Schedule";
	$projectEntities[ "Sales_by_Account" ] = array( "url" => "Sales_by_Account", "type" => 2 );
	$projectEntitiesReverse[ "Sales_by_Account" ] = "Sales_by_Account";
	$projectEntities[ "Sales_by_Lead_Source" ] = array( "url" => "Sales_by_Lead_Source", "type" => 2 );
	$projectEntitiesReverse[ "Sales_by_Lead_Source" ] = "Sales_by_Lead_Source";
	$projectEntities[ "Sales_by_Partner" ] = array( "url" => "Sales_by_Partner", "type" => 2 );
	$projectEntitiesReverse[ "Sales_by_Partner" ] = "Sales_by_Partner";
	$projectEntities[ "Sales_by_Rep" ] = array( "url" => "Sales_by_Rep", "type" => 2 );
	$projectEntitiesReverse[ "Sales_by_Rep" ] = "Sales_by_Rep";
	$projectEntities[ "DR_Reports" ] = array( "url" => "DR_Reports", "type" => 2 );
	$projectEntitiesReverse[ "DR_Reports" ] = "DR_Reports";
	$projectEntities[ "Sales_Order_Report" ] = array( "url" => "Sales_Order_Report", "type" => 2 );
	$projectEntitiesReverse[ "Sales_Order_Report" ] = "Sales_Order_Report";
	$projectEntities[ "Case_History" ] = array( "url" => "Case_History", "type" => 2 );
	$projectEntitiesReverse[ "Case_History" ] = "Case_History";
	$projectEntities[ "quality_control" ] = array( "url" => "quality_control", "type" => 0 );
	$projectEntitiesReverse[ "quality_control" ] = "quality_control";
	$projectEntities[ "quality_control_detail" ] = array( "url" => "quality_control_detail", "type" => 0 );
	$projectEntitiesReverse[ "quality_control_detail" ] = "quality_control_detail";
	$projectEntities[ "budget" ] = array( "url" => "budget", "type" => 0 );
	$projectEntitiesReverse[ "budget" ] = "budget";
	$projectEntities[ "budget_detail" ] = array( "url" => "budget_detail", "type" => 0 );
	$projectEntitiesReverse[ "budget_detail" ] = "budget_detail";
	$projectEntities[ "repairs_and_maintenance" ] = array( "url" => "repairs_and_maintenance", "type" => 0 );
	$projectEntitiesReverse[ "repairs_and_maintenance" ] = "repairs_and_maintenance";
	$projectEntities[ "production_form" ] = array( "url" => "production_form", "type" => 0 );
	$projectEntitiesReverse[ "production_form" ] = "production_form";
	$projectEntities[ "rfp" ] = array( "url" => "rfp", "type" => 0 );
	$projectEntitiesReverse[ "rfp" ] = "rfp";
	$projectEntities[ "telex" ] = array( "url" => "telex", "type" => 0 );
	$projectEntitiesReverse[ "telex" ] = "telex";
	$projectEntities[ "repairs_and_maintenance_detail" ] = array( "url" => "repairs_and_maintenance_detail", "type" => 0 );
	$projectEntitiesReverse[ "repairs_and_maintenance_detail" ] = "repairs_and_maintenance_detail";
	$projectEntities[ "repair_list" ] = array( "url" => "repair_list", "type" => 0 );
	$projectEntitiesReverse[ "repair_list" ] = "repair_list";
	$projectEntities[ "marketing_colateral" ] = array( "url" => "marketing_colateral", "type" => 0 );
	$projectEntitiesReverse[ "marketing_colateral" ] = "marketing_colateral";
	$projectEntities[ "email" ] = array( "url" => "email", "type" => 0 );
	$projectEntitiesReverse[ "email" ] = "email";
	$projectEntities[ "accessories" ] = array( "url" => "accessories", "type" => 0 );
	$projectEntitiesReverse[ "accessories" ] = "accessories";
	$projectEntities[ "sales_order_materials_detail" ] = array( "url" => "sales_order_materials_detail", "type" => 0 );
	$projectEntitiesReverse[ "sales_order_materials_detail" ] = "sales_order_materials_detail";
	$projectEntities[ "sales_order_progress_billing_detail" ] = array( "url" => "sales_order_progress_billing_detail", "type" => 0 );
	$projectEntitiesReverse[ "sales_order_progress_billing_detail" ] = "sales_order_progress_billing_detail";
	$projectEntities[ "sales_order" ] = array( "url" => "sales_order", "type" => 0 );
	$projectEntitiesReverse[ "sales_order" ] = "sales_order";
	$projectEntities[ "sales_order_detail" ] = array( "url" => "sales_order_detail", "type" => 0 );
	$projectEntitiesReverse[ "sales_order_detail" ] = "sales_order_detail";
	$projectEntities[ "vat_relief_importation" ] = array( "url" => "vat_relief_importation", "type" => 0 );
	$projectEntitiesReverse[ "vat_relief_importation" ] = "vat_relief_importation";
	$projectEntities[ "vat_relief_purchases" ] = array( "url" => "vat_relief_purchases", "type" => 0 );
	$projectEntitiesReverse[ "vat_relief_purchases" ] = "vat_relief_purchases";
	$projectEntities[ "vat_relief_sales" ] = array( "url" => "vat_relief_sales", "type" => 0 );
	$projectEntitiesReverse[ "vat_relief_sales" ] = "vat_relief_sales";
	$projectEntities[ "journal_ppe_disposal" ] = array( "url" => "journal_ppe_disposal", "type" => 0 );
	$projectEntitiesReverse[ "journal_ppe_disposal" ] = "journal_ppe_disposal";
	$projectEntities[ "credit_memo_detail_cgs" ] = array( "url" => "credit_memo_detail_cgs", "type" => 0 );
	$projectEntitiesReverse[ "credit_memo_detail_cgs" ] = "credit_memo_detail_cgs";
	$projectEntities[ "marketing_request_form" ] = array( "url" => "marketing_request_form", "type" => 0 );
	$projectEntitiesReverse[ "marketing_request_form" ] = "marketing_request_form";
	$projectEntities[ "Quotation_Dashboard_Customer" ] = array( "url" => "Quotation_Dashboard_Customer", "type" => 2 );
	$projectEntitiesReverse[ "Quotation_Dashboard_Customer" ] = "Quotation_Dashboard_Customer";
	$projectEntities[ "Sales_Order_Dashboard_Customer" ] = array( "url" => "Sales_Order_Dashboard_Customer", "type" => 2 );
	$projectEntitiesReverse[ "Sales_Order_Dashboard_Customer" ] = "Sales_Order_Dashboard_Customer";
	$projectEntities[ "Delivery_Receipt_Dashboard_Customer" ] = array( "url" => "Delivery_Receipt_Dashboard_Customer", "type" => 2 );
	$projectEntitiesReverse[ "Delivery_Receipt_Dashboard_Customer" ] = "Delivery_Receipt_Dashboard_Customer";
	$projectEntities[ "Sales_Dashboard_Customer" ] = array( "url" => "Sales_Dashboard_Customer", "type" => 2 );
	$projectEntitiesReverse[ "Sales_Dashboard_Customer" ] = "Sales_Dashboard_Customer";
	$projectEntities[ "Backload_Dashboard_Customer" ] = array( "url" => "Backload_Dashboard_Customer", "type" => 2 );
	$projectEntitiesReverse[ "Backload_Dashboard_Customer" ] = "Backload_Dashboard_Customer";
	$projectEntities[ "Purchase_Request_Dashboard_Supplier" ] = array( "url" => "Purchase_Request_Dashboard_Supplier", "type" => 2 );
	$projectEntitiesReverse[ "Purchase_Request_Dashboard_Supplier" ] = "Purchase_Request_Dashboard_Supplier";
	$projectEntities[ "Purchase_Order_Dashboard_Supplier" ] = array( "url" => "Purchase_Order_Dashboard_Supplier", "type" => 2 );
	$projectEntitiesReverse[ "Purchase_Order_Dashboard_Supplier" ] = "Purchase_Order_Dashboard_Supplier";
	$projectEntities[ "Purchase_Dashboard_Supplier" ] = array( "url" => "Purchase_Dashboard_Supplier", "type" => 2 );
	$projectEntitiesReverse[ "Purchase_Dashboard_Supplier" ] = "Purchase_Dashboard_Supplier";
	$projectEntities[ "Cash_Disbursement_Dashboard_Supplier" ] = array( "url" => "Cash_Disbursement_Dashboard_Supplier", "type" => 2 );
	$projectEntitiesReverse[ "Cash_Disbursement_Dashboard_Supplier" ] = "Cash_Disbursement_Dashboard_Supplier";
	$projectEntities[ "Debit_Memo_Dashboard_Supplier" ] = array( "url" => "Debit_Memo_Dashboard_Supplier", "type" => 2 );
	$projectEntitiesReverse[ "Debit_Memo_Dashboard_Supplier" ] = "Debit_Memo_Dashboard_Supplier";
	$projectEntities[ "Purchase_Request_Dashboard_Item" ] = array( "url" => "Purchase_Request_Dashboard_Item1", "type" => 2 );
	$projectEntitiesReverse[ "Purchase_Request_Dashboard_Item1" ] = "Purchase_Request_Dashboard_Item";
	$projectEntities[ "Purchase_Order_Dashboard_Item" ] = array( "url" => "Purchase_Order_Dashboard_Item", "type" => 2 );
	$projectEntitiesReverse[ "Purchase_Order_Dashboard_Item" ] = "Purchase_Order_Dashboard_Item";
	$projectEntities[ "Purchase_Order_Receiving_Dashboard_Item" ] = array( "url" => "Purchase_Order_Receiving_Dashboard_Item", "type" => 2 );
	$projectEntitiesReverse[ "Purchase_Order_Receiving_Dashboard_Item" ] = "Purchase_Order_Receiving_Dashboard_Item";
	$projectEntities[ "Production_Dashboard_Item" ] = array( "url" => "Production_Dashboard_Item", "type" => 2 );
	$projectEntitiesReverse[ "Production_Dashboard_Item" ] = "Production_Dashboard_Item";
	$projectEntities[ "budget_for_approval" ] = array( "url" => "budget_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "budget_for_approval" ] = "budget_for_approval";
	$projectEntities[ "budget_for_checking" ] = array( "url" => "budget_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "budget_for_checking" ] = "budget_for_checking";
	$projectEntities[ "budget_detail_for_checking" ] = array( "url" => "budget_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "budget_detail_for_checking" ] = "budget_detail_for_checking";
	$projectEntities[ "budget_detail_for_approval" ] = array( "url" => "budget_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "budget_detail_for_approval" ] = "budget_detail_for_approval";
	$projectEntities[ "request_for_payment_for_approval" ] = array( "url" => "request_for_payment_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "request_for_payment_for_approval" ] = "request_for_payment_for_approval";
	$projectEntities[ "request_for_payment_for_checking" ] = array( "url" => "request_for_payment_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "request_for_payment_for_checking" ] = "request_for_payment_for_checking";
	$projectEntities[ "repairs_and_maintenance_detail_for_approval" ] = array( "url" => "repairs_and_maintenance_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "repairs_and_maintenance_detail_for_approval" ] = "repairs_and_maintenance_detail_for_approval";
	$projectEntities[ "request_for_payment_detail_for_approval" ] = array( "url" => "request_for_payment_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "request_for_payment_detail_for_approval" ] = "request_for_payment_detail_for_approval";
	$projectEntities[ "request_for_payment_detail_for_checking" ] = array( "url" => "request_for_payment_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "request_for_payment_detail_for_checking" ] = "request_for_payment_detail_for_checking";
	$projectEntities[ "repairs_and_maintenance_for_approval" ] = array( "url" => "repairs_and_maintenance_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "repairs_and_maintenance_for_approval" ] = "repairs_and_maintenance_for_approval";
	$projectEntities[ "repairs_and_maintenance_for_checking" ] = array( "url" => "repairs_and_maintenance_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "repairs_and_maintenance_for_checking" ] = "repairs_and_maintenance_for_checking";
	$projectEntities[ "repairs_and_maintenance_detail_for_checking" ] = array( "url" => "repairs_and_maintenance_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "repairs_and_maintenance_detail_for_checking" ] = "repairs_and_maintenance_detail_for_checking";
	$projectEntities[ "Disposal_for_checking" ] = array( "url" => "Disposal_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Disposal_for_checking" ] = "Disposal_for_checking";
	$projectEntities[ "Active_campaign_report" ] = array( "url" => "Active_campaign_report", "type" => 2 );
	$projectEntitiesReverse[ "Active_campaign_report" ] = "Active_campaign_report";
	$projectEntities[ "For_Billing_report" ] = array( "url" => "For_Billing_report", "type" => 2 );
	$projectEntitiesReverse[ "For_Billing_report" ] = "For_Billing_report";
	$projectEntities[ "sales_order_for_project_mngmt" ] = array( "url" => "sales_order_for_project_mngmt", "type" => 2 );
	$projectEntitiesReverse[ "sales_order_for_project_mngmt" ] = "sales_order_for_project_mngmt";
	$projectEntities[ "inventory_transfer_cost_detail" ] = array( "url" => "inventory_transfer_cost_detail", "type" => 0 );
	$projectEntitiesReverse[ "inventory_transfer_cost_detail" ] = "inventory_transfer_cost_detail";
	$projectEntities[ "Inventory_per_Cost_center_Report" ] = array( "url" => "Inventory_per_Cost_center_Report", "type" => 2 );
	$projectEntitiesReverse[ "Inventory_per_Cost_center_Report" ] = "Inventory_per_Cost_center_Report";
	$projectEntities[ "Deposit_Report" ] = array( "url" => "Deposit_Report", "type" => 2 );
	$projectEntitiesReverse[ "Deposit_Report" ] = "Deposit_Report";
	$projectEntities[ "SO_for_Billing_Report" ] = array( "url" => "SO_for_Billing_Report", "type" => 2 );
	$projectEntitiesReverse[ "SO_for_Billing_Report" ] = "SO_for_Billing_Report";
	$projectEntities[ "Billing_due_dates_Report" ] = array( "url" => "Billing_due_dates_Report", "type" => 2 );
	$projectEntitiesReverse[ "Billing_due_dates_Report" ] = "Billing_due_dates_Report";
	$projectEntities[ "Packing_List_Report" ] = array( "url" => "Packing_List_Report", "type" => 2 );
	$projectEntitiesReverse[ "Packing_List_Report" ] = "Packing_List_Report";
	$projectEntities[ "Open_Trip_Ticket_Report" ] = array( "url" => "Open_Trip_Ticket_Report", "type" => 2 );
	$projectEntitiesReverse[ "Open_Trip_Ticket_Report" ] = "Open_Trip_Ticket_Report";
	$projectEntities[ "Transfer_Report" ] = array( "url" => "Transfer_Report", "type" => 2 );
	$projectEntitiesReverse[ "Transfer_Report" ] = "Transfer_Report";
	$projectEntities[ "Production_Report_invw" ] = array( "url" => "Production_Report_invw", "type" => 2 );
	$projectEntitiesReverse[ "Production_Report_invw" ] = "Production_Report_invw";
	$projectEntities[ "Supply_per_Department__Report_invw" ] = array( "url" => "Supply_per_Department__Report_invw", "type" => 2 );
	$projectEntitiesReverse[ "Supply_per_Department__Report_invw" ] = "Supply_per_Department__Report_invw";
	$projectEntities[ "Item_Balances_Report_invw" ] = array( "url" => "Item_Balances_Report_invw", "type" => 2 );
	$projectEntitiesReverse[ "Item_Balances_Report_invw" ] = "Item_Balances_Report_invw";
	$projectEntities[ "Open_Case_Report_s" ] = array( "url" => "Open_Case_Report_s", "type" => 2 );
	$projectEntitiesReverse[ "Open_Case_Report_s" ] = "Open_Case_Report_s";
	$projectEntities[ "Sales_Order_per_Campaign_Report_marketing" ] = array( "url" => "Sales_Order_per_Campaign_Report_marketing", "type" => 2 );
	$projectEntitiesReverse[ "Sales_Order_per_Campaign_Report_marketing" ] = "Sales_Order_per_Campaign_Report_marketing";
	$projectEntities[ "Sales_per_campaign_Report_marketing" ] = array( "url" => "Sales_per_campaign_Report_marketing", "type" => 2 );
	$projectEntitiesReverse[ "Sales_per_campaign_Report_marketing" ] = "Sales_per_campaign_Report_marketing";
	$projectEntities[ "Asset_per_Accountability_Report_invw" ] = array( "url" => "Asset_per_Accountability_Report_invw", "type" => 2 );
	$projectEntitiesReverse[ "Asset_per_Accountability_Report_invw" ] = "Asset_per_Accountability_Report_invw";
	$projectEntities[ "Deposit_in_transit_Report" ] = array( "url" => "Deposit_in_transit_Report", "type" => 2 );
	$projectEntitiesReverse[ "Deposit_in_transit_Report" ] = "Deposit_in_transit_Report";
	$projectEntities[ "Outstanding_Checks_Report" ] = array( "url" => "Outstanding_Checks_Report", "type" => 2 );
	$projectEntitiesReverse[ "Outstanding_Checks_Report" ] = "Outstanding_Checks_Report";
	$projectEntities[ "Attendance_Report" ] = array( "url" => "Attendance_Report", "type" => 2 );
	$projectEntitiesReverse[ "Attendance_Report" ] = "Attendance_Report";
	$projectEntities[ "Followup_Report" ] = array( "url" => "Followup_Report", "type" => 2 );
	$projectEntitiesReverse[ "Followup_Report" ] = "Followup_Report";
	$projectEntities[ "Coming_Meeting_Report" ] = array( "url" => "Coming_Meeting_Report", "type" => 2 );
	$projectEntitiesReverse[ "Coming_Meeting_Report" ] = "Coming_Meeting_Report";
	$projectEntities[ "Meeting_Report" ] = array( "url" => "Meeting_Report", "type" => 2 );
	$projectEntitiesReverse[ "Meeting_Report" ] = "Meeting_Report";
	$projectEntities[ "Next_Action_Report" ] = array( "url" => "Next_Action_Report", "type" => 2 );
	$projectEntitiesReverse[ "Next_Action_Report" ] = "Next_Action_Report";
	$projectEntities[ "Calls_Report" ] = array( "url" => "Calls_Report", "type" => 2 );
	$projectEntitiesReverse[ "Calls_Report" ] = "Calls_Report";
	$projectEntities[ "sales_per_sales_rep_Report" ] = array( "url" => "sales_per_sales_rep_Report", "type" => 2 );
	$projectEntitiesReverse[ "sales_per_sales_rep_Report" ] = "sales_per_sales_rep_Report";
	$projectEntities[ "Withholding_Tax_Collection_List_per_Month_Report" ] = array( "url" => "Withholding_Tax_Collection_List_per_Month_Report", "type" => 2 );
	$projectEntitiesReverse[ "Withholding_Tax_Collection_List_per_Month_Report" ] = "Withholding_Tax_Collection_List_per_Month_Report";
	$projectEntities[ "Business_Owner_Dashboard" ] = array( "url" => "Business_Owner_Dashboard", "type" => 4 );
	$projectEntitiesReverse[ "Business_Owner_Dashboard" ] = "Business_Owner_Dashboard";
	$projectEntities[ "Finance_Dashboard" ] = array( "url" => "Finance_Dashboard", "type" => 4 );
	$projectEntitiesReverse[ "Finance_Dashboard" ] = "Finance_Dashboard";
	$projectEntities[ "Purchasing_Dashboard" ] = array( "url" => "Purchasing_Dashboard", "type" => 4 );
	$projectEntitiesReverse[ "Purchasing_Dashboard" ] = "Purchasing_Dashboard";
	$projectEntities[ "Production_Dashboard" ] = array( "url" => "Production_Dashboard1", "type" => 4 );
	$projectEntitiesReverse[ "Production_Dashboard1" ] = "Production_Dashboard";
	$projectEntities[ "Marketing_Dashboard" ] = array( "url" => "Marketing_Dashboard1", "type" => 4 );
	$projectEntitiesReverse[ "Marketing_Dashboard1" ] = "Marketing_Dashboard";
	$projectEntities[ "Support_Dashboard" ] = array( "url" => "Support_Dashboard", "type" => 4 );
	$projectEntitiesReverse[ "Support_Dashboard" ] = "Support_Dashboard";
	$projectEntities[ "Sales_Dashboard" ] = array( "url" => "Sales_Dashboard", "type" => 4 );
	$projectEntitiesReverse[ "Sales_Dashboard" ] = "Sales_Dashboard";
	$projectEntities[ "Project_Manager_Dashboard" ] = array( "url" => "Project_Manager_Dashboard", "type" => 4 );
	$projectEntitiesReverse[ "Project_Manager_Dashboard" ] = "Project_Manager_Dashboard";
	$projectEntities[ "System_Admistration_Dashboard" ] = array( "url" => "System_Admistration_Dashboard", "type" => 4 );
	$projectEntitiesReverse[ "System_Admistration_Dashboard" ] = "System_Admistration_Dashboard";
	$projectEntities[ "purchase_report_per_item Chart" ] = array( "url" => "purchase_report_per_item_Chart", "type" => 3 );
	$projectEntitiesReverse[ "purchase_report_per_item_Chart" ] = "purchase_report_per_item Chart";
	$projectEntities[ "System_Audit_Report" ] = array( "url" => "System_Audit_Report", "type" => 2 );
	$projectEntitiesReverse[ "System_Audit_Report" ] = "System_Audit_Report";
	$projectEntities[ "sale_cost_detail" ] = array( "url" => "sale_cost_detail", "type" => 0 );
	$projectEntitiesReverse[ "sale_cost_detail" ] = "sale_cost_detail";
	$projectEntities[ "currency_exchange_rate" ] = array( "url" => "currency_exchange_rate", "type" => 0 );
	$projectEntitiesReverse[ "currency_exchange_rate" ] = "currency_exchange_rate";
	$projectEntities[ "cash_receipt_cost_detail" ] = array( "url" => "cash_receipt_cost_detail", "type" => 0 );
	$projectEntitiesReverse[ "cash_receipt_cost_detail" ] = "cash_receipt_cost_detail";
	$projectEntities[ "production_cost_detail" ] = array( "url" => "production_cost_detail", "type" => 0 );
	$projectEntitiesReverse[ "production_cost_detail" ] = "production_cost_detail";
	$projectEntities[ "Employee" ] = array( "url" => "Employee", "type" => 4 );
	$projectEntitiesReverse[ "Employee" ] = "Employee";
	$projectEntities[ "Quotation_Dashboard" ] = array( "url" => "Quotation_Dashboard", "type" => 2 );
	$projectEntitiesReverse[ "Quotation_Dashboard" ] = "Quotation_Dashboard";
	$projectEntities[ "Sales_Order_Dashboard" ] = array( "url" => "Sales_Order_Dashboard", "type" => 2 );
	$projectEntitiesReverse[ "Sales_Order_Dashboard" ] = "Sales_Order_Dashboard";
	$projectEntities[ "Reimbursement_Dashboard" ] = array( "url" => "Reimbursement_Dashboard", "type" => 2 );
	$projectEntitiesReverse[ "Reimbursement_Dashboard" ] = "Reimbursement_Dashboard";
	$projectEntities[ "sum_dr_qty" ] = array( "url" => "sum_dr_qty", "type" => 0 );
	$projectEntitiesReverse[ "sum_dr_qty" ] = "sum_dr_qty";
	$projectEntities[ "check_book_registry" ] = array( "url" => "check_book_registry", "type" => 0 );
	$projectEntitiesReverse[ "check_book_registry" ] = "check_book_registry";
	$projectEntities[ "check_book_registry_detail" ] = array( "url" => "check_book_registry_detail", "type" => 0 );
	$projectEntitiesReverse[ "check_book_registry_detail" ] = "check_book_registry_detail";
	$projectEntities[ "request_for_payment" ] = array( "url" => "request_for_payment", "type" => 0 );
	$projectEntitiesReverse[ "request_for_payment" ] = "request_for_payment";
	$projectEntities[ "request_for_payment_detail" ] = array( "url" => "request_for_payment_detail", "type" => 0 );
	$projectEntitiesReverse[ "request_for_payment_detail" ] = "request_for_payment_detail";
	$projectEntities[ "vat_type" ] = array( "url" => "vat_type", "type" => 0 );
	$projectEntitiesReverse[ "vat_type" ] = "vat_type";
	$projectEntities[ "quotation_detail" ] = array( "url" => "quotation_detail", "type" => 0 );
	$projectEntitiesReverse[ "quotation_detail" ] = "quotation_detail";
	$projectEntities[ "bal_purchase_order_detail" ] = array( "url" => "bal_purchase_order_detail", "type" => 0 );
	$projectEntitiesReverse[ "bal_purchase_order_detail" ] = "bal_purchase_order_detail";
	$projectEntities[ "bal_sales_detail" ] = array( "url" => "bal_sales_detail", "type" => 0 );
	$projectEntitiesReverse[ "bal_sales_detail" ] = "bal_sales_detail";
	$projectEntities[ "pending_items_reminders" ] = array( "url" => "pending_items_reminders", "type" => 2 );
	$projectEntitiesReverse[ "pending_items_reminders" ] = "pending_items_reminders";
	$projectEntities[ "petty_cash_for_liquidation" ] = array( "url" => "petty_cash_for_liquidation", "type" => 2 );
	$projectEntitiesReverse[ "petty_cash_for_liquidation" ] = "petty_cash_for_liquidation";
	$projectEntities[ "payment_method" ] = array( "url" => "payment_method", "type" => 0 );
	$projectEntitiesReverse[ "payment_method" ] = "payment_method";
	$projectEntities[ "account_type" ] = array( "url" => "account_type", "type" => 0 );
	$projectEntitiesReverse[ "account_type" ] = "account_type";
	$projectEntities[ "phase" ] = array( "url" => "phase", "type" => 0 );
	$projectEntitiesReverse[ "phase" ] = "phase";
	$projectEntities[ "item_cost_recalculation" ] = array( "url" => "item_cost_recalculation", "type" => 0 );
	$projectEntitiesReverse[ "item_cost_recalculation" ] = "item_cost_recalculation";
	$projectEntities[ "issue_components" ] = array( "url" => "issue_components", "type" => 0 );
	$projectEntitiesReverse[ "issue_components" ] = "issue_components";
	$projectEntities[ "issue_components_detail" ] = array( "url" => "issue_components_detail", "type" => 0 );
	$projectEntitiesReverse[ "issue_components_detail" ] = "issue_components_detail";
	$projectEntities[ "demand_plan" ] = array( "url" => "demand_plan", "type" => 0 );
	$projectEntitiesReverse[ "demand_plan" ] = "demand_plan";
	$projectEntities[ "input_vat_allocation" ] = array( "url" => "input_vat_allocation", "type" => 0 );
	$projectEntitiesReverse[ "input_vat_allocation" ] = "input_vat_allocation";
	$projectEntities[ "case_type" ] = array( "url" => "case_type", "type" => 0 );
	$projectEntitiesReverse[ "case_type" ] = "case_type";
	$projectEntities[ "qc_inspection" ] = array( "url" => "qc_inspection", "type" => 0 );
	$projectEntitiesReverse[ "qc_inspection" ] = "qc_inspection";
	$projectEntities[ "packing_list_for_checking" ] = array( "url" => "packing_list_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "packing_list_for_checking" ] = "packing_list_for_checking";
	$projectEntities[ "packing_list_for_approval" ] = array( "url" => "packing_list_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "packing_list_for_approval" ] = "packing_list_for_approval";
	$projectEntities[ "packing_list_detail_for_checking" ] = array( "url" => "packing_list_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "packing_list_detail_for_checking" ] = "packing_list_detail_for_checking";
	$projectEntities[ "packing_list_detail_for_approval" ] = array( "url" => "packing_list_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "packing_list_detail_for_approval" ] = "packing_list_detail_for_approval";
	$projectEntities[ "disposed_inv_detail" ] = array( "url" => "disposed_inv_detail", "type" => 0 );
	$projectEntitiesReverse[ "disposed_inv_detail" ] = "disposed_inv_detail";
	$projectEntities[ "disposed_inv" ] = array( "url" => "disposed_inv", "type" => 0 );
	$projectEntitiesReverse[ "disposed_inv" ] = "disposed_inv";
	$projectEntities[ "disposed_inv_for_checking" ] = array( "url" => "disposed_inv_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "disposed_inv_for_checking" ] = "disposed_inv_for_checking";
	$projectEntities[ "disposed_inv_for_approval" ] = array( "url" => "disposed_inv_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "disposed_inv_for_approval" ] = "disposed_inv_for_approval";
	$projectEntities[ "disposed_inv_detail_for_checking" ] = array( "url" => "disposed_inv_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "disposed_inv_detail_for_checking" ] = "disposed_inv_detail_for_checking";
	$projectEntities[ "disposed_inv_detail_for_approval" ] = array( "url" => "disposed_inv_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "disposed_inv_detail_for_approval" ] = "disposed_inv_detail_for_approval";
	$projectEntities[ "For_Approval_Dashboard" ] = array( "url" => "For_Approval_Dashboard", "type" => 2 );
	$projectEntitiesReverse[ "For_Approval_Dashboard" ] = "For_Approval_Dashboard";
	$projectEntities[ "For_Checking_Dashboard" ] = array( "url" => "For_Checking_Dashboard", "type" => 2 );
	$projectEntitiesReverse[ "For_Checking_Dashboard" ] = "For_Checking_Dashboard";
	$projectEntities[ "Customer_For_Approval" ] = array( "url" => "Customer_For_Approval", "type" => 2 );
	$projectEntitiesReverse[ "Customer_For_Approval" ] = "Customer_For_Approval";
	$projectEntities[ "request_for_payment_report" ] = array( "url" => "request_for_payment_report", "type" => 2 );
	$projectEntitiesReverse[ "request_for_payment_report" ] = "request_for_payment_report";
	$projectEntities[ "credit_memo_report" ] = array( "url" => "credit_memo_report", "type" => 2 );
	$projectEntitiesReverse[ "credit_memo_report" ] = "credit_memo_report";
	$projectEntities[ "debit_memo_report" ] = array( "url" => "debit_memo_report", "type" => 2 );
	$projectEntitiesReverse[ "debit_memo_report" ] = "debit_memo_report";
	$projectEntities[ "production_request_report" ] = array( "url" => "production_request_report", "type" => 2 );
	$projectEntitiesReverse[ "production_request_report" ] = "production_request_report";
	$projectEntities[ "Supplies_Balance_report" ] = array( "url" => "Supplies_Balance_report", "type" => 2 );
	$projectEntitiesReverse[ "Supplies_Balance_report" ] = "Supplies_Balance_report";
	$projectEntities[ "approved_sales_order" ] = array( "url" => "approved_sales_order", "type" => 2 );
	$projectEntitiesReverse[ "approved_sales_order" ] = "approved_sales_order";
	$projectEntities[ "purchase_order_for_receiving" ] = array( "url" => "purchase_order_for_receiving", "type" => 2 );
	$projectEntitiesReverse[ "purchase_order_for_receiving" ] = "purchase_order_for_receiving";
	$projectEntities[ "Dashboard" ] = array( "url" => "Dashboard", "type" => 4 );
	$projectEntitiesReverse[ "Dashboard" ] = "Dashboard";
	$projectEntities[ "packing_list" ] = array( "url" => "packing_list", "type" => 0 );
	$projectEntitiesReverse[ "packing_list" ] = "packing_list";
	$projectEntities[ "packing_list_detail" ] = array( "url" => "packing_list_detail", "type" => 0 );
	$projectEntitiesReverse[ "packing_list_detail" ] = "packing_list_detail";
	$projectEntities[ "open_quote" ] = array( "url" => "open_quote", "type" => 0 );
	$projectEntitiesReverse[ "open_quote" ] = "open_quote";
	$projectEntities[ "open_estimates" ] = array( "url" => "open_estimates", "type" => 0 );
	$projectEntitiesReverse[ "open_estimates" ] = "open_estimates";
	$projectEntities[ "open_po" ] = array( "url" => "open_po", "type" => 0 );
	$projectEntitiesReverse[ "open_po" ] = "open_po";
	$projectEntities[ "open_pr" ] = array( "url" => "open_pr", "type" => 0 );
	$projectEntitiesReverse[ "open_pr" ] = "open_pr";
	$projectEntities[ "open_production_request" ] = array( "url" => "open_production_request", "type" => 0 );
	$projectEntitiesReverse[ "open_production_request" ] = "open_production_request";
	$projectEntities[ "open_qc" ] = array( "url" => "open_qc", "type" => 0 );
	$projectEntitiesReverse[ "open_qc" ] = "open_qc";
	$projectEntities[ "open_receiving" ] = array( "url" => "open_receiving", "type" => 0 );
	$projectEntitiesReverse[ "open_receiving" ] = "open_receiving";
	$projectEntities[ "open_request_for_payment" ] = array( "url" => "open_request_for_payment", "type" => 0 );
	$projectEntitiesReverse[ "open_request_for_payment" ] = "open_request_for_payment";
	$projectEntities[ "open_so" ] = array( "url" => "open_so", "type" => 0 );
	$projectEntitiesReverse[ "open_so" ] = "open_so";
	$projectEntities[ "open_so_for_invoice" ] = array( "url" => "open_so_for_invoice", "type" => 0 );
	$projectEntitiesReverse[ "open_so_for_invoice" ] = "open_so_for_invoice";
	$projectEntities[ "open_so_for_project_management" ] = array( "url" => "open_so_for_project_management", "type" => 0 );
	$projectEntitiesReverse[ "open_so_for_project_management" ] = "open_so_for_project_management";
	$projectEntities[ "open_backload" ] = array( "url" => "open_backload", "type" => 0 );
	$projectEntitiesReverse[ "open_backload" ] = "open_backload";
	$projectEntities[ "purchase_order_receiving_detail" ] = array( "url" => "purchase_order_receiving_detail", "type" => 0 );
	$projectEntitiesReverse[ "purchase_order_receiving_detail" ] = "purchase_order_receiving_detail";
	$projectEntities[ "Purchase_Order_Receiving_Dashboard_Supplier" ] = array( "url" => "Purchase_Order_Receiving_Dashboard_Supplier1", "type" => 2 );
	$projectEntitiesReverse[ "Purchase_Order_Receiving_Dashboard_Supplier1" ] = "Purchase_Order_Receiving_Dashboard_Supplier";
	$projectEntities[ "avg_inv" ] = array( "url" => "avg_inv", "type" => 1 );
	$projectEntitiesReverse[ "avg_inv" ] = "avg_inv";
	$projectEntities[ "open_backload_Report" ] = array( "url" => "open_backload_Report", "type" => 2 );
	$projectEntitiesReverse[ "open_backload_Report" ] = "open_backload_Report";
	$projectEntities[ "open_estimates_Report" ] = array( "url" => "open_estimates_Report", "type" => 2 );
	$projectEntitiesReverse[ "open_estimates_Report" ] = "open_estimates_Report";
	$projectEntities[ "open_po_Report" ] = array( "url" => "open_po_Report", "type" => 2 );
	$projectEntitiesReverse[ "open_po_Report" ] = "open_po_Report";
	$projectEntities[ "open_pr_Report" ] = array( "url" => "open_pr_Report", "type" => 2 );
	$projectEntitiesReverse[ "open_pr_Report" ] = "open_pr_Report";
	$projectEntities[ "open_production_request_Report" ] = array( "url" => "open_production_request_Report", "type" => 2 );
	$projectEntitiesReverse[ "open_production_request_Report" ] = "open_production_request_Report";
	$projectEntities[ "open_qc_Report" ] = array( "url" => "open_qc_Report", "type" => 2 );
	$projectEntitiesReverse[ "open_qc_Report" ] = "open_qc_Report";
	$projectEntities[ "open_so_for_project_management_Report" ] = array( "url" => "open_so_for_project_management_Report", "type" => 2 );
	$projectEntitiesReverse[ "open_so_for_project_management_Report" ] = "open_so_for_project_management_Report";
	$projectEntities[ "open_so_for_invoice_Report" ] = array( "url" => "open_so_for_invoice_Report", "type" => 2 );
	$projectEntitiesReverse[ "open_so_for_invoice_Report" ] = "open_so_for_invoice_Report";
	$projectEntities[ "open_so_Report" ] = array( "url" => "open_so_Report", "type" => 2 );
	$projectEntitiesReverse[ "open_so_Report" ] = "open_so_Report";
	$projectEntities[ "open_receiving_Report" ] = array( "url" => "open_receiving_Report", "type" => 2 );
	$projectEntitiesReverse[ "open_receiving_Report" ] = "open_receiving_Report";
	$projectEntities[ "monthly_consuption_for_production_Report" ] = array( "url" => "monthly_consuption_for_production_Report", "type" => 2 );
	$projectEntitiesReverse[ "monthly_consuption_for_production_Report" ] = "monthly_consuption_for_production_Report";
	$projectEntities[ "issue_components_for_approval" ] = array( "url" => "issue_components_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "issue_components_for_approval" ] = "issue_components_for_approval";
	$projectEntities[ "issue_components_for_checking" ] = array( "url" => "issue_components_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "issue_components_for_checking" ] = "issue_components_for_checking";
	$projectEntities[ "issue_components_detail_for_checking" ] = array( "url" => "issue_components_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "issue_components_detail_for_checking" ] = "issue_components_detail_for_checking";
	$projectEntities[ "issue_components_detail_for_approval" ] = array( "url" => "issue_components_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "issue_components_detail_for_approval" ] = "issue_components_detail_for_approval";
	$projectEntities[ "Purchases_Order_for_checking" ] = array( "url" => "Purchases_Order_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Purchases_Order_for_checking" ] = "Purchases_Order_for_checking";
	$projectEntities[ "Purchases_Order_for_approval" ] = array( "url" => "Purchases_Order_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Purchases_Order_for_approval" ] = "Purchases_Order_for_approval";
	$projectEntities[ "Receiving_for_checking" ] = array( "url" => "Receiving_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Receiving_for_checking" ] = "Receiving_for_checking";
	$projectEntities[ "Receiving_for_approval" ] = array( "url" => "Receiving_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Receiving_for_approval" ] = "Receiving_for_approval";
	$projectEntities[ "inventory_tranfer_Report" ] = array( "url" => "inventory_tranfer_Report", "type" => 2 );
	$projectEntitiesReverse[ "inventory_tranfer_Report" ] = "inventory_tranfer_Report";
	$projectEntities[ "Purchase_vs_Production_Report" ] = array( "url" => "Purchase_vs_Production_Report", "type" => 2 );
	$projectEntitiesReverse[ "Purchase_vs_Production_Report" ] = "Purchase_vs_Production_Report";
	$projectEntities[ "cash_sales_Report" ] = array( "url" => "cash_sales_Report", "type" => 2 );
	$projectEntitiesReverse[ "cash_sales_Report" ] = "cash_sales_Report";
	$projectEntities[ "Center_Replenishment_for_approval" ] = array( "url" => "Center_Replenishment_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Center_Replenishment_for_approval" ] = "Center_Replenishment_for_approval";
	$projectEntities[ "Center_Replenishment_for_checking" ] = array( "url" => "Center_Replenishment_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Center_Replenishment_for_checking" ] = "Center_Replenishment_for_checking";
	$projectEntities[ "Purchase_vs_Production" ] = array( "url" => "Purchase_vs_Production", "type" => 1 );
	$projectEntitiesReverse[ "Purchase_vs_Production" ] = "Purchase_vs_Production";
	$projectEntities[ "Production_Summary" ] = array( "url" => "Production_Summary", "type" => 2 );
	$projectEntitiesReverse[ "Production_Summary" ] = "Production_Summary";
	$projectEntities[ "pending_production_request" ] = array( "url" => "pending_production_request", "type" => 2 );
	$projectEntitiesReverse[ "pending_production_request" ] = "pending_production_request";
	$projectEntities[ "quality_control_for_approval" ] = array( "url" => "quality_control_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "quality_control_for_approval" ] = "quality_control_for_approval";
	$projectEntities[ "quality_control_for_checking" ] = array( "url" => "quality_control_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "quality_control_for_checking" ] = "quality_control_for_checking";
	$projectEntities[ "quality_control_detail_for_approval" ] = array( "url" => "quality_control_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "quality_control_detail_for_approval" ] = "quality_control_detail_for_approval";
	$projectEntities[ "raw_cake_balance" ] = array( "url" => "raw_cake_balance", "type" => 2 );
	$projectEntitiesReverse[ "raw_cake_balance" ] = "raw_cake_balance";
	$projectEntities[ "bal_cashreceipt" ] = array( "url" => "bal_cashreceipt", "type" => 0 );
	$projectEntitiesReverse[ "bal_cashreceipt" ] = "bal_cashreceipt";
	$projectEntities[ "opportunity" ] = array( "url" => "opportunity", "type" => 0 );
	$projectEntitiesReverse[ "opportunity" ] = "opportunity";
	$projectEntities[ "Current_Accounts" ] = array( "url" => "Current_Accounts", "type" => 1 );
	$projectEntitiesReverse[ "Current_Accounts" ] = "Current_Accounts";
	$projectEntities[ "opportunity_details" ] = array( "url" => "opportunity_details", "type" => 1 );
	$projectEntitiesReverse[ "opportunity_details" ] = "opportunity_details";
	$projectEntities[ "Collection_Report_per_Sales_Rep" ] = array( "url" => "Collection_Report_per_Sales_Rep", "type" => 2 );
	$projectEntitiesReverse[ "Collection_Report_per_Sales_Rep" ] = "Collection_Report_per_Sales_Rep";
	$projectEntities[ "Inventory_per_lot_no" ] = array( "url" => "Inventory_per_lot_no", "type" => 2 );
	$projectEntitiesReverse[ "Inventory_per_lot_no" ] = "Inventory_per_lot_no";
	$projectEntities[ "marketing_request_form_for_checking" ] = array( "url" => "marketing_request_form_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "marketing_request_form_for_checking" ] = "marketing_request_form_for_checking";
	$projectEntities[ "marketing_request_form_for_approval" ] = array( "url" => "marketing_request_form_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "marketing_request_form_for_approval" ] = "marketing_request_form_for_approval";
	$projectEntities[ "Accounts_Payable_Dashboard" ] = array( "url" => "Accounts_Payable_Dashboard", "type" => 4 );
	$projectEntitiesReverse[ "Accounts_Payable_Dashboard" ] = "Accounts_Payable_Dashboard";
	$projectEntities[ "Cashflow_forcast" ] = array( "url" => "Cashflow_forcast", "type" => 2 );
	$projectEntitiesReverse[ "Cashflow_forcast" ] = "Cashflow_forcast";
	$projectEntities[ "Business_Owner_Sales_Report" ] = array( "url" => "Business_Owner_Sales_Report", "type" => 2 );
	$projectEntitiesReverse[ "Business_Owner_Sales_Report" ] = "Business_Owner_Sales_Report";
	$projectEntities[ "Business_Owner_Purchase_Report" ] = array( "url" => "Business_Owner_Purchase_Report", "type" => 2 );
	$projectEntitiesReverse[ "Business_Owner_Purchase_Report" ] = "Business_Owner_Purchase_Report";
	$projectEntities[ "AR_Billing_due_dates_Report" ] = array( "url" => "AR_Billing_due_dates_Report", "type" => 2 );
	$projectEntitiesReverse[ "AR_Billing_due_dates_Report" ] = "AR_Billing_due_dates_Report";
	$projectEntities[ "AR_Aging_of_AR_Report" ] = array( "url" => "AR_Aging_of_AR_Report", "type" => 2 );
	$projectEntitiesReverse[ "AR_Aging_of_AR_Report" ] = "AR_Aging_of_AR_Report";
	$projectEntities[ "AR_Top_5_Customer_balances" ] = array( "url" => "AR_Top_5_Customer_balances", "type" => 3 );
	$projectEntitiesReverse[ "AR_Top_5_Customer_balances" ] = "AR_Top_5_Customer_balances";
	$projectEntities[ "AR_Progress_Billing_Alert" ] = array( "url" => "AR_Progress_Billing_Alert", "type" => 2 );
	$projectEntitiesReverse[ "AR_Progress_Billing_Alert" ] = "AR_Progress_Billing_Alert";
	$projectEntities[ "Account_Receivable_Dashboard" ] = array( "url" => "Account_Receivable_Dashboard", "type" => 4 );
	$projectEntitiesReverse[ "Account_Receivable_Dashboard" ] = "Account_Receivable_Dashboard";
	$projectEntities[ "Finance_approved_request_for_payment" ] = array( "url" => "Finance_approved_request_for_payment", "type" => 2 );
	$projectEntitiesReverse[ "Finance_approved_request_for_payment" ] = "Finance_approved_request_for_payment";
	$projectEntities[ "Finance_Cash_flow_forcast" ] = array( "url" => "Finance_Cash_flow_forcast", "type" => 2 );
	$projectEntitiesReverse[ "Finance_Cash_flow_forcast" ] = "Finance_Cash_flow_forcast";
	$projectEntities[ "Finance_Approved_Employee_Loan" ] = array( "url" => "Finance_Approved_Employee_Loan", "type" => 2 );
	$projectEntitiesReverse[ "Finance_Approved_Employee_Loan" ] = "Finance_Approved_Employee_Loan";
	$projectEntities[ "AP_Aging_of_Payables" ] = array( "url" => "AP_Aging_of_Payables", "type" => 2 );
	$projectEntitiesReverse[ "AP_Aging_of_Payables" ] = "AP_Aging_of_Payables";
	$projectEntities[ "Purchasing_Item_below_reorder" ] = array( "url" => "Purchasing_Item_below_reorder", "type" => 2 );
	$projectEntitiesReverse[ "Purchasing_Item_below_reorder" ] = "Purchasing_Item_below_reorder";
	$projectEntities[ "production_per_item" ] = array( "url" => "production_per_item", "type" => 3 );
	$projectEntitiesReverse[ "production_per_item" ] = "production_per_item";
	$projectEntities[ "AP_Top_5_Supplier_balances" ] = array( "url" => "AP_Top_5_Supplier_balances", "type" => 3 );
	$projectEntitiesReverse[ "AP_Top_5_Supplier_balances" ] = "AP_Top_5_Supplier_balances";
	$projectEntities[ "Warehouse_Item_balances" ] = array( "url" => "Warehouse_Item_balances", "type" => 3 );
	$projectEntitiesReverse[ "Warehouse_Item_balances" ] = "Warehouse_Item_balances";
	$projectEntities[ "DR_per_Item" ] = array( "url" => "DR_per_Item", "type" => 3 );
	$projectEntitiesReverse[ "DR_per_Item" ] = "DR_per_Item";
	$projectEntities[ "Logistic_Open_Trip_Ticket" ] = array( "url" => "Logistic_Open_Trip_Ticket", "type" => 2 );
	$projectEntitiesReverse[ "Logistic_Open_Trip_Ticket" ] = "Logistic_Open_Trip_Ticket";
	$projectEntities[ "Logistic_Available_Vehicles" ] = array( "url" => "Logistic_Available_Vehicles", "type" => 2 );
	$projectEntitiesReverse[ "Logistic_Available_Vehicles" ] = "Logistic_Available_Vehicles";
	$projectEntities[ "Marketing_Open_Campaign" ] = array( "url" => "Marketing_Open_Campaign", "type" => 2 );
	$projectEntitiesReverse[ "Marketing_Open_Campaign" ] = "Marketing_Open_Campaign";
	$projectEntities[ "Marketing_Sales_per_campaign" ] = array( "url" => "Marketing_Sales_per_campaign", "type" => 2 );
	$projectEntitiesReverse[ "Marketing_Sales_per_campaign" ] = "Marketing_Sales_per_campaign";
	$projectEntities[ "Support_Open_Case_Report" ] = array( "url" => "Support_Open_Case_Report", "type" => 2 );
	$projectEntitiesReverse[ "Support_Open_Case_Report" ] = "Support_Open_Case_Report";
	$projectEntities[ "Support_Service_center" ] = array( "url" => "Support_Service_center", "type" => 2 );
	$projectEntitiesReverse[ "Support_Service_center" ] = "Support_Service_center";
	$projectEntities[ "Sale_Meeting_Report" ] = array( "url" => "Sale_Meeting_Report", "type" => 2 );
	$projectEntitiesReverse[ "Sale_Meeting_Report" ] = "Sale_Meeting_Report";
	$projectEntities[ "Sales_Next_Action_Report" ] = array( "url" => "Sales_Next_Action_Report", "type" => 2 );
	$projectEntitiesReverse[ "Sales_Next_Action_Report" ] = "Sales_Next_Action_Report";
	$projectEntities[ "Petty_Cash_Dashboard" ] = array( "url" => "Petty_Cash_Dashboard", "type" => 4 );
	$projectEntitiesReverse[ "Petty_Cash_Dashboard" ] = "Petty_Cash_Dashboard";
	$projectEntities[ "open_quote_Report" ] = array( "url" => "open_quote_Report", "type" => 2 );
	$projectEntitiesReverse[ "open_quote_Report" ] = "open_quote_Report";
	$projectEntities[ "Open_cases_per_agent" ] = array( "url" => "Open_cases_per_agent", "type" => 3 );
	$projectEntitiesReverse[ "Open_cases_per_agent" ] = "Open_cases_per_agent";
	$projectEntities[ "Marketing_sales_per_campaing_chart" ] = array( "url" => "Marketing_sales_per_campaing_chart", "type" => 3 );
	$projectEntitiesReverse[ "Marketing_sales_per_campaing_chart" ] = "Marketing_sales_per_campaing_chart";
	$projectEntities[ "Logistics_Dashboard" ] = array( "url" => "Logistics_Dashboard", "type" => 4 );
	$projectEntitiesReverse[ "Logistics_Dashboard" ] = "Logistics_Dashboard";
	$projectEntities[ "Accounting_Dashboard" ] = array( "url" => "Accounting_Dashboard", "type" => 4 );
	$projectEntitiesReverse[ "Accounting_Dashboard" ] = "Accounting_Dashboard";
	$projectEntities[ "AP_open_po_receiving" ] = array( "url" => "AP_open_po_receiving", "type" => 2 );
	$projectEntitiesReverse[ "AP_open_po_receiving" ] = "AP_open_po_receiving";
	$projectEntities[ "Production_Inventory_Balance_Report" ] = array( "url" => "Production_Inventory_Balance_Report", "type" => 2 );
	$projectEntitiesReverse[ "Production_Inventory_Balance_Report" ] = "Production_Inventory_Balance_Report";
	$projectEntities[ "Production_WIP" ] = array( "url" => "Production_WIP", "type" => 2 );
	$projectEntitiesReverse[ "Production_WIP" ] = "Production_WIP";
	$projectEntities[ "Warehouse_Dashboard" ] = array( "url" => "Warehouse_Dashboard", "type" => 4 );
	$projectEntitiesReverse[ "Warehouse_Dashboard" ] = "Warehouse_Dashboard";
	$projectEntities[ "Warehouse_Inventory_Balance_Report" ] = array( "url" => "Warehouse_Inventory_Balance_Report", "type" => 2 );
	$projectEntitiesReverse[ "Warehouse_Inventory_Balance_Report" ] = "Warehouse_Inventory_Balance_Report";
	$projectEntities[ "logistic_open_inventory_tranfer" ] = array( "url" => "logistic_open_inventory_tranfer", "type" => 2 );
	$projectEntitiesReverse[ "logistic_open_inventory_tranfer" ] = "logistic_open_inventory_tranfer";
	$projectEntities[ "Sales_Leads" ] = array( "url" => "Sales_Leads", "type" => 2 );
	$projectEntitiesReverse[ "Sales_Leads" ] = "Sales_Leads";
	$projectEntities[ "Sales_opportunity_Next_Action" ] = array( "url" => "Sales_opportunity_Next_Action", "type" => 2 );
	$projectEntitiesReverse[ "Sales_opportunity_Next_Action" ] = "Sales_opportunity_Next_Action";
	$projectEntities[ "Sales_Active_opportunity" ] = array( "url" => "Sales_Active_opportunity", "type" => 2 );
	$projectEntitiesReverse[ "Sales_Active_opportunity" ] = "Sales_Active_opportunity";
	$projectEntities[ "production_detail" ] = array( "url" => "production_detail", "type" => 0 );
	$projectEntitiesReverse[ "production_detail" ] = "production_detail";
	$projectEntities[ "quality_control_detail_for_checking" ] = array( "url" => "quality_control_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "quality_control_detail_for_checking" ] = "quality_control_detail_for_checking";
	$projectEntities[ "EWT_Certificates_summay" ] = array( "url" => "EWT_Certificates_summay", "type" => 2 );
	$projectEntitiesReverse[ "EWT_Certificates_summay" ] = "EWT_Certificates_summay";
	$projectEntities[ "tax_person_status" ] = array( "url" => "tax_person_status", "type" => 0 );
	$projectEntitiesReverse[ "tax_person_status" ] = "tax_person_status";
	$projectEntities[ "journal_entry_inv_transfer" ] = array( "url" => "journal_entry_inv_transfer", "type" => 0 );
	$projectEntitiesReverse[ "journal_entry_inv_transfer" ] = "journal_entry_inv_transfer";
	$projectEntities[ "Recon_withdraw" ] = array( "url" => "Recon_withdraw", "type" => 1 );
	$projectEntitiesReverse[ "Recon_withdraw" ] = "Recon_withdraw";
	$projectEntities[ "lot" ] = array( "url" => "lot", "type" => 0 );
	$projectEntitiesReverse[ "lot" ] = "lot";
	$projectEntities[ "serial_no" ] = array( "url" => "serial_no", "type" => 0 );
	$projectEntitiesReverse[ "serial_no" ] = "serial_no";
	$projectEntities[ "request_for_quote" ] = array( "url" => "request_for_quote", "type" => 0 );
	$projectEntitiesReverse[ "request_for_quote" ] = "request_for_quote";
	$projectEntities[ "request_for_quoter_detail" ] = array( "url" => "request_for_quoter_detail", "type" => 0 );
	$projectEntitiesReverse[ "request_for_quoter_detail" ] = "request_for_quoter_detail";
	$projectEntities[ "request_for_quote_for_checking" ] = array( "url" => "request_for_quote_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "request_for_quote_for_checking" ] = "request_for_quote_for_checking";
	$projectEntities[ "request_for_quote_for_approval" ] = array( "url" => "request_for_quote_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "request_for_quote_for_approval" ] = "request_for_quote_for_approval";
	$projectEntities[ "request_for_quoter_detail_for_checking" ] = array( "url" => "request_for_quoter_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "request_for_quoter_detail_for_checking" ] = "request_for_quoter_detail_for_checking";
	$projectEntities[ "request_for_quoter_detail_for_approval" ] = array( "url" => "request_for_quoter_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "request_for_quoter_detail_for_approval" ] = "request_for_quoter_detail_for_approval";
	$projectEntities[ "estimate_cost" ] = array( "url" => "estimate_cost", "type" => 0 );
	$projectEntitiesReverse[ "estimate_cost" ] = "estimate_cost";
	$projectEntities[ "bal_sales_order_detail" ] = array( "url" => "bal_sales_order_detail", "type" => 0 );
	$projectEntitiesReverse[ "bal_sales_order_detail" ] = "bal_sales_order_detail";
	$projectEntities[ "Setup_Wizard" ] = array( "url" => "Setup_Wizard", "type" => 1 );
	$projectEntitiesReverse[ "Setup_Wizard" ] = "Setup_Wizard";
	$projectEntities[ "Overview" ] = array( "url" => "Overview", "type" => 2 );
	$projectEntitiesReverse[ "Overview" ] = "Overview";
	$projectEntities[ "Setup_Checklist" ] = array( "url" => "Setup_Checklist", "type" => 2 );
	$projectEntitiesReverse[ "Setup_Checklist" ] = "Setup_Checklist";
	$projectEntities[ "project_bill_of_material" ] = array( "url" => "project_bill_of_material", "type" => 0 );
	$projectEntitiesReverse[ "project_bill_of_material" ] = "project_bill_of_material";
	$projectEntities[ "project_progress_billing" ] = array( "url" => "project_progress_billing", "type" => 0 );
	$projectEntitiesReverse[ "project_progress_billing" ] = "project_progress_billing";
	$projectEntities[ "memorized_transactions" ] = array( "url" => "memorized_transactions", "type" => 0 );
	$projectEntitiesReverse[ "memorized_transactions" ] = "memorized_transactions";
	$projectEntities[ "change_password" ] = array( "url" => "change_password", "type" => 1 );
	$projectEntitiesReverse[ "change_password" ] = "change_password";
	$projectEntities[ "checklist" ] = array( "url" => "checklist", "type" => 0 );
	$projectEntitiesReverse[ "checklist" ] = "checklist";
	$projectEntities[ "stage" ] = array( "url" => "stage", "type" => 0 );
	$projectEntitiesReverse[ "stage" ] = "stage";
	$projectEntities[ "bal_total_po_detail" ] = array( "url" => "bal_total_po_detail", "type" => 0 );
	$projectEntitiesReverse[ "bal_total_po_detail" ] = "bal_total_po_detail";
	$projectEntities[ "item_production_phase" ] = array( "url" => "item_production_phase", "type" => 0 );
	$projectEntitiesReverse[ "item_production_phase" ] = "item_production_phase";
	$projectEntities[ "production_bill_of_material" ] = array( "url" => "production_bill_of_material", "type" => 0 );
	$projectEntitiesReverse[ "production_bill_of_material" ] = "production_bill_of_material";
	$projectEntities[ "production_stage_cost_detail" ] = array( "url" => "production_stage_cost_detail", "type" => 0 );
	$projectEntitiesReverse[ "production_stage_cost_detail" ] = "production_stage_cost_detail";
	$projectEntities[ "employees" ] = array( "url" => "employees", "type" => 0 );
	$projectEntitiesReverse[ "employees" ] = "employees";
	$projectEntities[ "sale_assemble_detail" ] = array( "url" => "sale_assemble_detail", "type" => 0 );
	$projectEntitiesReverse[ "sale_assemble_detail" ] = "sale_assemble_detail";
	$projectEntities[ "Check_Release" ] = array( "url" => "Check_Release", "type" => 1 );
	$projectEntitiesReverse[ "Check_Release" ] = "Check_Release";
	$projectEntities[ "unused_check_book_registry" ] = array( "url" => "unused_check_book_registry", "type" => 1 );
	$projectEntitiesReverse[ "unused_check_book_registry" ] = "unused_check_book_registry";
	$projectEntities[ "item_bom_type" ] = array( "url" => "item_bom_type", "type" => 0 );
	$projectEntitiesReverse[ "item_bom_type" ] = "item_bom_type";
	$projectEntities[ "canvas" ] = array( "url" => "canvas", "type" => 1 );
	$projectEntitiesReverse[ "canvas" ] = "canvas";
	$projectEntities[ "canvas_detail" ] = array( "url" => "canvas_detail", "type" => 1 );
	$projectEntitiesReverse[ "canvas_detail" ] = "canvas_detail";
	$projectEntities[ "inventory_adjustment_type" ] = array( "url" => "inventory_adjustment_type", "type" => 0 );
	$projectEntitiesReverse[ "inventory_adjustment_type" ] = "inventory_adjustment_type";
	$projectEntities[ "Help" ] = array( "url" => "Help", "type" => 2 );
	$projectEntitiesReverse[ "Help" ] = "Help";
	$projectEntities[ "Global Accounting Systems" ] = array( "url" => "Global_Accounting_Systems", "type" => 2 );
	$projectEntitiesReverse[ "Global_Accounting_Systems" ] = "Global Accounting Systems";
	$projectEntities[ "bank_collection" ] = array( "url" => "bank_collection", "type" => 0 );
	$projectEntitiesReverse[ "bank_collection" ] = "bank_collection";
	$projectEntities[ "bank_disbursement" ] = array( "url" => "bank_disbursement", "type" => 0 );
	$projectEntitiesReverse[ "bank_disbursement" ] = "bank_disbursement";
	$projectEntities[ "project_progress" ] = array( "url" => "project_progress", "type" => 1 );
	$projectEntitiesReverse[ "project_progress" ] = "project_progress";
	$projectEntities[ "inv_fifo" ] = array( "url" => "inv_fifo", "type" => 0 );
	$projectEntitiesReverse[ "inv_fifo" ] = "inv_fifo";
	$projectEntities[ "inventory_tranfer_request" ] = array( "url" => "inventory_tranfer_request", "type" => 0 );
	$projectEntitiesReverse[ "inventory_tranfer_request" ] = "inventory_tranfer_request";
	$projectEntities[ "branch_sales" ] = array( "url" => "branch_sales", "type" => 0 );
	$projectEntitiesReverse[ "branch_sales" ] = "branch_sales";
	$projectEntities[ "branch_sales_detail" ] = array( "url" => "branch_sales_detail", "type" => 0 );
	$projectEntitiesReverse[ "branch_sales_detail" ] = "branch_sales_detail";
	$projectEntities[ "inventory_transfer__request_detail" ] = array( "url" => "inventory_transfer__request_detail", "type" => 0 );
	$projectEntitiesReverse[ "inventory_transfer__request_detail" ] = "inventory_transfer__request_detail";
	$projectEntities[ "inventory_tranfer_view" ] = array( "url" => "inventory_tranfer_view", "type" => 1 );
	$projectEntitiesReverse[ "inventory_tranfer_view" ] = "inventory_tranfer_view";
	$projectEntities[ "inventory_transfer_detail_view" ] = array( "url" => "inventory_transfer_detail_view", "type" => 1 );
	$projectEntitiesReverse[ "inventory_transfer_detail_view" ] = "inventory_transfer_detail_view";
	$projectEntities[ "import_inv_adj" ] = array( "url" => "import_inv_adj", "type" => 0 );
	$projectEntitiesReverse[ "import_inv_adj" ] = "import_inv_adj";
	$projectEntities[ "import_inventory_transfer" ] = array( "url" => "import_inventory_transfer", "type" => 0 );
	$projectEntitiesReverse[ "import_inventory_transfer" ] = "import_inventory_transfer";
	$projectEntities[ "loyalty_points" ] = array( "url" => "loyalty_points", "type" => 0 );
	$projectEntitiesReverse[ "loyalty_points" ] = "loyalty_points";
	$projectEntities[ "bal_sales_credit_memo" ] = array( "url" => "bal_sales_credit_memo", "type" => 0 );
	$projectEntitiesReverse[ "bal_sales_credit_memo" ] = "bal_sales_credit_memo";
	$projectEntities[ "expense" ] = array( "url" => "expense", "type" => 0 );
	$projectEntitiesReverse[ "expense" ] = "expense";
	$projectEntities[ "bal_sales_credit_memo1" ] = array( "url" => "bal_sales_credit_memo1", "type" => 1 );
	$projectEntitiesReverse[ "bal_sales_credit_memo1" ] = "bal_sales_credit_memo1";
	$projectEntities[ "bal_invoice_credit_memo" ] = array( "url" => "bal_invoice_credit_memo", "type" => 0 );
	$projectEntitiesReverse[ "bal_invoice_credit_memo" ] = "bal_invoice_credit_memo";
	$projectEntities[ "item_price_per_category" ] = array( "url" => "item_price_per_category", "type" => 0 );
	$projectEntitiesReverse[ "item_price_per_category" ] = "item_price_per_category";
	$projectEntities[ "item_group_price_per_category" ] = array( "url" => "item_group_price_per_category", "type" => 0 );
	$projectEntitiesReverse[ "item_group_price_per_category" ] = "item_group_price_per_category";
	$projectEntities[ "report_center_fs" ] = array( "url" => "report_center_fs", "type" => 1 );
	$projectEntitiesReverse[ "report_center_fs" ] = "report_center_fs";
	$projectEntities[ "report_center_books" ] = array( "url" => "report_center_books", "type" => 1 );
	$projectEntitiesReverse[ "report_center_books" ] = "report_center_books";
	$projectEntities[ "report_center_sales" ] = array( "url" => "report_center_sales", "type" => 1 );
	$projectEntitiesReverse[ "report_center_sales" ] = "report_center_sales";
	$projectEntities[ "report_center_tax" ] = array( "url" => "report_center_tax", "type" => 1 );
	$projectEntitiesReverse[ "report_center_tax" ] = "report_center_tax";
	$projectEntities[ "report_center_ap" ] = array( "url" => "report_center_ap", "type" => 1 );
	$projectEntitiesReverse[ "report_center_ap" ] = "report_center_ap";
	$projectEntities[ "report_center_ar" ] = array( "url" => "report_center_ar", "type" => 1 );
	$projectEntitiesReverse[ "report_center_ar" ] = "report_center_ar";
	$projectEntities[ "report_center_purchases" ] = array( "url" => "report_center_purchases", "type" => 1 );
	$projectEntitiesReverse[ "report_center_purchases" ] = "report_center_purchases";
	$projectEntities[ "report_center_inventory" ] = array( "url" => "report_center_inventory", "type" => 1 );
	$projectEntitiesReverse[ "report_center_inventory" ] = "report_center_inventory";
	$projectEntities[ "picklist" ] = array( "url" => "picklist", "type" => 0 );
	$projectEntitiesReverse[ "picklist" ] = "picklist";
	$projectEntities[ "picklist_detail" ] = array( "url" => "picklist_detail", "type" => 0 );
	$projectEntitiesReverse[ "picklist_detail" ] = "picklist_detail";
	$projectEntities[ "picklist_detail_done" ] = array( "url" => "picklist_detail_done", "type" => 0 );
	$projectEntitiesReverse[ "picklist_detail_done" ] = "picklist_detail_done";
	$projectEntities[ "projects" ] = array( "url" => "projects", "type" => 1 );
	$projectEntitiesReverse[ "projects" ] = "projects";
	$projectEntities[ "item_cost_center_detail" ] = array( "url" => "item_cost_center_detail", "type" => 0 );
	$projectEntitiesReverse[ "item_cost_center_detail" ] = "item_cost_center_detail";
	$projectEntities[ "DR_Dashboard_Report" ] = array( "url" => "DR_Dashboard_Report", "type" => 2 );
	$projectEntitiesReverse[ "DR_Dashboard_Report" ] = "DR_Dashboard_Report";
	$projectEntities[ "trip_ticket_Dashboard_Report" ] = array( "url" => "trip_ticket_Dashboard_Report", "type" => 2 );
	$projectEntitiesReverse[ "trip_ticket_Dashboard_Report" ] = "trip_ticket_Dashboard_Report";
	$projectEntities[ "inventory_tranfer_Dashboard_Report" ] = array( "url" => "inventory_tranfer_Dashboard_Report", "type" => 2 );
	$projectEntitiesReverse[ "inventory_tranfer_Dashboard_Report" ] = "inventory_tranfer_Dashboard_Report";
	$projectEntities[ "project_replenishment_Dashboard_Report" ] = array( "url" => "project_replenishment_Dashboard_Report", "type" => 2 );
	$projectEntitiesReverse[ "project_replenishment_Dashboard_Report" ] = "project_replenishment_Dashboard_Report";
	$projectEntities[ "picklist_for_approval" ] = array( "url" => "picklist_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "picklist_for_approval" ] = "picklist_for_approval";
	$projectEntities[ "picklist_for_checking" ] = array( "url" => "picklist_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "picklist_for_checking" ] = "picklist_for_checking";
	$projectEntities[ "picklist_detail_for_approval" ] = array( "url" => "picklist_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "picklist_detail_for_approval" ] = "picklist_detail_for_approval";
	$projectEntities[ "picklist_detail_for_checking" ] = array( "url" => "picklist_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "picklist_detail_for_checking" ] = "picklist_detail_for_checking";
	$projectEntities[ "bal_purchase_debit_memo" ] = array( "url" => "bal_purchase_debit_memo", "type" => 0 );
	$projectEntitiesReverse[ "bal_purchase_debit_memo" ] = "bal_purchase_debit_memo";
	$projectEntities[ "bal_purchase_head_debit_memo" ] = array( "url" => "bal_purchase_head_debit_memo", "type" => 0 );
	$projectEntitiesReverse[ "bal_purchase_head_debit_memo" ] = "bal_purchase_head_debit_memo";
	$projectEntities[ "return_components" ] = array( "url" => "return_components", "type" => 0 );
	$projectEntitiesReverse[ "return_components" ] = "return_components";
	$projectEntities[ "return_components_detail" ] = array( "url" => "return_components_detail", "type" => 0 );
	$projectEntitiesReverse[ "return_components_detail" ] = "return_components_detail";
	$projectEntities[ "open_production" ] = array( "url" => "open_production", "type" => 1 );
	$projectEntitiesReverse[ "open_production" ] = "open_production";
	$projectEntities[ "production _imp" ] = array( "url" => "production__imp", "type" => 0 );
	$projectEntitiesReverse[ "production__imp" ] = "production _imp";
	$projectEntities[ "issue_component_imp" ] = array( "url" => "issue_component_imp", "type" => 0 );
	$projectEntitiesReverse[ "issue_component_imp" ] = "issue_component_imp";
	$projectEntities[ "request_for_payment_imp" ] = array( "url" => "request_for_payment_imp", "type" => 0 );
	$projectEntitiesReverse[ "request_for_payment_imp" ] = "request_for_payment_imp";
	$projectEntities[ "fixed_asset_time_usage" ] = array( "url" => "fixed_asset_time_usage", "type" => 0 );
	$projectEntitiesReverse[ "fixed_asset_time_usage" ] = "fixed_asset_time_usage";
	$projectEntities[ "fixed_asset_time_usage_detail" ] = array( "url" => "fixed_asset_time_usage_detail", "type" => 0 );
	$projectEntitiesReverse[ "fixed_asset_time_usage_detail" ] = "fixed_asset_time_usage_detail";
	$projectEntities[ "service_visit" ] = array( "url" => "service_visit", "type" => 0 );
	$projectEntitiesReverse[ "service_visit" ] = "service_visit";
	$projectEntities[ "so_imp" ] = array( "url" => "so_imp", "type" => 0 );
	$projectEntitiesReverse[ "so_imp" ] = "so_imp";
	$projectEntities[ "open_cases" ] = array( "url" => "open_cases", "type" => 1 );
	$projectEntitiesReverse[ "open_cases" ] = "open_cases";
	$projectEntities[ "phase_bill_of_materials" ] = array( "url" => "phase_bill_of_materials", "type" => 0 );
	$projectEntitiesReverse[ "phase_bill_of_materials" ] = "phase_bill_of_materials";
	$projectEntities[ "production_itemized_detail" ] = array( "url" => "production_itemized_detail", "type" => 0 );
	$projectEntitiesReverse[ "production_itemized_detail" ] = "production_itemized_detail";
	$projectEntities[ "production_search" ] = array( "url" => "production_search", "type" => 1 );
	$projectEntitiesReverse[ "production_search" ] = "production_search";
	$projectEntities[ "production_stage_detail" ] = array( "url" => "production_stage_detail", "type" => 1 );
	$projectEntitiesReverse[ "production_stage_detail" ] = "production_stage_detail";
	$projectEntities[ "service_report" ] = array( "url" => "service_report", "type" => 0 );
	$projectEntitiesReverse[ "service_report" ] = "service_report";
	$projectEntities[ "service_report_detail" ] = array( "url" => "service_report_detail", "type" => 0 );
	$projectEntitiesReverse[ "service_report_detail" ] = "service_report_detail";
	$projectEntities[ "excise_tax" ] = array( "url" => "excise_tax", "type" => 0 );
	$projectEntitiesReverse[ "excise_tax" ] = "excise_tax";
	$projectEntities[ "repairs_particular" ] = array( "url" => "repairs_particular", "type" => 0 );
	$projectEntitiesReverse[ "repairs_particular" ] = "repairs_particular";
	$projectEntities[ "waranty_detail" ] = array( "url" => "waranty_detail", "type" => 0 );
	$projectEntitiesReverse[ "waranty_detail" ] = "waranty_detail";
	$projectEntities[ "phase_dropdown1" ] = array( "url" => "phase_dropdown1", "type" => 1 );
	$projectEntitiesReverse[ "phase_dropdown1" ] = "phase_dropdown1";
	$projectEntities[ "phase_dropdown" ] = array( "url" => "phase_dropdown", "type" => 1 );
	$projectEntitiesReverse[ "phase_dropdown" ] = "phase_dropdown";
	$projectEntities[ "project_name" ] = array( "url" => "project_name", "type" => 2 );
	$projectEntitiesReverse[ "project_name" ] = "project_name";
	$projectEntities[ "phase_billing" ] = array( "url" => "phase_billing", "type" => 0 );
	$projectEntitiesReverse[ "phase_billing" ] = "phase_billing";
	$projectEntities[ "new_ticket" ] = array( "url" => "new_ticket", "type" => 0 );
	$projectEntitiesReverse[ "new_ticket" ] = "new_ticket";
	$projectEntities[ "new_user" ] = array( "url" => "new_user", "type" => 0 );
	$projectEntitiesReverse[ "new_user" ] = "new_user";
	$projectEntities[ "gas_type" ] = array( "url" => "gas_type", "type" => 0 );
	$projectEntitiesReverse[ "gas_type" ] = "gas_type";
	$projectEntities[ "inventory_tranfer_mobile" ] = array( "url" => "inventory_tranfer_mobile", "type" => 1 );
	$projectEntitiesReverse[ "inventory_tranfer_mobile" ] = "inventory_tranfer_mobile";
	$projectEntities[ "inventory_transfer_detail_mobile" ] = array( "url" => "inventory_transfer_detail_mobile", "type" => 1 );
	$projectEntitiesReverse[ "inventory_transfer_detail_mobile" ] = "inventory_transfer_detail_mobile";
	$projectEntities[ "Item Expiration Report" ] = array( "url" => "item_expiration_report", "type" => 2 );
	$projectEntitiesReverse[ "item_expiration_report" ] = "Item Expiration Report";
	$projectEntities[ "item_with_balances" ] = array( "url" => "item_with_balances", "type" => 1 );
	$projectEntitiesReverse[ "item_with_balances" ] = "item_with_balances";
	$projectEntities[ "Import_Bank_Statement" ] = array( "url" => "import_bank_statement", "type" => 1 );
	$projectEntitiesReverse[ "import_bank_statement" ] = "Import_Bank_Statement";
	$projectEntities[ "salesapp" ] = array( "url" => "salesapp", "type" => 0 );
	$projectEntitiesReverse[ "salesapp" ] = "salesapp";
	$projectEntities[ "bank_list" ] = array( "url" => "bank_list", "type" => 0 );
	$projectEntitiesReverse[ "bank_list" ] = "bank_list";
	$projectEntities[ "import_sales" ] = array( "url" => "import_sales", "type" => 0 );
	$projectEntitiesReverse[ "import_sales" ] = "import_sales";
	$projectEntities[ "production_stage_report" ] = array( "url" => "production_stage_report", "type" => 1 );
	$projectEntitiesReverse[ "production_stage_report" ] = "production_stage_report";
	$projectEntities[ "user_roles" ] = array( "url" => "user_roles", "type" => 0 );
	$projectEntitiesReverse[ "user_roles" ] = "user_roles";
	$projectEntities[ "check_formats" ] = array( "url" => "check_formats", "type" => 0 );
	$projectEntitiesReverse[ "check_formats" ] = "check_formats";
	$projectEntities[ "asset_accountability" ] = array( "url" => "asset_accountability", "type" => 0 );
	$projectEntitiesReverse[ "asset_accountability" ] = "asset_accountability";
	$projectEntities[ "asset_accountability_detail" ] = array( "url" => "asset_accountability_detail", "type" => 0 );
	$projectEntitiesReverse[ "asset_accountability_detail" ] = "asset_accountability_detail";
	$projectEntities[ "cash_pullout" ] = array( "url" => "cash_pullout", "type" => 0 );
	$projectEntitiesReverse[ "cash_pullout" ] = "cash_pullout";
	$projectEntities[ "cash_pullout_detail" ] = array( "url" => "cash_pullout_detail", "type" => 0 );
	$projectEntitiesReverse[ "cash_pullout_detail" ] = "cash_pullout_detail";
	$projectEntities[ "liquidation" ] = array( "url" => "liquidation", "type" => 0 );
	$projectEntitiesReverse[ "liquidation" ] = "liquidation";
	$projectEntities[ "liquidation_detail" ] = array( "url" => "liquidation_detail", "type" => 0 );
	$projectEntitiesReverse[ "liquidation_detail" ] = "liquidation_detail";
	$projectEntities[ "loan" ] = array( "url" => "loan", "type" => 0 );
	$projectEntitiesReverse[ "loan" ] = "loan";
	$projectEntities[ "loan__detail" ] = array( "url" => "loan__detail", "type" => 0 );
	$projectEntitiesReverse[ "loan__detail" ] = "loan__detail";
	$projectEntities[ "marketing_job_order" ] = array( "url" => "marketing_job_order", "type" => 0 );
	$projectEntitiesReverse[ "marketing_job_order" ] = "marketing_job_order";
	$projectEntities[ "spoilage" ] = array( "url" => "spoilage", "type" => 0 );
	$projectEntitiesReverse[ "spoilage" ] = "spoilage";
	$projectEntities[ "spoilage_detail" ] = array( "url" => "spoilage_detail", "type" => 0 );
	$projectEntitiesReverse[ "spoilage_detail" ] = "spoilage_detail";
	$projectEntities[ "tara_sheet" ] = array( "url" => "tara_sheet", "type" => 0 );
	$projectEntitiesReverse[ "tara_sheet" ] = "tara_sheet";
	$projectEntities[ "tara_sheet_detail" ] = array( "url" => "tara_sheet_detail", "type" => 0 );
	$projectEntitiesReverse[ "tara_sheet_detail" ] = "tara_sheet_detail";
	$projectEntities[ "transmittal_slip" ] = array( "url" => "transmittal_slip", "type" => 0 );
	$projectEntitiesReverse[ "transmittal_slip" ] = "transmittal_slip";
	$projectEntities[ "transmittal_slip_detail" ] = array( "url" => "transmittal_slip_detail", "type" => 0 );
	$projectEntitiesReverse[ "transmittal_slip_detail" ] = "transmittal_slip_detail";
	$projectEntities[ "spoilage_detail2" ] = array( "url" => "spoilage_detail2", "type" => 0 );
	$projectEntitiesReverse[ "spoilage_detail2" ] = "spoilage_detail2";
	$projectEntities[ "admin_rights" ] = array( "url" => "admin_rights", "type" => 1 );
	$projectEntitiesReverse[ "admin_rights" ] = "admin_rights";
	$projectEntities[ "admin_members" ] = array( "url" => "admin_members", "type" => 1 );
	$projectEntitiesReverse[ "admin_members" ] = "admin_members";
	$projectEntities[ "admin_users" ] = array( "url" => "admin_users", "type" => 1 );
	$projectEntitiesReverse[ "admin_users" ] = "admin_users";
	$projectEntities[ "open_rfp_to_apv" ] = array( "url" => "open_rfp_to_apv", "type" => 0 );
	$projectEntitiesReverse[ "open_rfp_to_apv" ] = "open_rfp_to_apv";
	$projectEntities[ "canvass" ] = array( "url" => "canvass", "type" => 1 );
	$projectEntitiesReverse[ "canvass" ] = "canvass";
	$projectEntities[ "canvass_for_checking" ] = array( "url" => "canvass_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "canvass_for_checking" ] = "canvass_for_checking";
	$projectEntities[ "canvass_for_approval" ] = array( "url" => "canvass_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "canvass_for_approval" ] = "canvass_for_approval";
	$projectEntities[ "canvass_to_purchase_order" ] = array( "url" => "canvass_to_purchase_order", "type" => 1 );
	$projectEntitiesReverse[ "canvass_to_purchase_order" ] = "canvass_to_purchase_order";
	$projectEntities[ "open_cv_for_por" ] = array( "url" => "open_cv_for_por", "type" => 1 );
	$projectEntitiesReverse[ "open_cv_for_por" ] = "open_cv_for_por";
	$projectEntities[ "open_cv_for_liq" ] = array( "url" => "open_cv_for_liq", "type" => 1 );
	$projectEntitiesReverse[ "open_cv_for_liq" ] = "open_cv_for_liq";
	$projectEntities[ "open_dr_for_invoice" ] = array( "url" => "open_dr_for_invoice", "type" => 0 );
	$projectEntitiesReverse[ "open_dr_for_invoice" ] = "open_dr_for_invoice";
	$projectEntities[ "customer_shipping_address" ] = array( "url" => "customer_shipping_address1", "type" => 0 );
	$projectEntitiesReverse[ "customer_shipping_address1" ] = "customer_shipping_address";
	$projectEntities[ "region" ] = array( "url" => "region", "type" => 0 );
	$projectEntitiesReverse[ "region" ] = "region";
	$projectEntities[ "province" ] = array( "url" => "province", "type" => 0 );
	$projectEntitiesReverse[ "province" ] = "province";
	$projectEntities[ "city" ] = array( "url" => "city", "type" => 0 );
	$projectEntitiesReverse[ "city" ] = "city";
	$projectEntities[ "barangay" ] = array( "url" => "barangay", "type" => 0 );
	$projectEntitiesReverse[ "barangay" ] = "barangay";
}

function findTable( $table ) {
	global $projectEntities, $projectEntitiesReverse;
	fillProjectEntites();

	if( isset( $projectEntities[ $table ] ) )
		return $table;
	$uTable = strtoupper( $table );

	foreach( $projectEntities as $t => $d ) {
		if( $uTable == strtoupper($t) )
			return $t;
		$gt = GoodFieldName($t);
		if( $table == $gt || $uTable == strtoupper( $gt ) )
			return $t;
	}

	//	check shortTableName
	if( isset( $projectEntitiesReverse[ $table ] ) )
		return $projectEntitiesReverse[ $table ];
	foreach( $projectEntitiesReverse as $t => $v ) {
		if( $uTable == strtoupper($t) )
			return $v;
	}
	return "";
}
//	return table short name
function GetTableURL( $table )
{
	if( $table == "<global>")
		return GLOBAL_PAGES_SHORT;
	global $strTableName, $projectEntities;

	if(!$table) {
		return "";
	}
	fillProjectEntites();

	if( isset( $projectEntities[ $table ] ) )
		return $projectEntities[ $table ][ "url" ];

	return "";
}

function GetEntityType($table = "")
{
	global $strTableName, $projectEntities;
	if(!$table)
		$table=$strTableName;
	fillProjectEntites();

	if( isset( $projectEntities[ $table ] ) )
		return $projectEntities[ $table ][ "type" ];

	return "";
}


//	return strTableName by short table name
function GetTableByShort( $shortTName )
{
	global $projectEntitiesReverse;
	fillProjectEntites();
	if(!$shortTName)
		return false;
	return $projectEntitiesReverse[ $shortTName ];
}

//	A
	$g_defaultOptionValues["Absolute"] = false;
	$g_defaultOptionValues["acceptFileTypes"] = ".+$";
	$g_defaultOptionValues["addFields"] = array();
	$g_defaultOptionValues["addPageEvents"] = false;
	$g_defaultOptionValues["ajaxCodeSnippetAdded"] = false;
	$g_defaultOptionValues["advSearchFields"] = array();
	$g_defaultOptionValues["afterAddAction"] = 1;
	$g_defaultOptionValues["afterAddActionDetTable"] = "";
	$g_defaultOptionValues["afterEditAction"] = 1;
	$g_defaultOptionValues["afterEditActionDetTable"] = "";
	$g_defaultOptionValues["AllowToAdd"] = false;
	$g_defaultOptionValues["allowFieldsReordering"] = false;
	$g_defaultOptionValues["allowShowHideFields"] = false;
	$g_defaultOptionValues["allSearchFields"] = array();
	$g_defaultOptionValues["arrGroupsPerPage"] = array();
	$g_defaultOptionValues["arrGridTabs"] = array();
	$g_defaultOptionValues["arrRecsPerPage"] = array();

	$g_defaultOptionValues["audioTitleField"] = "";
	$g_defaultOptionValues["audit"] = false;
	$g_defaultOptionValues["autoCompleteFields"] = array();
	$g_defaultOptionValues["autoCompleteFieldsAdd"] = array();
	$g_defaultOptionValues["autoCompleteFieldsEdit"] = array();
	$g_defaultOptionValues["autoCompleteFieldsOnEdit"] = false;
	$g_defaultOptionValues["AutoInc"] = false;
	$g_defaultOptionValues["autoUpload"] = false;
	$g_defaultOptionValues["autoUpdatable"] = false;
//	B
	$g_defaultOptionValues["badgeColor"] = "";
	$g_defaultOptionValues["bInlineAdd"] = false;
	$g_defaultOptionValues["bInlineEdit"] = false;
	$g_defaultOptionValues["bUpdateSelected"] = false;
	$g_defaultOptionValues["bIsEncrypted"] = false;
	$g_defaultOptionValues["bListPage"] = false;
	$g_defaultOptionValues["bPrinterPage"] = false;
	$g_defaultOptionValues["buttonsAdded"] = false;
//	C
	$g_defaultOptionValues["CompatibilityMode"] = false;
	$g_defaultOptionValues["categoryFields"] = array();
	$g_defaultOptionValues["ChartRefreshTime"] = 60;
	$g_defaultOptionValues["chartXml"] = "";
	$g_defaultOptionValues["chartType"] = "";
	$g_defaultOptionValues["closePopupAfterAdd"] = false;
	$g_defaultOptionValues["closePopupAfterEdit"] = false;
	$g_defaultOptionValues["controlWidth"] = 160;
	$g_defaultOptionValues["copy"] = false;
	$g_defaultOptionValues["CreateThumbnail"] = false;
	$g_defaultOptionValues["crossTabReport"] = false;
	$g_defaultOptionValues["CustomDisplay"] = false;
//	D
	$g_defaultOptionValues["detailsLinksOnList"] = DL_SINGLE;
	$g_defaultOptionValues["dashElements"] = array();
	$g_defaultOptionValues["dashCells"] = array();
	$g_defaultOptionValues["DateEditType"] = 0;
	$g_defaultOptionValues["DecimalDigits"] = "";
	$g_defaultOptionValues["defaultSearchOption"] = "";
	$g_defaultOptionValues["DefaultValue"] = "";
	$g_defaultOptionValues["delete"] = false;
	$g_defaultOptionValues["DeleteAssociatedFile"] = false;
	$g_defaultOptionValues["denyDuplicates"] = false;
	$g_defaultOptionValues["DependentLookups"] = array();
	$g_defaultOptionValues["descendingOrder"] = false;
	$g_defaultOptionValues["DisplayField"] = "";
//	E
	$g_defaultOptionValues["edit"] = false;
	$g_defaultOptionValues["editFields"] = array();
	$g_defaultOptionValues["EditFormat"] = "";
	$g_defaultOptionValues["EditParams"] = "";
	$g_defaultOptionValues["exportTo"] = false;
	$g_defaultOptionValues["exportFormatting"] = EXPORT_BOTH;
	$g_defaultOptionValues["exportDelimiter"] = ",";
//	F
	$g_defaultOptionValues["fieldIsVideoUrl"] = false;
	$g_defaultOptionValues["fieldIsImageUrl"] = false;
	$g_defaultOptionValues["FieldType"] = "";
	$g_defaultOptionValues["FieldPermissions"] = false;
	$g_defaultOptionValues["Filename"] = "";
	$g_defaultOptionValues["filterBy"] = 0;
	$g_defaultOptionValues["filterFields"] = array();
	$g_defaultOptionValues["filterFormat"] = FF_VALUE_LIST;
	$g_defaultOptionValues["filterIntervals"] = array();
	$g_defaultOptionValues["filterTotals"] = FT_NONE;
	$g_defaultOptionValues["filterMultiSelect"] = FM_NONE;
	$g_defaultOptionValues["filterCheckedMessageText"] = "";
	$g_defaultOptionValues["filterCheckedMessageType"] = "";
	$g_defaultOptionValues["filterUncheckedMessageText"] = "";
	$g_defaultOptionValues["filterUncheckedMessageType"] = "";
	$g_defaultOptionValues["filterSliderStepType"] = 1;
	$g_defaultOptionValues["filterSliderStepValue"] = 1;
	$g_defaultOptionValues["filterKnobsType"] = 0;
	$g_defaultOptionValues["filterApplyBtn"] = false;
	$g_defaultOptionValues["flexibleSearch"] = false;
	$g_defaultOptionValues["FormatTimeAttrs"] = array();
	$g_defaultOptionValues["freeInput"] = false;
	$g_defaultOptionValues["FullName"] = "";
	$g_defaultOptionValues["fieldEvents"] = false;
	$g_defaultOptionValues["fieldViewEvents"] = false;
//	G
	$g_defaultOptionValues["googleLikeFields"] = array();
	$g_defaultOptionValues["GoodName"] = "";
//	H
	$g_defaultOptionValues["hasEncryptedFields"] = false;
	$g_defaultOptionValues["hideEmptyFieldsOnView"] = false;
	$g_defaultOptionValues["HorizontalLookup"] = false;
	$g_defaultOptionValues["hlNewWindow"] = false;
	$g_defaultOptionValues["hlType"] = 0;
	$g_defaultOptionValues["hlLinkWordNameType"] = "";
	$g_defaultOptionValues["hlLinkWordText"] = "";
	$g_defaultOptionValues["hlTitleField"] = "";
	$g_defaultOptionValues["highlightSearchResults"] = false;
	$g_defaultOptionValues["HTML5InuptType"] = "text";
//	I
	$g_defaultOptionValues["Index"] = 0;
	$g_defaultOptionValues["InitialYearFactor"] = 100;
	$g_defaultOptionValues["ImageHeight"] = 0;
	$g_defaultOptionValues["ImageWidth"] = 0;
	//$g_defaultOptionValues["import"] = false;
	$g_defaultOptionValues["inlineAdd"] = false;
	$g_defaultOptionValues["inlineAddFields"] = array();
	$g_defaultOptionValues["inlineEdit"] = false;
	$g_defaultOptionValues["updateSelected"] = false;
	$g_defaultOptionValues["inlineEditFields"] = array();
	$g_defaultOptionValues["isDisplayLoading"] = false;
	$g_defaultOptionValues["tableType"] = "";
	$g_defaultOptionValues["IsRequired"] = false;
	$g_defaultOptionValues["insertNull"] = false;
	$g_defaultOptionValues["isResizeColumns"] = false;
	$g_defaultOptionValues["isSeparate"] = false;
	$g_defaultOptionValues["UpdateLatLng"] = false;
	$g_defaultOptionValues["isUseAjaxSuggest"] = false;
	$g_defaultOptionValues["isUseAudio"] = false;
	$g_defaultOptionValues["isUseFieldsMaps"] = false;
	$g_defaultOptionValues["isUseTimeForSearch"] = false;
	$g_defaultOptionValues["isUseToolTips"] = false;
	$g_defaultOptionValues["isUseVideo"] = false;
	$g_defaultOptionValues["listGridLayout"] = 0;
	$g_defaultOptionValues["isExistTotalFields"] = false;
	$g_defaultOptionValues["isViewPagePDFFitToPage"] = 1;
	$g_defaultOptionValues["isPrinterPagePDFFitToPage"] = 1;
	$g_defaultOptionValues["isLandscapeViewPDFOrientation"] = 0;
	$g_defaultOptionValues["isLandscapePrinterPagePDFOrientation"] = 0;
	$g_defaultOptionValues["isPrinterPagePDF"] = false;
	$g_defaultOptionValues["isViewPagePDF"] = false;
	$g_defaultOptionValues["isSQLExpression"] = false;
	$g_defaultOptionValues["isPrinterPageFitToPage"] = 1;
//	J
//	K
	$g_defaultOptionValues["Keys"] = array();
//	L
	$g_defaultOptionValues["Label"] = "";
	$g_defaultOptionValues["LookupConnId"] = "";
	$g_defaultOptionValues["LastYearFactor"] = 10;
	$g_defaultOptionValues["LCType"] = LCT_DROPDOWN;
	$g_defaultOptionValues["listPageId"] = "";
	$g_defaultOptionValues["addPageId"] = "";
	$g_defaultOptionValues["LinkField"] = "";
	$g_defaultOptionValues["LinkFieldType"] = 0;
	$g_defaultOptionValues["LinkPrefix"] = "";
	$g_defaultOptionValues["list"] = false;
	$g_defaultOptionValues["listAjax"] = false;

	$g_defaultOptionValues["locking"] = false;
	$g_defaultOptionValues["LookupDesc"] = false;
	$g_defaultOptionValues["LookupOrderBy"] = "";
	$g_defaultOptionValues["LookupTable"] = "";
	$g_defaultOptionValues["LookupType"] = 0;
	$g_defaultOptionValues["LookupUnique"] = false;
	$g_defaultOptionValues["LookupValues"] = array();
	$g_defaultOptionValues["LookupWhere"] = "";
	$g_defaultOptionValues["LookupWhereCode"] = false;
//	M
	$g_defaultOptionValues["mainTableOwnerID"] = "";
	$g_defaultOptionValues["mapData"] = array();
	$g_defaultOptionValues["masterListFields"] = array();
	$g_defaultOptionValues["maxFileSize"] = null;
	$g_defaultOptionValues["maxImageHeight"] = null;
	$g_defaultOptionValues["maxImageWidth"] = null;
	$g_defaultOptionValues["maxNumberOfFiles"] = 1;
	$g_defaultOptionValues["maxTotalFilesSize"] = null;
	$g_defaultOptionValues["moveNext"] = "";
	$g_defaultOptionValues["Multiselect"] = false;
//	N
	$g_defaultOptionValues["NCSearch"] = false;
	$g_defaultOptionValues["NeedEncode"] = false;
	$g_defaultOptionValues["NewSize"] = 0;
	$g_defaultOptionValues["noRecordsFirstPage"] = false;
	$g_defaultOptionValues["nSecOptions"] = ADVSECURITY_NONE;
	$g_defaultOptionValues["NumberOfChars"] = 0;
	$g_defaultOptionValues["numberOfVisibleItems"] = 10;
	$g_defaultOptionValues["nViewPagePDFScale"] = 100;
	$g_defaultOptionValues["nPrinterPagePDFScale"] = 100;
	$g_defaultOptionValues["nPrinterPageScale"] = 100;
	$g_defaultOptionValues["nPrinterPDFSplitRecords"] = 0;
	$g_defaultOptionValues["nPrinterSplitRecords"] = 0;


//	O
	$g_defaultOptionValues["OriginalTable"] = "";
	$g_defaultOptionValues["orderindexes"] = array();
	$g_defaultOptionValues["OwnerID"] = 0;
	$g_defaultOptionValues["ownerTable"] = "";
	$g_defaultOptionValues["OraSequenceName"] = "";
//	P
	$g_defaultOptionValues["pageSize"] = 0;
	$g_defaultOptionValues["panelSearchFields"] = array();
	$g_defaultOptionValues["parentFilterField"] = "";
	$g_defaultOptionValues["printFriendly"] = false;
	$g_defaultOptionValues["printFields"] = array();
	$g_defaultOptionValues["printGridLayout"] = 0;
	$g_defaultOptionValues["printReportLayout"] = 0;
	$g_defaultOptionValues["printerPageOrientation"] = 0;
//	Q
//	R
	$g_defaultOptionValues["recsLimit"] = 0;
	$g_defaultOptionValues["recsPerRowList"] = 1;
	$g_defaultOptionValues["recsPerRowPrint"] = 1;

	$g_defaultOptionValues["ResizeImage"] = false;
	$g_defaultOptionValues["RewindEnabled"] = false;
	$g_defaultOptionValues["rowHighlite"] = false;
	$g_defaultOptionValues["requiredSearchFields"] = array();
	$g_defaultOptionValues["reportPrintGroupsPerPage"] = 3;
	$g_defaultOptionValues["reportPrintPDFGroupsPerPage"] = 0;

//	S
	$g_defaultOptionValues["searchFields"] = false;
	$g_defaultOptionValues["searchPanelOptions"] = array();
	$g_defaultOptionValues["SelectSize"] = 1;
	$g_defaultOptionValues["selectExportDelimiter"] = false;
	$g_defaultOptionValues["selectExportFields"] = false;
	$g_defaultOptionValues["searchIsRequiredForFilters"] = false;
	$g_defaultOptionValues["searchOptionsList"] = array();
	$g_defaultOptionValues["ShortName"] = "";
	$g_defaultOptionValues["shortTableName"] = "";
	$g_defaultOptionValues["showAddInPopup"] = false;
	$g_defaultOptionValues["ShowCustomExpr"] = false;
	$g_defaultOptionValues["showEditInPopup"] = false;
	$g_defaultOptionValues["ShowFileSize"] = false;
	$g_defaultOptionValues["ShowIcon"] = false;
	$g_defaultOptionValues["showSearchPanel"] = false;
	$g_defaultOptionValues["showSimpleSearchOptions"] = false;
	$g_defaultOptionValues["ShowThumbnail"] = false;
	$g_defaultOptionValues["ShowTime"] = false;
	$g_defaultOptionValues["showViewInPopup"] = false;
	$g_defaultOptionValues["showCollapsed"] = false;
	$g_defaultOptionValues["showWithNoRecords"] = false;
	$g_defaultOptionValues["SimpleAdd"] = false;
	$g_defaultOptionValues["ShowListOfThumbnails"] = false;
	$g_defaultOptionValues["sortValueType"] = SORT_BY_DISP_VALUE;
	$g_defaultOptionValues["sqlFrom"] = "";
	$g_defaultOptionValues["sqlHead"] = "";
	$g_defaultOptionValues["sqlTail"] = "";
	$g_defaultOptionValues["sqlWhereExpr"] = "";
	$g_defaultOptionValues["sqlquery"] = null;
	$g_defaultOptionValues["strField"] = "";
	$g_defaultOptionValues["strFilename"] = "";
	$g_defaultOptionValues["strName"] = "";
	$g_defaultOptionValues["strOrderBy"] = "";
	$g_defaultOptionValues["StrThumbnail"] = "";
	$g_defaultOptionValues["strSortControlSettingsJSON"] = "";
	$g_defaultOptionValues["strClickActionJSON"] = "";
	$g_defaultOptionValues["ThumbnailSize"] = 0;
	$g_defaultOptionValues["scrollGridBody"] = false;
//	T
	$g_defaultOptionValues["totalsFields"] = array();
	$g_defaultOptionValues["ThumbHeight"] = 72;
	$g_defaultOptionValues["ThumbWidth"] = 72;
	$g_defaultOptionValues["truncateText"] = false;
//	U
	$g_defaultOptionValues["UploadFolder"] = "";
	$g_defaultOptionValues["UploadCodeExpression"] = false;
	$g_defaultOptionValues["UseCategory"] = false;
	$g_defaultOptionValues["UseRTE"] = false;
	$g_defaultOptionValues["UseTimestamp"] = false;
//	V
	$g_defaultOptionValues["validateAs"] = array();
	$g_defaultOptionValues["videoHeight"] = 200;
	$g_defaultOptionValues["videoWidth"] = 300;
	$g_defaultOptionValues["videoTitleField"] = "";
	$g_defaultOptionValues["view"] = false;
	$g_defaultOptionValues["ViewFormat"] = "";
//	W
	$g_defaultOptionValues["warnLeavingPages"] = false;
	$g_defaultOptionValues["weekdays"] = "";
	$g_defaultOptionValues["weekdayMessage"] = array();
//	X
//	Y
//	Z
$g_defaultOptionValues["groupChart"] = false;
$g_defaultOptionValues["chartLabelInterval"] = 0;
$g_defaultOptionValues["chartSeries"] = array();



//	A
	$g_settingsType["Absolute"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["acceptFileTypes"] = SETTING_TYPE_EDIT;
	$g_settingsType["acceptFileTypesHtml"] = SETTING_TYPE_EDIT;
	$g_settingsType["AllowToAdd"] = SETTING_TYPE_EDIT;
	$g_settingsType["autoCompleteFields"] = SETTING_TYPE_EDIT;
	$g_settingsType["autoCompleteFieldsOnEdit"] = SETTING_TYPE_EDIT;
	$g_settingsType["AutoInc"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["audioTitleField"] = SETTING_TYPE_VIEW;
	$g_settingsType["autoUpload"] = SETTING_TYPE_EDIT;
	$g_settingsType["autoUpdatable"] = SETTING_TYPE_EDIT;
//	B
	$g_settingsType["bInlineAdd"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["bInlineEdit"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["bUpdateSelected"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["bIsEncrypted"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["bListPage"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["bPrinterPage"] = SETTING_TYPE_GLOBAL;
//	C
	$g_settingsType["CompatibilityMode"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["categoryFields"] = SETTING_TYPE_EDIT;
	$g_settingsType["controlWidth"] = SETTING_TYPE_EDIT;
	$g_settingsType["CreateThumbnail"] = SETTING_TYPE_EDIT;
	$g_settingsType["CustomDisplay"] = SETTING_TYPE_EDIT;
//	D
	$g_settingsType["DateEditType"] = SETTING_TYPE_EDIT;
	$g_settingsType["DecimalDigits"] = SETTING_TYPE_VIEW;
	$g_settingsType["DeleteAssociatedFile"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["denyDuplicates"] = SETTING_TYPE_EDIT;
	$g_settingsType["DependentLookups"] = SETTING_TYPE_EDIT;
	$g_settingsType["DisplayField"] = SETTING_TYPE_EDIT;
//	E
	$g_settingsType["EditFormat"] = SETTING_TYPE_EDIT;
	$g_settingsType["EditParams"] = SETTING_TYPE_EDIT;
//	F
	$g_settingsType["fieldIsVideoUrl"] = SETTING_TYPE_VIEW;
	$g_settingsType["fieldIsImageUrl"] = SETTING_TYPE_VIEW;
	$g_settingsType["FieldType"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["FieldPermissions"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["Filename"] = SETTING_TYPE_VIEW;
	$g_settingsType["FormatTimeAttrs"] = SETTING_TYPE_EDIT;
	$g_settingsType["freeInput"] = SETTING_TYPE_EDIT;
	$g_settingsType["FullName"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["fieldEvents"] = SETTING_TYPE_EDIT;
	$g_settingsType["fieldViewEvents"] = SETTING_TYPE_VIEW;
//	G
	$g_settingsType["GoodName"] = SETTING_TYPE_GLOBAL;
//	H
	$g_settingsType["HorizontalLookup"] = SETTING_TYPE_EDIT;
	$g_settingsType["hlNewWindow"] = SETTING_TYPE_VIEW;
	$g_settingsType["hlType"] = SETTING_TYPE_VIEW;
	$g_settingsType["hlLinkWordNameType"] = SETTING_TYPE_VIEW;
	$g_settingsType["hlLinkWordText"] = SETTING_TYPE_VIEW;
	$g_settingsType["hlTitleField"] = SETTING_TYPE_VIEW;
	$g_settingsType["HTML5InuptType"] = SETTING_TYPE_EDIT;
//	I
	$g_settingsType["Index"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["InitialYearFactor"] = SETTING_TYPE_EDIT;
	$g_settingsType["ImageHeight"] = SETTING_TYPE_VIEW;
	$g_settingsType["ImageWidth"] = SETTING_TYPE_VIEW;
	$g_settingsType["insertNull"] = SETTING_TYPE_EDIT;
	$g_settingsType["IsRequired"] = SETTING_TYPE_EDIT;
	$g_settingsType["isSeparate"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["UpdateLatLng"] = SETTING_TYPE_EDIT;
//	J
//	K
//	L
	$g_settingsType["Label"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["LastYearFactor"] = SETTING_TYPE_EDIT;
	$g_settingsType["LCType"] = SETTING_TYPE_EDIT;
	$g_settingsType["listPageId"] = SETTING_TYPE_EDIT;
	$g_settingsType["addPageId"] = SETTING_TYPE_EDIT;
	$g_settingsType["LinkField"] = SETTING_TYPE_EDIT;
	$g_settingsType["LinkFieldType"] = SETTING_TYPE_EDIT;
	$g_settingsType["LinkPrefix"] = SETTING_TYPE_VIEW;
	$g_settingsType["LookupConnId"] = SETTING_TYPE_EDIT;
	$g_settingsType["LookupDesc"] = SETTING_TYPE_EDIT;
	$g_settingsType["LookupOrderBy"] = SETTING_TYPE_EDIT;
	$g_settingsType["LookupTable"] = SETTING_TYPE_EDIT;
	$g_settingsType["LookupType"] = SETTING_TYPE_EDIT;
	$g_settingsType["LookupUnique"] = SETTING_TYPE_EDIT;
	$g_settingsType["LookupValues"] = SETTING_TYPE_EDIT;
	$g_settingsType["LookupWhere"] = SETTING_TYPE_EDIT;
	$g_settingsType["LookupWhereCode"] = SETTING_TYPE_EDIT;
//	M
	$g_settingsType["mapData"] = SETTING_TYPE_VIEW;
	$g_settingsType["maxFileSize"] = SETTING_TYPE_EDIT;
	$g_settingsType["maxImageHeight"] = SETTING_TYPE_EDIT;
	$g_settingsType["maxImageWidth"] = SETTING_TYPE_EDIT;
	$g_settingsType["maxNumberOfFiles"] = SETTING_TYPE_EDIT;
	$g_settingsType["maxTotalFilesSize"] = SETTING_TYPE_EDIT;
	$g_settingsType["Multiselect"] = SETTING_TYPE_EDIT;
//	N
	$g_settingsType["nCols"] = SETTING_TYPE_EDIT;
	$g_settingsType["NeedEncode"] = SETTING_TYPE_VIEW;
	$g_settingsType["NewSize"] = SETTING_TYPE_EDIT;
	$g_settingsType["nRows"] = SETTING_TYPE_EDIT;
	$g_settingsType["nSecOptions"] = ADVSECURITY_NONE;
	$g_settingsType["NumberOfChars"] = SETTING_TYPE_VIEW;
//	O
	$g_settingsType["ownerTable"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["OraSequenceName"] = SETTING_TYPE_GLOBAL;
//	P
//	Q
//	R
	$g_settingsType["ResizeImage"] = SETTING_TYPE_EDIT;
	$g_settingsType["RewindEnabled"] = SETTING_TYPE_VIEW;
//	S
	$g_settingsType["SelectSize"] = SETTING_TYPE_EDIT;
	$g_settingsType["ShowCustomExpr"] = SETTING_TYPE_VIEW;
	$g_settingsType["ShowFileSize"] = SETTING_TYPE_VIEW;
	$g_settingsType["ShowIcon"] = SETTING_TYPE_VIEW;
	$g_settingsType["ShowThumbnail"] = SETTING_TYPE_VIEW;
	$g_settingsType["ShowTime"] = SETTING_TYPE_EDIT;
	$g_settingsType["SimpleAdd"] = SETTING_TYPE_EDIT;
	$g_settingsType["ShowListOfThumbnails"] = SETTING_TYPE_VIEW;
	$g_settingsType["strField"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["strFilename"] = SETTING_TYPE_VIEW;
	$g_settingsType["filterTotalFields"] = '';
	$g_settingsType["strEditMask"] = SETTING_TYPE_EDIT;
	$g_settingsType["strName"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["StrThumbnail"] = SETTING_TYPE_EDIT;
//	T
	$g_settingsType["ThumbnailSize"] = SETTING_TYPE_EDIT;
	$g_settingsType["ThumbHeight"] = SETTING_TYPE_VIEW;
	$g_settingsType["ThumbWidth"] = SETTING_TYPE_VIEW;
	$g_settingsType["truncateText"] = SETTING_TYPE_VIEW;
	$g_settingsType["timeFormatData"] = SETTING_TYPE_VIEW;
//	U
	$g_settingsType["UploadFolder"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["UploadCodeExpression"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["UseCategory"] = SETTING_TYPE_EDIT;
	$g_settingsType["UseRTE"] = SETTING_TYPE_EDIT;
	$g_settingsType["UseTimestamp"] = SETTING_TYPE_EDIT;
//	V
	$g_settingsType["validateAs"] = SETTING_TYPE_EDIT;
	$g_settingsType["videoHeight"] = SETTING_TYPE_VIEW;
	$g_settingsType["videoWidth"] = SETTING_TYPE_VIEW;
	$g_settingsType["videoTitleField"] = SETTING_TYPE_VIEW;
	$g_settingsType["ViewFormat"] = SETTING_TYPE_VIEW;

	$g_settingsType["multipleImgMode"] = SETTING_TYPE_VIEW;
	$g_settingsType["maxImages"] = SETTING_TYPE_VIEW;
	$g_settingsType["showGallery"] = SETTING_TYPE_VIEW;
	$g_settingsType["galleryMode"] = SETTING_TYPE_VIEW;
	$g_settingsType["captionMode"] = SETTING_TYPE_VIEW;
	$g_settingsType["captionField"] = SETTING_TYPE_VIEW;
	$g_settingsType["imageBorder"] = SETTING_TYPE_VIEW;
	$g_settingsType["imageFullWidth"] = SETTING_TYPE_VIEW;

	$g_settingsType["weekdays"] = SETTING_TYPE_EDIT;
	$g_settingsType["weekdayMessage"] = SETTING_TYPE_EDIT;

?>